#include <iostream>

namespace rvg { namespace color {

inline std::ostream &operator<<(std::ostream &out, const Color &c) {
    if (c.opacity != 1.f) {
        return out << "rgb(" << c.red << ',' << c.green 
            << ',' << c.blue << ')';
    } else  {
        return out << "rgba(" << c.red << ',' << c.green 
            << ',' << c.blue << ',' << c.opacity << ')';
    }
}

static inline Color rgb(float r, float g, float b) {
    return Color{r, g, b, 1.f};
}

static inline Color rgba(float r, float g, float b, float a) {
    return Color{r, g, b, a};
}

static inline Color hsv(float h, float s, float v, float o = 1.f) {
    if (s >= 0.f) { // chromatic case
        if (h >= 1.f) h = 0.f; // flip back to zero if exactly 1.f
        h *= 6.f;
        int p = int(h);
        float f = h-p;
        float m = v*(1.f-s);
        float n = v*(1.f-s*f);
        float k = v*(1.f-s*(1.f-f));
        switch (p) {
            case 0: return Color{v, k, m, o};
            case 1: return Color{n, v, m, o};
            case 2: return Color{m, v, k, o};
            case 3: return Color{m, n, v, o};
            case 4: return Color{k, m, v, o};
            case 5: return Color{v, m, n, o};
            default: return Color{0.f, 0.f, 0.f, o};
        }
    } else { // grayscale
        return Color{v, v, v, o};
    }
}

}} // namespace rvg::color
