#include <sstream>

#include "constants.h"

namespace rvg {

std::ostream &operator<<(std::ostream &out, SegmentType segment_type) {
    switch (segment_type) {
        case SegmentType::close_path: out << "close_path"; break;
        case SegmentType::move_to: out << "move_to"; break;
        case SegmentType::line_to: out << "line_to"; break;
        case SegmentType::hline_to: out << "hline_to"; break;
        case SegmentType::vline_to: out << "vline_to"; break;
        case SegmentType::quad_to: out << "quad_to"; break;
        case SegmentType::squad_to: out << "squad_to"; break;
        case SegmentType::rquad_to: out << "rquad_to"; break;
        case SegmentType::cubic_to: out << "cubic_to"; break;
        case SegmentType::scubic_to: out << "scubic_to"; break;
        case SegmentType::svgarc_to: out << "svgarc_to"; break;
        case SegmentType::fline_to: out << "fline_to"; break;
    }
    return out;
}


std::ostream &operator<<(std::ostream &out, SpreadMethod spread_method) {
    switch (spread_method) {
        case SpreadMethod::pad: out << "pad"; break;
        case SpreadMethod::repeat: out << "repeat"; break;
        case SpreadMethod::reflect: out << "reflect"; break;
        case SpreadMethod::transparent: out << "transparent"; break;
        default: out << "invalid spread!"; break;
    }
    return out;
}

std::ostream &operator<<(std::ostream &out, PaintType paint_type) {
    switch (paint_type) {
        case PaintType::solid: out << "solid"; break;
        case PaintType::linear_gradient: out << "linear gradient"; break;
        case PaintType::radial_gradient: out << "radial gradient"; break;
        case PaintType::texture: out << "texture"; break;
        default: out << "invalid paint!"; break;
    }
    return out;
}

std::ostream &operator<<(std::ostream &out, CapStyle cap_style) {
    switch (cap_style) {
        case CapStyle::butt: out << "butt"; break;
        case CapStyle::round: out << "round"; break;
        case CapStyle::square: out << "square"; break;
        case CapStyle::triangle: out << "triangle"; break;
        default: out << "invalid cap style!"; break;
    }
    return out;
}

std::ostream &operator<<(std::ostream &out, JoinStyle join_style) {
    switch (join_style) {
        case JoinStyle::miter: out << "miter"; break;
        case JoinStyle::round: out << "round"; break;
        case JoinStyle::bevel: out << "bevel"; break;
        default: out << "invalid join style!"; break;
    }
    return out;
}

} // namespace rvg
