#ifndef RVG_CONSTANTS_H
#define RVG_CONSTANTS_H

#include <iosfwd>

// include constants defined in C
#include "rvgc.h"

namespace rvg {

enum class MatrixMode {
  path_to_surface = RVG_MATRIX_PATH_TO_SURFACE,
  image_to_surface = RVG_MATRIX_IMAGE_TO_SURFACE,
  paint_to_user = RVG_MATRIX_PAINT_TO_USER,
  surface_to_surface = RVG_MATRIX_SURFACE_TO_SURFACE
};

enum class PrefilterKernel {
  dirac = RVG_DIRAC,
  box = RVG_BOX,
  catmull_rom = RVG_CATMULL_ROM,
  mitchell_netravali = RVG_MITCHELL_NETRAVALI,
  cardinal_cubic_spline = RVG_CARDINAL_CUBIC_BSPLINE
};

enum class SamplingPattern {
  balzer = RVG_BALZER,
  kensler = RVG_KENSLER,
  regular = RVG_REGULAR,
  stratified = RVG_STRATIFIED
};

enum class SamplingRate {
  point = RVG_SAMPLING_RATE_POINT,
  low = RVG_SAMPLING_RATE_LOW,
  medium = RVG_SAMPLING_RATE_MEDIUM,
  high = RVG_SAMPLING_RATE_HIGH,
  custom = RVG_SAMPLING_RATE_CUSTOM
};

enum class DeviceType {
  reference = RVG_DEVICE_REFERENCE,
  cpu = RVG_DEVICE_CPU,
  gpu = RVG_DEVICE_GPU,
  svg = RVG_DEVICE_SVG
};

enum class SegmentType {
  close_path = RVG_CLOSE_PATH,
  move_to = RVG_MOVE_TO,
  line_to = RVG_LINE_TO,
  hline_to = RVG_HLINE_TO,
  vline_to = RVG_VLINE_TO,
  quad_to = RVG_QUAD_TO,
  squad_to = RVG_SQUAD_TO,
  rquad_to = RVG_RQUAD_TO,
  cubic_to = RVG_CUBIC_TO,
  scubic_to = RVG_SCUBIC_TO,
  // internal segment types
  fline_to = RVG_FLINE_TO,
  // fill lines are lines that close a path and are ignored
  // by the stroker
  svgarc_to = RVG_SVGARC_TO
  // SVG arcs never actually make it into concrete paths.
  // they are converted to rational arcs. But we need the
  // constant so the few functions that deal with it can
  // distinguish them from other commands
};

std::ostream &operator<<(std::ostream &out, SegmentType segment_type);

enum class XFormType {
  identity = RVG_XFORM_IDENTITY,
  tx = RVG_XFORM_TX,
  ty = RVG_XFORM_TY,
  rotation = RVG_XFORM_ROTATION,
  translation = RVG_XFORM_TRANSLATION,
  scale = RVG_XFORM_SCALE,
  shear = RVG_XFORM_SHEAR,
  linear = RVG_XFORM_LINEAR,
  affine = RVG_XFORM_AFFINE,
  projective = RVG_XFORM_PROJECTIVE
};

enum class CommandType {
  close_path_abs = RVG_CLOSE_PATH_ABS,
  close_path_rel = RVG_CLOSE_PATH_REL,
  close_path = RVG_CLOSE_PATH_ABS,
  move_to_abs = RVG_MOVE_TO_ABS,
  move_to_rel = RVG_MOVE_TO_REL,
  line_to_abs = RVG_LINE_TO_ABS,
  line_to_rel = RVG_LINE_TO_REL,
  hline_to_abs = RVG_HLINE_TO_ABS,
  hline_to_rel = RVG_HLINE_TO_REL,
  vline_to_abs = RVG_VLINE_TO_ABS,
  vline_to_rel = RVG_VLINE_TO_REL,
  quad_to_abs = RVG_QUAD_TO_ABS,
  quad_to_rel = RVG_QUAD_TO_REL,
  squad_to_abs = RVG_SQUAD_TO_ABS,
  squad_to_rel = RVG_SQUAD_TO_REL,
  rquad_to_abs = RVG_RQUAD_TO_ABS,
  rquad_to_rel = RVG_RQUAD_TO_REL,
  cubic_to_abs = RVG_CUBIC_TO_ABS,
  cubic_to_rel = RVG_CUBIC_TO_REL,
  scubic_to_abs = RVG_SCUBIC_TO_ABS,
  scubic_to_rel = RVG_SCUBIC_TO_REL,
  svgarc_to_abs = RVG_SVGARC_TO_ABS,
  svgarc_to_rel = RVG_SVGARC_TO_REL
};

enum class CapStyle {
  butt = RVG_CAP_BUTT,
  round = RVG_CAP_ROUND,
  square = RVG_CAP_SQUARE,
  triangle = RVG_CAP_TRIANGLE
};

std::ostream &operator<<(std::ostream &out, CapStyle cap_style);

enum class JoinStyle {
  miter = RVG_JOIN_MITER,
  round = RVG_JOIN_ROUND,
  bevel = RVG_JOIN_BEVEL
};

std::ostream &operator<<(std::ostream &out, JoinStyle join_style);

enum class FillRule {
  even_odd = RVG_EVEN_ODD,
  non_zero = RVG_NON_ZERO
};

std::ostream &operator<<(std::ostream &out, FillRule fill_rule);

enum class PaintType {
  solid = RVG_PAINT_TYPE_SOLID,
  linear_gradient = RVG_PAINT_TYPE_LINEAR_GRADIENT,
  radial_gradient = RVG_PAINT_TYPE_RADIAL_GRADIENT,
  texture = RVG_PAINT_TYPE_TEXTURE
};

std::ostream &operator<<(std::ostream &out, PaintType paint_type);

enum class SpreadMethod {
  pad = RVG_SPREAD_PAD,
  repeat = RVG_SPREAD_REPEAT,
  reflect = RVG_SPREAD_REFLECT,
  transparent = RVG_SPREAD_TRANSPARENT
};

std::ostream &operator<<(std::ostream &out, SpreadMethod spread_mode);

enum class BlendMode {
  src = RVG_BLEND_SRC,
  src_over = RVG_BLEND_SRC_OVER,
  dst_over = RVG_BLEND_DST_OVER,
  src_in = RVG_BLEND_SRC_IN,
  dst_in = RVG_BLEND_DST_IN,
  multiply = RVG_BLEND_MULTIPLY,
  screen = RVG_BLEND_SCREEN,
  darken = RVG_BLEND_DARKEN,
  lighten = RVG_BLEND_LIGHTEN,
  additive = RVG_BLEND_ADDITIVE
};

std::ostream &operator<<(std::ostream &out, BlendMode blend_mode);

} // namespace rvg

#endif // RVG_CONSTANTS_H
