local drv = require"rvg10.cudagl"
local fd = dofile(arg[1])
local vp_ = fd.viewport(drv)
local xf = drv.identity();

-- adjust initial size
local width = 1024
local dim = { w=(vp_.max.x - vp_.min.x), h=(vp_.max.y - vp_.min.y) }
local vp = drv.viewport(0, 0, width, width*dim.h/dim.w);
local mp = { x=width/2, y=width*dim.h/(dim.w*2)}
local nscene = fd.scene(drv):translate(-vp.min.x,-vp.min.y):scale(width/dim.w, width/dim.w)
local height = width*dim.h/dim.w

drv.init( vp )
local tree = drv.shortcut( nscene, vp )
print (tree)

function show_frame( sxf )
    print( "->frame" )
    drv.show(tree, sxf, vp)
    print( "" )
end

function display_cb()
    show_frame( xf )
end

function keyboard_cb( k )
    print( "Key:", k )
    show_frame( xf )
end


local xftype = 1 -- tranlation
local lpt = {x=0,y=0}
local ldi = {x=0,y=0}

function find_rotation( x, y )
    local dx = x-mp.x
    local dy = y-mp.y
    local n = math.sqrt(dx*dx + dy*dy)
    local v = { x=dx/n, y=dy/n }
    local c = ldi.x * v.x + ldi.y*v.y
    local s = -ldi.y * v.x + ldi.x*v.y
    local t0 = drv.translation(-mp.x, -mp.y)
    local r = drv.rotation(c,s)
    local t1 = drv.translation(mp.x, mp.y)
    return t1 * r * t0
end

function find_scale(x, y, s)
    local t0 = drv.translation(-x, -y)
    local r = drv.scale(s,s)
    local t1 = drv.translation(x, y)
    return t1 * r * t0
end

function mouse_cb( b, s, x, y )
    y = height - y
    print( "mouse:", b, s, x, y )
    xftype = b
    if b==1 then -- middle button
        if s==0 then --down
            lpt = { x=x, y=y }
        else
            local d = { x = x - lpt.x, y = y - lpt.y }
            xf = drv.translation(d.x, d.y) * xf
        end
    elseif b==0 then
        if s==0 then
            local dx = x-mp.x
            local dy = y-mp.y
            local n = math.sqrt(dx*dx + dy*dy)
            ldi = { x=dx/n, y=dy/n }
        else
            local x = find_rotation(x,y)
            xf = x * xf
        end
    elseif b==3 then --wheel up
        if s==0 then
            xf = find_scale(x,y,1.125) * xf
        end
    elseif b==4 then --wheel down
        if s==0 then
            xf = find_scale(x,y,.8) * xf
        end
    end
    show_frame( xf )
end

function mouse_move_cb( x, y )
    y = height - y
    --print( "mouse:", x, y )
    local xx = drv.identity()
    if xftype == 1 then
        local d = { x = x - lpt.x, y = y - lpt.y }
        xx = drv.translation(d.x, d.y) * xf
    elseif xftype == 0 then
        xx = find_rotation(x,y) * xf
    end
    show_frame( xx )
end
drv.display_cb(display_cb)
drv.keyboard_cb(keyboard_cb)
drv.mouse_cb(mouse_cb)
drv.mouse_move_cb(mouse_move_cb)
local s = fd.scene(drv)
local id = drv.identity()
drv.render()
