#include <cuda_runtime.h>
#include <stdexcept>
#include "alloc.h"
#include "error.h"
#include "timer.h"

namespace gpu
{

unsigned TOTAL_MEMORY_USED = 0;
unsigned MAX_MEM_USED = 0;

void update_memused(int mem)
{
    TOTAL_MEMORY_USED += mem;
    MAX_MEM_USED = std::max(MAX_MEM_USED,TOTAL_MEMORY_USED);
}
unsigned mem_used()
{
    return TOTAL_MEMORY_USED;
}
unsigned max_mem_used()
{
    return MAX_MEM_USED;
}


void *cuda_new(const void *host, size_t elem_size)
{
    void *dev = cuda_new_array(1, elem_size);

    cudaMemcpy(dev, host, elem_size, cudaMemcpyHostToDevice);
    check_cuda_error("Error during memcpy from host to device");

    return dev;
}

void *cuda_new_array(size_t elements, size_t elem_size)
{
    void *ptr = NULL;

    cudaMalloc(&ptr, elements*elem_size);
    check_cuda_error("Memory allocation error");
    if(ptr == NULL)
        throw std::runtime_error("Memory allocation error");

    return ptr;
}

void cuda_delete(void *ptr)
{
    cudaFree(ptr);
    check_cuda_error("Error freeing memory");
}

} // namespace gpu
