#ifndef GPU_ALLOC_H
#define GPU_ALLOC_H

#include "error.h"

namespace gpu
{

void *cuda_new_array(size_t elements, size_t elem_size);
void *cuda_new(const void *host, size_t elem_size);

template <class T>
T *cuda_new_array(size_t elements)
{
    return reinterpret_cast<T *>(cuda_new_array(elements, sizeof(T)));
}

#if __cplusplus >= 201103L
template <class T, class... ARGS>
T *cuda_new(ARGS &&... args)
{
    T h(std::forward<ARGS>(args)...);

    return reinterpret_cast<T *>(cuda_new(&h, sizeof(T)));
}
#endif

void cuda_delete(void *ptr);

struct cuda_deleter
{
    template <class T>
    void operator()(T *ptr) const
    {
        cuda_delete(ptr);
    }
};

template <class T>
class cuda_allocator : public std::allocator<T>
{
public:
    typedef typename std::allocator<T>::pointer pointer;
    typedef typename std::allocator<T>::size_type size_type;

    pointer allocate(size_type n, std::allocator<void>::const_pointer hint=0)
    {
        return cuda_new_array<T>(n);
    }

    void deallocate(pointer ptr, size_type n)
    {
        cuda_delete(ptr);
    }

    void construct(pointer ptr, const T &val)
    {
        // do nothing
    }
    void destroy(pointer ptr)
    {
        // do nothing
    }
};

void update_memused(int mem);
unsigned mem_used();
unsigned max_mem_used();

} // namespace gpu

#endif
