#ifndef GPU_CUDPP_H
#define GPU_CUDPP_H

#include "op.h"

namespace gpu
{

class Plan;

template<class T> class vector;

class Cudpp
{
public:
    Cudpp();
    ~Cudpp();

    // pointer interface
    // unsigned
    void inclusive_scan(const unsigned *in, size_t size, unsigned *out,
                        Operation op = OP_ADD, bool reverse=false);
    void exclusive_scan(const unsigned *in, size_t size, unsigned *out,
                        Operation op = OP_ADD, bool reverse=false);

    void segmented_inclusive_scan(const unsigned *in, size_t size,
                                  const unsigned *flags, unsigned *out,
                                  Operation op = OP_ADD, bool reverse=false);

    void segmented_exclusive_scan(const unsigned *in, size_t size,
                                  const unsigned *flags, unsigned *out,
                                  Operation op = OP_ADD, bool reverse=false);

    void reverse_inclusive_scan(const unsigned *in, size_t size, unsigned *out,
                        Operation op = OP_ADD)
    {
        inclusive_scan(in, size, out, op, true);
    }
    void reverse_exclusive_scan(const unsigned *in, size_t size, unsigned *out,
                        Operation op = OP_ADD)
    {
        exclusive_scan(in, size, out, op, true);
    }

    void reverse_segmented_inclusive_scan(const unsigned *in, size_t size,
                                  const unsigned *flags, unsigned *out,
                                  Operation op = OP_ADD)
    {
        segmented_inclusive_scan(in, size, flags, out, op, true);
    }

    void reverse_segmented_exclusive_scan(const unsigned *in, size_t size,
                                  const unsigned *flags, unsigned *out,
                                  Operation op = OP_ADD)
    {
        segmented_exclusive_scan(in, size, flags, out, op, true);
    }

    void radix_sort(unsigned *keys, size_t size, unsigned *values = NULL);

    // int

    void inclusive_scan(const int *in, size_t size, int *out,
                        Operation op = OP_ADD, bool reverse=false);
    void exclusive_scan(const int *in, size_t size, int *out,
                        Operation op = OP_ADD, bool reverse=false);

    void segmented_inclusive_scan(const int *in, size_t size,
                                  const unsigned *flags, int *out,
                                  Operation op = OP_ADD, bool reverse=false);

    void segmented_exclusive_scan(const int *in, size_t size,
                                  const unsigned *flags, int *out,
                                  Operation op = OP_ADD, bool reverse=false);

    void reverse_inclusive_scan(const int *in, size_t size, int *out,
                        Operation op = OP_ADD)
    {
        inclusive_scan(in, size, out, op, true);
    }
    void reverse_exclusive_scan(const int *in, size_t size, int *out,
                        Operation op = OP_ADD)
    {
        exclusive_scan(in, size, out, op, true);
    }

    void reverse_segmented_inclusive_scan(const int *in, size_t size,
                                  const unsigned *flags, int *out,
                                  Operation op = OP_ADD)
    {
        segmented_inclusive_scan(in, size, flags, out, op, true);
    }

    void reverse_segmented_exclusive_scan(const int *in, size_t size,
                                  const unsigned *flags, int *out,
                                  Operation op = OP_ADD)
    {
        segmented_exclusive_scan(in, size, flags, out, op, true);
    }

    void radix_sort(int *keys, size_t size, int *values = NULL);

    // vector interface
    // unsigned
    void inclusive_scan(const vector<unsigned> &in, vector<unsigned> &out,
                                  Operation op = OP_ADD);
    void exclusive_scan(const vector<unsigned> &in, vector<unsigned> &out,
                                  Operation op = OP_ADD);

    void segmented_inclusive_scan(const vector<unsigned> &in,
                             const vector<unsigned> &flags,
                             vector<unsigned> &out,
                             Operation op = OP_ADD);

    void segmented_exclusive_scan(const vector<unsigned> &in,
                             const vector<unsigned> &flags,
                             vector<unsigned> &out,
                             Operation op = OP_ADD);

    void reverse_inclusive_scan(const vector<unsigned> &in, vector<unsigned> &out,
                                  Operation op = OP_ADD);
    void reverse_exclusive_scan(const vector<unsigned> &in, vector<unsigned> &out,
                                  Operation op = OP_ADD);

    void reverse_segmented_inclusive_scan(const vector<unsigned> &in,
                             const vector<unsigned> &flags,
                             vector<unsigned> &out,
                             Operation op = OP_ADD);

    void reverse_segmented_exclusive_scan(const vector<unsigned> &in,
                             const vector<unsigned> &flags,
                             vector<unsigned> &out,
                             Operation op = OP_ADD);

    void radix_sort(vector<unsigned> &keys);
    void radix_sort(vector<unsigned> &keys, vector<unsigned> &values);

    // int
    void inclusive_scan(const vector<int> &in, vector<int> &out,
                                  Operation op = OP_ADD);
    void exclusive_scan(const vector<int> &in, vector<int> &out,
                                  Operation op = OP_ADD);

    void segmented_inclusive_scan(const vector<int> &in,
                             const vector<unsigned> &flags,
                             vector<int> &out,
                             Operation op = OP_ADD);

    void segmented_exclusive_scan(const vector<int> &in,
                             const vector<unsigned> &flags,
                             vector<int> &out,
                             Operation op = OP_ADD);

    void reverse_inclusive_scan(const vector<int> &in, vector<int> &out,
                                  Operation op = OP_ADD);
    void reverse_exclusive_scan(const vector<int> &in, vector<int> &out,
                                  Operation op = OP_ADD);

    void reverse_segmented_inclusive_scan(const vector<int> &in,
                             const vector<unsigned> &flags,
                             vector<int> &out,
                             Operation op = OP_ADD);

    void reverse_segmented_exclusive_scan(const vector<int> &in,
                             const vector<unsigned> &flags,
                             vector<int> &out,
                             Operation op = OP_ADD);

    void radix_sort(vector<int> &keys);
    void radix_sort(vector<int> &keys, vector<int> &values);

private:
    Plan *m_plan;
};

}

#endif
