#ifndef GPU_MEMORY_H
#define GPU_MEMORY_H

namespace gpu
{

// ld_global_cg/*{{{*/

__device__ __inline__
short ld_global_cg(const short *mem)

{
    short r;
    asm("ld.global.cg.s16 %0,[%1];" : "=h"(r) : "l"(mem));
    return r;
}

__device__ __inline__
short2 ld_global_cg(const short2 *mem)

{
    short2 r;
    asm("ld.global.cg.v2.s16 {%0,%1},[%2];" : "=h"(r.x), "=h"(r.y) : "l"(mem));
    return r;
}

__device__ __inline__
short4 ld_global_cg(const short4 *mem)

{
    short4 r;
    asm("ld.global.cg.v4.s16 {%0,%1,%2,%3},[%4];"
        : "=h"(r.x), "=h"(r.y), "=h"(r.z), "=h"(r.w) : "l"(mem));
    return r;
}

__device__ __inline__
unsigned short ld_global_cg(const unsigned short *mem)

{
    unsigned short r;
    asm("ld.global.cg.u16 %0,[%1];" : "=h"(r) : "l"(mem));
    return r;
}

__device__ __inline__
ushort2 ld_global_cg(const ushort2 *mem)

{
    ushort2 r;
    asm("ld.global.cg.v2.u16 {%0,%1},[%2];" : "=h"(r.x), "=h"(r.y) : "l"(mem));
    return r;
}

__device__ __inline__
ushort4 ld_global_cg(const ushort4 *mem)

{
    ushort4 r;
    asm("ld.global.cg.v4.u16 {%0,%1,%2,%3},[%4];"
        : "=h"(r.x), "=h"(r.y), "=h"(r.z), "=h"(r.w) : "l"(mem));
    return r;
}

__device__ __inline__
int ld_global_cg(const int *mem)

{
    int r;
    asm("ld.global.cg.s32 %0,[%1];" : "=r"(r) : "l"(mem));
    return r;
}

__device__ __inline__
int2 ld_global_cg(const int2 *mem)

{
    int2 r;
    asm("ld.global.cg.v2.s32 {%0,%1},[%2];" : "=r"(r.x), "=r"(r.y) : "l"(mem));
    return r;
}

__device__ __inline__
int4 ld_global_cg(const int4 *mem)

{
    int4 r;
    asm("ld.global.cg.v4.s32 {%0,%1,%2,%3},[%4];"
        : "=r"(r.x), "=r"(r.y), "=r"(r.z), "=r"(r.w) : "l"(mem));
    return r;
}

__device__ __inline__
unsigned ld_global_cg(const unsigned *mem)

{
    unsigned r;
    asm("ld.global.cg.u32 %0,[%1];" : "=r"(r) : "l"(mem));
    return r;
}

__device__ __inline__
uint2 ld_global_cg(const uint2 *mem)

{
    uint2 r;
    asm("ld.global.cg.v2.u32 {%0,%1},[%2];" : "=r"(r.x), "=r"(r.y) : "l"(mem));
    return r;
}

__device__ __inline__
uint4 ld_global_cg(const uint4 *mem)

{
    uint4 r;
    asm("ld.global.cg.v4.u32 {%0,%1,%2,%3},[%4];"
        : "=r"(r.x), "=r"(r.y), "=r"(r.z), "=r"(r.w) : "l"(mem));
    return r;
}

__device__ __inline__
float ld_global_cg(const float *mem)

{
    float r;
    asm("ld.global.cg.f32 %0,[%1];" : "=f"(r) : "l"(mem));
    return r;
}

__device__ __inline__
float2 ld_global_cg(const float2 *mem)

{
    float2 r;
    asm("ld.global.cg.v2.f32 {%0,%1},[%2];" : "=f"(r.x), "=f"(r.y) : "l"(mem));
    return r;
}

__device__ __inline__
float4 ld_global_cg(const float4 *mem)

{
    float4 r;
    asm("ld.global.cg.v4.f32 {%0,%1,%2,%3},[%4];"
        : "=f"(r.x), "=f"(r.y), "=f"(r.z), "=f"(r.w) : "l"(mem));
    return r;
}

template <class T>
__device__ __inline__
T *ld_global_cg(const T *const *mem)

{
    T *r;
    asm("ld.global.cg.u64 %0,[%1];" : "=l"(r) : "l"(mem));
    return r;
}
/*}}}*/

// st_global_cg/*{{{*/

__device__ __inline__
void st_global_cg(short *mem, short v)
{
    asm("st.global.cg.s16 [%0],%1;" :: "l"(mem), "h"(v));
}

__device__ __inline__
void st_global_cg(short2 *mem, short2 v)
{
    asm("st.global.cg.v2.s16 [%0],{%1,%2};"
        :: "l"(mem), "h"(v.x), "h"(v.y));
}

__device__ __inline__
void st_global_cg(short4 *mem, short4 v)
{
    asm("st.global.cg.v4.s16 [%0],{%1,%2,%3,%4};"
        :: "l"(mem), "h"(v.x), "h"(v.y), "h"(v.z), "h"(v.w));
}

__device__ __inline__
void st_global_cg(unsigned short *mem, unsigned short v)
{
    asm("st.global.cg.u16 [%0],%1;" :: "l"(mem), "h"(v));
}

__device__ __inline__
void st_global_cg(ushort2 *mem, ushort2 v)
{
    asm("st.global.cg.v2.u16 [%0],{%1,%2};"
        :: "l"(mem), "h"(v.x), "h"(v.y));
}

__device__ __inline__
void st_global_cg(ushort4 *mem, ushort4 v)
{
    asm("st.global.cg.v4.u16 [%0],{%1,%2,%3,%4};"
        :: "l"(mem), "h"(v.x), "h"(v.y), "h"(v.z), "h"(v.w));
}

__device__ __inline__
void st_global_cg(int *mem, int v)
{
    asm("st.global.cg.s32 [%0],%1;" :: "l"(mem), "r"(v));
}

__device__ __inline__
void st_global_cg(int2 *mem, int2 v)
{
    asm("st.global.cg.v2.s32 [%0],{%1,%2};"
        :: "l"(mem), "r"(v.x), "r"(v.y));
}

__device__ __inline__
void st_global_cg(int4 *mem, int4 v)
{
    asm("st.global.cg.v4.s32 [%0],{%1,%2,%3,%4};"
        :: "l"(mem), "r"(v.x), "r"(v.y), "r"(v.z), "r"(v.w));
}

__device__ __inline__
void st_global_cg(unsigned int *mem, unsigned int v)
{
    asm("st.global.cg.u32 [%0],%1;" :: "l"(mem), "r"(v));
}

__device__ __inline__
void st_global_cg(uint2 *mem, uint2 v)
{
    asm("st.global.cg.v2.u32 [%0],{%1,%2};"
        :: "l"(mem), "r"(v.x), "r"(v.y));
}

__device__ __inline__
void st_global_cg(uint4 *mem, uint4 v)
{
    asm("st.global.cg.v4.u32 [%0],{%1,%2,%3,%4};"
        :: "l"(mem), "r"(v.x), "r"(v.y), "r"(v.z), "r"(v.w));
}

__device__ __inline__
void st_global_cg(float *mem, float v)

{
    asm("st.global.cg.f32 [%0],%1;" :: "l"(mem), "f"(v));
}

__device__ __inline__
void st_global_cg(float2 *mem, float2 v)

{
    asm("st.global.cg.v2.f32 [%0],{%1,%2};"
        :: "l"(mem), "f"(v.x), "f"(v.y));
}

__device__ __inline__
void st_global_cg(float4 *mem, float4 v)

{
    asm("st.global.cg.v4.f32 [%0],{%1,%2,%3,%4};"
        :: "l"(mem), "f"(v.x), "f"(v.y), "f"(v.z), "f"(v.w));
}

template <class T>
__device__ __inline__
void st_global_cg(T *const *mem, const T *v)

{
    asm("st.global.cg.u64 [%0],%1;" :: "l"(mem), "l"(v));
}
/*}}}*/

// ld_global_cs/*{{{*/

__device__ __inline__
short ld_global_cs(const short *mem)

{
    short r;
    asm("ld.global.cs.s16 %0,[%1];" : "=h"(r) : "l"(mem));
    return r;
}

__device__ __inline__
short2 ld_global_cs(const short2 *mem)

{
    short2 r;
    asm("ld.global.cs.v2.s16 {%0,%1},[%2];" : "=h"(r.x), "=h"(r.y) : "l"(mem));
    return r;
}

__device__ __inline__
short4 ld_global_cs(const short4 *mem)

{
    short4 r;
    asm("ld.global.cs.v4.s16 {%0,%1,%2,%3},[%4];"
        : "=h"(r.x), "=h"(r.y), "=h"(r.z), "=h"(r.w) : "l"(mem));
    return r;
}

__device__ __inline__
unsigned short ld_global_cs(const unsigned short *mem)

{
    unsigned short r;
    asm("ld.global.cs.u16 %0,[%1];" : "=h"(r) : "l"(mem));
    return r;
}

__device__ __inline__
ushort2 ld_global_cs(const ushort2 *mem)

{
    ushort2 r;
    asm("ld.global.cs.v2.u16 {%0,%1},[%2];" : "=h"(r.x), "=h"(r.y) : "l"(mem));
    return r;
}

__device__ __inline__
ushort4 ld_global_cs(const ushort4 *mem)

{
    ushort4 r;
    asm("ld.global.cs.v4.u16 {%0,%1,%2,%3},[%4];"
        : "=h"(r.x), "=h"(r.y), "=h"(r.z), "=h"(r.w) : "l"(mem));
    return r;
}

__device__ __inline__
int ld_global_cs(const int *mem)

{
    int r;
    asm("ld.global.cs.s32 %0,[%1];" : "=r"(r) : "l"(mem));
    return r;
}

__device__ __inline__
int2 ld_global_cs(const int2 *mem)

{
    int2 r;
    asm("ld.global.cs.v2.s32 {%0,%1},[%2];" : "=r"(r.x), "=r"(r.y) : "l"(mem));
    return r;
}

__device__ __inline__
int4 ld_global_cs(const int4 *mem)

{
    int4 r;
    asm("ld.global.cs.v4.s32 {%0,%1,%2,%3},[%4];"
        : "=r"(r.x), "=r"(r.y), "=r"(r.z), "=r"(r.w) : "l"(mem));
    return r;
}

__device__ __inline__
unsigned ld_global_cs(const unsigned *mem)

{
    unsigned r;
    asm("ld.global.cs.u32 %0,[%1];" : "=r"(r) : "l"(mem));
    return r;
}

__device__ __inline__
uint2 ld_global_cs(const uint2 *mem)

{
    uint2 r;
    asm("ld.global.cs.v2.u32 {%0,%1},[%2];" : "=r"(r.x), "=r"(r.y) : "l"(mem));
    return r;
}

__device__ __inline__
uint4 ld_global_cs(const uint4 *mem)

{
    uint4 r;
    asm("ld.global.cs.v4.u32 {%0,%1,%2,%3},[%4];"
        : "=r"(r.x), "=r"(r.y), "=r"(r.z), "=r"(r.w) : "l"(mem));
    return r;
}

__device__ __inline__
float ld_global_cs(const float *mem)

{
    float r;
    asm("ld.global.cs.f32 %0,[%1];" : "=f"(r) : "l"(mem));
    return r;
}

__device__ __inline__
float2 ld_global_cs(const float2 *mem)

{
    float2 r;
    asm("ld.global.cs.v2.f32 {%0,%1},[%2];" : "=f"(r.x), "=f"(r.y) : "l"(mem));
    return r;
}

__device__ __inline__
float4 ld_global_cs(const float4 *mem)

{
    float4 r;
    asm("ld.global.cs.v4.f32 {%0,%1,%2,%3},[%4];"
        : "=f"(r.x), "=f"(r.y), "=f"(r.z), "=f"(r.w) : "l"(mem));
    return r;
}

template <class T>
__device__ __inline__
T *ld_global_cs(const T *const *mem)

{
    T *r;
    asm("ld.global.cs.u64 %0,[%1];" : "=l"(r) : "l"(mem));
    return r;
}
/*}}}*/

// st_global_cs/*{{{*/

__device__ __inline__
void st_global_cs(short *mem, short v)
{
    asm("st.global.cs.s16 [%0],%1;" :: "l"(mem), "h"(v));
}

__device__ __inline__
void st_global_cs(short2 *mem, short2 v)
{
    asm("st.global.cs.v2.s16 [%0],{%1,%2};"
        :: "l"(mem), "h"(v.x), "h"(v.y));
}

__device__ __inline__
void st_global_cs(short4 *mem, short4 v)
{
    asm("st.global.cs.v4.s16 [%0],{%1,%2,%3,%4};"
        :: "l"(mem), "h"(v.x), "h"(v.y), "h"(v.z), "h"(v.w));
}

__device__ __inline__
void st_global_cs(unsigned short *mem, unsigned short v)
{
    asm("st.global.cs.u16 [%0],%1;" :: "l"(mem), "h"(v));
}

__device__ __inline__
void st_global_cs(ushort2 *mem, ushort2 v)
{
    asm("st.global.cs.v2.u16 [%0],{%1,%2};"
        :: "l"(mem), "h"(v.x), "h"(v.y));
}

__device__ __inline__
void st_global_cs(ushort4 *mem, ushort4 v)
{
    asm("st.global.cs.v4.u16 [%0],{%1,%2,%3,%4};"
        :: "l"(mem), "h"(v.x), "h"(v.y), "h"(v.z), "h"(v.w));
}

__device__ __inline__
void st_global_cs(int *mem, int v)
{
    asm("st.global.cs.s32 [%0],%1;" :: "l"(mem), "r"(v));
}

__device__ __inline__
void st_global_cs(int2 *mem, int2 v)
{
    asm("st.global.cs.v2.s32 [%0],{%1,%2};"
        :: "l"(mem), "r"(v.x), "r"(v.y));
}

__device__ __inline__
void st_global_cs(int4 *mem, int4 v)
{
    asm("st.global.cs.v4.s32 [%0],{%1,%2,%3,%4};"
        :: "l"(mem), "r"(v.x), "r"(v.y), "r"(v.z), "r"(v.w));
}

__device__ __inline__
void st_global_cs(unsigned int *mem, unsigned int v)
{
    asm("st.global.cs.u32 [%0],%1;" :: "l"(mem), "r"(v));
}

__device__ __inline__
void st_global_cs(uint2 *mem, uint2 v)
{
    asm("st.global.cs.v2.u32 [%0],{%1,%2};"
        :: "l"(mem), "r"(v.x), "r"(v.y));
}

__device__ __inline__
void st_global_cs(uint4 *mem, uint4 v)
{
    asm("st.global.cs.v4.u32 [%0],{%1,%2,%3,%4};"
        :: "l"(mem), "r"(v.x), "r"(v.y), "r"(v.z), "r"(v.w));
}

__device__ __inline__
void st_global_cs(float *mem, float v)

{
    asm("st.global.cs.f32 [%0],%1;" :: "l"(mem), "f"(v));
}

__device__ __inline__
void st_global_cs(float2 *mem, float2 v)

{
    asm("st.global.cs.v2.f32 [%0],{%1,%2};"
        :: "l"(mem), "f"(v.x), "f"(v.y));
}

__device__ __inline__
void st_global_cs(float4 *mem, float4 v)

{
    asm("st.global.cs.v4.f32 [%0],{%1,%2,%3,%4};"
        :: "l"(mem), "f"(v.x), "f"(v.y), "f"(v.z), "f"(v.w));
}

template <class T>
__device__ __inline__
void st_global_cs(T *const *mem, const T *v)

{
    asm("st.global.cs.u64 [%0],%1;" :: "l"(mem), "l"(v));
}
/*}}}*/

// ld_global_ca/*{{{*/

__device__ __inline__
short ld_global_ca(const short *mem)

{
    short r;
    asm("ld.global.ca.s16 %0,[%1];" : "=h"(r) : "l"(mem));
    return r;
}

__device__ __inline__
short2 ld_global_ca(const short2 *mem)

{
    short2 r;
    asm("ld.global.ca.v2.s16 {%0,%1},[%2];" : "=h"(r.x), "=h"(r.y) : "l"(mem));
    return r;
}

__device__ __inline__
short4 ld_global_ca(const short4 *mem)

{
    short4 r;
    asm("ld.global.ca.v4.s16 {%0,%1,%2,%3},[%4];"
        : "=h"(r.x), "=h"(r.y), "=h"(r.z), "=h"(r.w) : "l"(mem));
    return r;
}

__device__ __inline__
unsigned short ld_global_ca(const unsigned short *mem)

{
    unsigned short r;
    asm("ld.global.ca.u16 %0,[%1];" : "=h"(r) : "l"(mem));
    return r;
}

__device__ __inline__
ushort2 ld_global_ca(const ushort2 *mem)

{
    ushort2 r;
    asm("ld.global.ca.v2.u16 {%0,%1},[%2];" : "=h"(r.x), "=h"(r.y) : "l"(mem));
    return r;
}

__device__ __inline__
ushort4 ld_global_ca(const ushort4 *mem)

{
    ushort4 r;
    asm("ld.global.ca.v4.u16 {%0,%1,%2,%3},[%4];"
        : "=h"(r.x), "=h"(r.y), "=h"(r.z), "=h"(r.w) : "l"(mem));
    return r;
}

__device__ __inline__
int ld_global_ca(const int *mem)

{
    int r;
    asm("ld.global.ca.s32 %0,[%1];" : "=r"(r) : "l"(mem));
    return r;
}

__device__ __inline__
int2 ld_global_ca(const int2 *mem)

{
    int2 r;
    asm("ld.global.ca.v2.s32 {%0,%1},[%2];" : "=r"(r.x), "=r"(r.y) : "l"(mem));
    return r;
}

__device__ __inline__
int4 ld_global_ca(const int4 *mem)

{
    int4 r;
    asm("ld.global.ca.v4.s32 {%0,%1,%2,%3},[%4];"
        : "=r"(r.x), "=r"(r.y), "=r"(r.z), "=r"(r.w) : "l"(mem));
    return r;
}

__device__ __inline__
unsigned ld_global_ca(const unsigned *mem)

{
    unsigned r;
    asm("ld.global.ca.u32 %0,[%1];" : "=r"(r) : "l"(mem));
    return r;
}

__device__ __inline__
uint2 ld_global_ca(const uint2 *mem)

{
    uint2 r;
    asm("ld.global.ca.v2.u32 {%0,%1},[%2];" : "=r"(r.x), "=r"(r.y) : "l"(mem));
    return r;
}

__device__ __inline__
uint4 ld_global_ca(const uint4 *mem)

{
    uint4 r;
    asm("ld.global.ca.v4.u32 {%0,%1,%2,%3},[%4];"
        : "=r"(r.x), "=r"(r.y), "=r"(r.z), "=r"(r.w) : "l"(mem));
    return r;
}

__device__ __inline__
float ld_global_ca(const float *mem)

{
    float r;
    asm("ld.global.ca.f32 %0,[%1];" : "=f"(r) : "l"(mem));
    return r;
}

__device__ __inline__
float2 ld_global_ca(const float2 *mem)

{
    float2 r;
    asm("ld.global.ca.v2.f32 {%0,%1},[%2];" : "=f"(r.x), "=f"(r.y) : "l"(mem));
    return r;
}

__device__ __inline__
float4 ld_global_ca(const float4 *mem)

{
    float4 r;
    asm("ld.global.ca.v4.f32 {%0,%1,%2,%3},[%4];"
        : "=f"(r.x), "=f"(r.y), "=f"(r.z), "=f"(r.w) : "l"(mem));
    return r;
}

template <class T>
__device__ __inline__
T *ld_global_ca(const T *const *mem)

{
    T *r;
    asm("ld.global.ca.u64 %0,[%1];" : "=l"(r) : "l"(mem));
    return r;
}
/*}}}*/

// st_global_wb/*{{{*/

__device__ __inline__
void st_global_wb(short *mem, short v)
{
    asm("st.global.wb.s16 [%0],%1;" :: "l"(mem), "h"(v));
}

__device__ __inline__
void st_global_wb(short2 *mem, short2 v)
{
    asm("st.global.wb.v2.s16 [%0],{%1,%2};"
        :: "l"(mem), "h"(v.x), "h"(v.y));
}

__device__ __inline__
void st_global_wb(short4 *mem, short4 v)
{
    asm("st.global.wb.v4.s16 [%0],{%1,%2,%3,%4};"
        :: "l"(mem), "h"(v.x), "h"(v.y), "h"(v.z), "h"(v.w));
}

__device__ __inline__
void st_global_wb(unsigned short *mem, unsigned short v)
{
    asm("st.global.wb.u16 [%0],%1;" :: "l"(mem), "h"(v));
}

__device__ __inline__
void st_global_wb(ushort2 *mem, ushort2 v)
{
    asm("st.global.wb.v2.u16 [%0],{%1,%2};"
        :: "l"(mem), "h"(v.x), "h"(v.y));
}

__device__ __inline__
void st_global_wb(ushort4 *mem, ushort4 v)
{
    asm("st.global.wb.v4.u16 [%0],{%1,%2,%3,%4};"
        :: "l"(mem), "h"(v.x), "h"(v.y), "h"(v.z), "h"(v.w));
}

__device__ __inline__
void st_global_wb(int *mem, int v)
{
    asm("st.global.wb.s32 [%0],%1;" :: "l"(mem), "r"(v));
}

__device__ __inline__
void st_global_wb(int2 *mem, int2 v)
{
    asm("st.global.wb.v2.s32 [%0],{%1,%2};"
        :: "l"(mem), "r"(v.x), "r"(v.y));
}

__device__ __inline__
void st_global_wb(int4 *mem, int4 v)
{
    asm("st.global.wb.v4.s32 [%0],{%1,%2,%3,%4};"
        :: "l"(mem), "r"(v.x), "r"(v.y), "r"(v.z), "r"(v.w));
}

__device__ __inline__
void st_global_wb(unsigned int *mem, unsigned int v)
{
    asm("st.global.wb.u32 [%0],%1;" :: "l"(mem), "r"(v));
}

__device__ __inline__
void st_global_wb(uint2 *mem, uint2 v)
{
    asm("st.global.wb.v2.u32 [%0],{%1,%2};"
        :: "l"(mem), "r"(v.x), "r"(v.y));
}

__device__ __inline__
void st_global_wb(uint4 *mem, uint4 v)
{
    asm("st.global.wb.v4.u32 [%0],{%1,%2,%3,%4};"
        :: "l"(mem), "r"(v.x), "r"(v.y), "r"(v.z), "r"(v.w));
}

__device__ __inline__
void st_global_wb(float *mem, float v)

{
    asm("st.global.wb.f32 [%0],%1;" :: "l"(mem), "f"(v));
}

__device__ __inline__
void st_global_wb(float2 *mem, float2 v)

{
    asm("st.global.wb.v2.f32 [%0],{%1,%2};"
        :: "l"(mem), "f"(v.x), "f"(v.y));
}

__device__ __inline__
void st_global_wb(float4 *mem, float4 v)

{
    asm("st.global.wb.v4.f32 [%0],{%1,%2,%3,%4};"
        :: "l"(mem), "f"(v.x), "f"(v.y), "f"(v.z), "f"(v.w));
}

template <class T>
__device__ __inline__
void st_global_wb(T *const *mem, const T *v)

{
    asm("st.global.wb.u64 [%0],%1;" :: "l"(mem), "l"(v));
}
/*}}}*/

// ldu_global/*{{{*/

__device__ __inline__
short ldu_global(const short *mem)

{
    short r;
    asm("ldu.global.s16 %0,[%1];" : "=h"(r) : "l"(mem));
    return r;
}

__device__ __inline__
short2 ldu_global(const short2 *mem)

{
    short2 r;
    asm("ldu.global.v2.s16 {%0,%1},[%2];" : "=h"(r.x), "=h"(r.y) : "l"(mem));
    return r;
}

__device__ __inline__
short4 ldu_global(const short4 *mem)

{
    short4 r;
    asm("ldu.global.v4.s16 {%0,%1,%2,%3},[%4];"
        : "=h"(r.x), "=h"(r.y), "=h"(r.z), "=h"(r.w) : "l"(mem));
    return r;
}

__device__ __inline__
unsigned short ldu_global(const unsigned short *mem)

{
    unsigned short r;
    asm("ldu.global.u16 %0,[%1];" : "=h"(r) : "l"(mem));
    return r;
}

__device__ __inline__
ushort2 ldu_global(const ushort2 *mem)

{
    ushort2 r;
    asm("ldu.global.v2.u16 {%0,%1},[%2];" : "=h"(r.x), "=h"(r.y) : "l"(mem));
    return r;
}

__device__ __inline__
ushort4 ldu_global(const ushort4 *mem)

{
    ushort4 r;
    asm("ldu.global.v4.u16 {%0,%1,%2,%3},[%4];"
        : "=h"(r.x), "=h"(r.y), "=h"(r.z), "=h"(r.w) : "l"(mem));
    return r;
}

__device__ __inline__
int ldu_global(const int *mem)

{
    int r;
    asm("ldu.global.s32 %0,[%1];" : "=r"(r) : "l"(mem));
    return r;
}

__device__ __inline__
int2 ldu_global(const int2 *mem)

{
    int2 r;
    asm("ldu.global.v2.s32 {%0,%1},[%2];" : "=r"(r.x), "=r"(r.y) : "l"(mem));
    return r;
}

__device__ __inline__
int4 ldu_global(const int4 *mem)

{
    int4 r;
    asm("ldu.global.v4.s32 {%0,%1,%2,%3},[%4];"
        : "=r"(r.x), "=r"(r.y), "=r"(r.z), "=r"(r.w) : "l"(mem));
    return r;
}

__device__ __inline__
unsigned ldu_global(const unsigned *mem)

{
    unsigned r;
    asm("ldu.global.u32 %0,[%1];" : "=r"(r) : "l"(mem));
    return r;
}

__device__ __inline__
uint2 ldu_global(const uint2 *mem)

{
    uint2 r;
    asm("ldu.global.v2.u32 {%0,%1},[%2];" : "=r"(r.x), "=r"(r.y) : "l"(mem));
    return r;
}

__device__ __inline__
uint4 ldu_global(const uint4 *mem)

{
    uint4 r;
    asm("ldu.global.v4.u32 {%0,%1,%2,%3},[%4];"
        : "=r"(r.x), "=r"(r.y), "=r"(r.z), "=r"(r.w) : "l"(mem));
    return r;
}

__device__ __inline__
float ldu_global(const float *mem)

{
    float r;
    asm("ldu.global.f32 %0,[%1];" : "=f"(r) : "l"(mem));
    return r;
}

__device__ __inline__
float2 ldu_global(const float2 *mem)

{
    float2 r;
    asm("ldu.global.v2.f32 {%0,%1},[%2];" : "=f"(r.x), "=f"(r.y) : "l"(mem));
    return r;
}

__device__ __inline__
float4 ldu_global(const float4 *mem)

{
    float4 r;
    asm("ldu.global.v4.f32 {%0,%1,%2,%3},[%4];"
        : "=f"(r.x), "=f"(r.y), "=f"(r.z), "=f"(r.w) : "l"(mem));
    return r;
}

template <class T>
__device__ __inline__
T *ldu_global(const T *const *mem)

{
    T *r;
    asm("ldu.global.u64 %0,[%1];" : "=l"(r) : "l"(mem));
    return r;
}
/*}}}*/

}

#endif
