#include "alloc.h"

namespace gpu
{

template <class T>
value<T>::value(const value &val)
{
    m_value = cuda_new_array<T>(1);

    cudaMemcpy(m_value, val.m_value, sizeof(T), cudaMemcpyDeviceToDevice);
    check_cuda_error("Error during memcpy from device to device");
}

template <class T>
value<T>::value(const T &val)
{
    m_value = cuda_new_array<T>(1);

    cudaMemcpy(m_value, &val, sizeof(T), cudaMemcpyHostToDevice);
    check_cuda_error("Error during memcpy from host to device");
}

template <class T>
value<T>::value()
{
    m_value = cuda_new_array<T>(1);
}

template <class T>
value<T>::~value()
{
    cuda_delete(m_value);
}

template <class T>
value<T> &value<T>::operator=(const T &val)
{
    cudaMemcpy(m_value, &val, sizeof(T), cudaMemcpyHostToDevice);
    return *this;
}

template <class T>
value<T> &value<T>::operator=(const value &val)
{
    cudaMemcpy(m_value, val.m_value, sizeof(T), cudaMemcpyDeviceToDevice);
    return *this;
}

template <class T>
value<T>::operator T() const
{
    T val;
    cudaMemcpy(&val, m_value, sizeof(T), cudaMemcpyDeviceToHost);
    return val;
}

template <class T>
T *value<T>::operator&()
{
    return m_value;
}

template <class T>
const T *value<T>::operator&() const
{
    return m_value;
}

} // namespace gpu
