#include <cstdlib>
#include <new>
#include <cstdio>
#include <vector>
#include <iostream>
#include <stdexcept>
#include <ImfOutputFile.h>
#include <ImfInputFile.h>
#include <ImfChannelList.h>
#include <ImfStringAttribute.h>
#include <ImfMatrixAttribute.h>
#include <ImathBox.h>
#include <half.h>

#include "exrimage.h"
#include "color.h"

typedef struct _t_attr {
    char *key, *text;
} t_attr;

static std::vector<t_attr> g_text;

template <class T>
void unsafe_store(const char *name, const image::rgba<T> &rgba);

#if 0
template <>
void unsafe_store(const char *name, const image::rgba<float> &rgba) {
    int width = rgba.width();
    int height = rgba.height();
    Imf::Header header(width, height);
    header.channels().insert("R", Imf::Channel(Imf::HALF));
    header.channels().insert("G", Imf::Channel(Imf::HALF));
    header.channels().insert("B", Imf::Channel(Imf::HALF));
    header.channels().insert("A", Imf::Channel(Imf::HALF));
    for (unsigned i = 0; i < g_text.size(); i++) {
        header.insert(g_text[i].key, Imf::StringAttribute(g_text[i].text));
    }
    Imf::OutputFile file(name, header);
    Imf::FrameBuffer buffer;
    half *tmp = new half[width*height*4];
    for (int i = 0; i < height; i++) {
        for (int j = 0; j < width; j++) {
            int p = i*width+j;
            int q = (height-i-1)*width+j;
            tmp[q+0*width*height] = static_cast<half>(rgba.r[p]);
            tmp[q+1*width*height] = static_cast<half>(rgba.g[p]);
            tmp[q+2*width*height] = static_cast<half>(rgba.b[p]);
            tmp[q+3*width*height] = static_cast<half>(rgba.a[p]);
        }
    }
    buffer.insert("R", Imf::Slice(Imf::HALF, (char *) (tmp+0*width*height),
        sizeof(half), sizeof(half)*width));
    buffer.insert("G", Imf::Slice(Imf::HALF, (char *) (tmp+1*width*height),
        sizeof(half), sizeof(half)*width));
    buffer.insert("B", Imf::Slice(Imf::HALF, (char *) (tmp+2*width*height),
        sizeof(half), sizeof(half)*width));
    buffer.insert("A", Imf::Slice(Imf::HALF, (char *) (tmp+3*width*height),
        sizeof(half), sizeof(half)*width));
    file.setFrameBuffer(buffer);
    file.writePixels(height);
    delete tmp;
}
#endif

template <>
void unsafe_store(const char *name, const image::rgba<float> &rgba) {
    int width = rgba.width();
    int height = rgba.height();
    Imf::Header header(width, height);
    header.channels().insert("R", Imf::Channel(Imf::FLOAT));
    header.channels().insert("G", Imf::Channel(Imf::FLOAT));
    header.channels().insert("B", Imf::Channel(Imf::FLOAT));
    header.channels().insert("A", Imf::Channel(Imf::FLOAT));
    // copy meta-data
    for (unsigned i = 0; i < g_text.size(); i++) {
        header.insert(g_text[i].key, Imf::StringAttribute(g_text[i].text));
    }
    Imf::OutputFile file(name, header);
    Imf::FrameBuffer buffer;
    int xs = sizeof(float);
    int ys = sizeof(float)*width;
	ys = -ys;
    int os = width*(height-1)*sizeof(float);
    buffer.insert("R", Imf::Slice(Imf::FLOAT, (char *) (rgba.r)+os, xs, ys));
    buffer.insert("G", Imf::Slice(Imf::FLOAT, (char *) (rgba.g)+os, xs, ys));
    buffer.insert("B", Imf::Slice(Imf::FLOAT, (char *) (rgba.b)+os, xs, ys));
    buffer.insert("A", Imf::Slice(Imf::FLOAT, (char *) (rgba.a)+os, xs, ys));
    file.setFrameBuffer(buffer);
    file.writePixels(height);
}

template <class T>
void unsafe_load(const char *name, image::rgba<T> *rgba);


#if 0
template <>
void unsafe_load(const char *name, image::rgba<float> *rgba) {
    Imf::InputFile file(name);
    Imath::Box2i dw = file.header().dataWindow();
    int width = dw.max.x - dw.min.x + 1;
    int height = dw.max.y - dw.min.y + 1;
    half *tmp = new half[width*height*4];
    Imf::FrameBuffer buffer;
    if (!rgba->resize(height, width))
        throw std::bad_alloc();
    buffer.insert("R", Imf::Slice (Imf::HALF,
        (char *) (tmp+0*width*height-dw.min.x-dw.min.y*width), sizeof(half),
        sizeof(half)*width, 1, 1, 0.f));
    buffer.insert("G", Imf::Slice (Imf::HALF,
        (char *) (tmp+1*width*height-dw.min.x-dw.min.y*width), sizeof(half),
        sizeof(half)*width, 1, 1, 0.f));
    buffer.insert("B", Imf::Slice (Imf::HALF,
        (char *) (tmp+2*width*height-dw.min.x-dw.min.y*width), sizeof(half),
        sizeof(half)*width, 1, 1, 0.f));
    buffer.insert("A", Imf::Slice (Imf::HALF,
        (char *) (tmp+3*width*height-dw.min.x-dw.min.y*width), sizeof(half),
        sizeof(half)*width, 1, 1, 1.f));
    file.setFrameBuffer(buffer);
    file.readPixels(dw.min.y, dw.max.y);
    for (int i = 0; i < height; i++) {
        for (int j = 0; j < width; j++) {
            int p = i*width+j;
            int q = (height-i-1)*width+j;
            rgba->r[p] = static_cast<float>(tmp[p]);
            rgba->r[p] = static_cast<float>(tmp[q+0*width*height]);
            rgba->g[p] = static_cast<float>(tmp[q+1*width*height]);
            rgba->b[p] = static_cast<float>(tmp[q+2*width*height]);
            rgba->a[p] = static_cast<float>(tmp[q+3*width*height]);
        }
    }
    delete tmp;
}
#endif


template <>
void unsafe_load(const char *name, image::rgba<float> *rgba) {
    Imf::InputFile file(name);
    Imath::Box2i dw = file.header().dataWindow();
    int width = dw.max.x - dw.min.x + 1;
    int height = dw.max.y - dw.min.y + 1;
    Imf::FrameBuffer buffer;
    if (!rgba->resize(height, width)) throw std::bad_alloc();
    int xs = sizeof(float);
    int ys = sizeof(float)*width;
	ys = -ys;
    int os = -dw.min.x*xs - (height-1)*ys - dw.min.y*ys;
    buffer.insert("R", Imf::Slice (Imf::FLOAT, (char *) (rgba->r)+os,
        xs, ys, 1, 1, 0.f));
    buffer.insert("G", Imf::Slice (Imf::FLOAT, (char *) (rgba->g)+os,
        xs, ys, 1, 1, 0.f));
    buffer.insert("B", Imf::Slice (Imf::FLOAT, (char *) (rgba->b)+os,
        xs, ys, 1, 1, 0.f));
    buffer.insert("A", Imf::Slice (Imf::FLOAT, (char *) (rgba->a)+os,
        xs, ys, 1, 1, 1.f));
    file.setFrameBuffer(buffer);
    file.readPixels(dw.min.y, dw.max.y);
}

namespace image { namespace exr {

    void free_text(void) {
        for (unsigned i = 0; i < g_text.size(); i++) {
            free(g_text[i].key);
            free(g_text[i].text);
        }
        g_text.resize(0);
    }

    void init_text(int argc, char **argv) {
        free_text();
        g_text.resize(argc);
        for (int i = 0; i < argc; i++) {
            char key[256];
            sprintf(key, "argv:%02d", i);
            g_text[i].key = strdup(key);
            g_text[i].text = strdup(argv[i]);
        }
    }

    void push_text(const char *key, const char *text) {
        t_attr entry;
        entry.key = strdup(key);
        entry.text = strdup(text);
        g_text.push_back(entry);
    }

    void pop_text(int n) {
        for (int i = 0; i < n; i++) {
            if (!g_text.empty()) {
                free(g_text.back().key);
                free(g_text.back().text);
                g_text.pop_back();
            }
        }
    }

    template <>
    int load(const char *name, image::rgba<float> *rgba) {
        try {
            unsafe_load(name, rgba);
        } catch (const std::exception &exc) {
            std::cerr << exc.what() << std::endl;
            return 0;
        }
        return 1;
    }

    template <>
    int store(const char *name, const image::rgba<float> &rgba) {
        try {
            unsafe_store(name, rgba);
        } catch (const std::exception &exc) {
            std::cerr << exc.what() << std::endl;
            return 0;
        }
        return 1;
    }
} } // namespaces png and image
