#ifndef NEHAB_BBOX_H
#define NEHAB_BBOX_H

#include <cfloat>
#include "point.h"

namespace math
{
    struct Box
    {
        Box()
        {
            min.x = min.y = FLT_MAX;
            max.x = max.y = -FLT_MAX;
        }

        Box(const Point &_min, const Point &_max)
           : min(_min), max(_max)
        {
        }

        void reset(const Point &p)
        {
            min.x = max.x = p.x;
            min.y = max.y = p.y;
        }

        void reset()
        {
            min.x = min.y = FLT_MAX;
            max.x = max.y = -FLT_MAX;
        }

        void merge(const Point &p)
        {
            if(p.x < min.x)
                min.x = p.x;
            if(p.y < min.y)
                min.y = p.y;
            if(p.x > max.x)
                max.x = p.x;
            if(p.y > max.y)
                max.y = p.y;
        }

        bool contains(const Point &p) const
        {
            return min.x <= p.x && p.x <= max.x &&
                   min.y <= p.y && p.y <= max.y;
        }

        bool intersects(const Box &other) const
        {
            return (other.min.x <= max.x && min.x <= other.max.x) &&
                   (other.min.y <= max.y && min.y <= other.max.y);
        }

        void sub_boxes(Box sub[4]) const
        {
            Point mp = 0.5f*( min + max );
            //bottom_left
            sub[0] = Box(min, mp);
            //bottom_right
            sub[1] = Box(Point(mp.x, min.y), Point(max.x, mp.y));
            //top_left
            sub[2] = Box(Point(min.x, mp.y), Point(mp.x, max.y));
            //top_right
            sub[3] = Box(mp, max);
        }

        Point min, max;
    };
}

#endif
