#ifndef NEHAB_MATH_H
#define NEHAB_MATH_H

#include <cmath>

namespace math {

static const float PI = 3.141592653589f;
// Not used as a kernel because it is infinite
// Used in windowed-sinc approximations
static inline float sincf(float t)
{
    // if f is too small, use Maclaurin series to
    // imprecision near zero
    if (fabsf(t) < 0.1)
        return 1.f+t*t*(-1.f/6.f+t*t*(1.f/120.f-t*t/5040.f));
    // otherwise, use trigonometric function definition
    return sinf(t)/t;
}

#ifdef _WIN32
static inline float roundf(float t) { return floorf(t + .5f); }
static inline float rintf(float t) { return roundf(t); } // revert to round
static inline float log1pf(float x) { return logf(1.f+x); }
static inline float expm1f(float x) { return expf(x)-1.f; }
#else
using ::roundf;
using ::rintf;
using ::log1pf;
using ::expm1f;
#endif

} // namespace math

#endif
