#ifndef NEHAB_POINT_H
#define NEHAB_POINT_H

#include <vector>
#include <boost/functional/hash.hpp>

namespace math
{
    struct Point
    {
        Point(void) { x = 0; y = 0; }
        Point(float x, float y) { this->x = x; this->y = y; }

        Point operator+(const Point &o) const { return Point(x+o.x, y+o.y); }
        Point operator-(const Point &o) const { return Point(x-o.x, y-o.y); }
        Point operator*(float f) const { return Point(f*x, f*y); }
        Point operator*(const Point &o) const { return Point(o.x*x, o.y*y); }
        Point operator/(float f) const { return Point(x/f, y/f); }
        bool operator==(const Point &o) const { return x == o.x && y == o.y; }
        bool operator!=(const Point &o) const { return x != o.x || y != o.y; }
        Point operator-(void) const { return Point(-x, -y); }

        float x, y;
    };

    static inline Point operator*(float f, const Point &p)
    {
        return Point(f*p.x, f*p.y);
    }

    static inline float dot(const Point &p0, const Point &p1)
    {
        return p0.x*p1.x + p0.y*p1.y;
    }

    static inline float cross(const Point &p0, const Point &p1)
    {
        return p0.x*p1.y - p0.y*p1.x;
    }

    static inline Point cross(const Point &p)
    {
        return Point(-p.y, p.x);
    }

    using boost::hash_value;
    using boost::hash_combine;

    inline
    size_t hash_value(const Point &p)
    {
        size_t h = hash_value(p.x);
        hash_combine(h, p.y);
        return h;
    }
}

#endif

