#ifndef ROD_UTIL_H
#define ROD_UTIL_H

// most of these utilities are in C++TR1 or C++11,
// but were supporting C++03, so here they are...

#include <limits> // for numeric_limits
#include <string>

namespace rod
{

inline std::string strupr(const std::string &str)
{
    std::string ret;
    ret.reserve(str.size());

    for(size_t i=0; i<ret.size(); ++i)
        ret.push_back(toupper(str[i]));
    return ret;
}

template <bool EN, class T=void>
struct enable_if
{
    typedef T type;
};

template <class T>
struct enable_if<false,T>
{
};

template <class FROM, class TO>
struct is_convertible
{
private:
    struct yes {};
    struct no {yes m[2]; };

    static yes can_convert(TO *) {}
    static no can_convert(const void *) {}

public:
    static const bool value
        = sizeof(can_convert((FROM *)NULL)) == sizeof(yes);
};

template <class T>
struct remove_const
{
    typedef T type;
};

template <class T>
struct remove_const<const T>
{
    typedef T type;
};

template <class T>
struct remove_volatile
{
    typedef T type;
};

template <class T>
struct remove_volatile<volatile T>
{
    typedef T type;
};

template <class T>
struct remove_cv
{
    typedef typename remove_const<typename remove_volatile<T>::type>::type type;
};

template <class FROM, class TO>
struct copy_const
{
    typedef TO type;
};

template <class FROM, class TO>
struct copy_const<const FROM, TO>
{
    typedef const TO type;
};

template <class T>
struct is_const
{
    static const bool value = false;
};

template <class T>
struct is_const<const T>
{
    static const bool value = true;
};

template <class T>
struct is_volatile
{
    static const bool value = false;
};

template <class T>
struct is_volatile<volatile T>
{
    static const bool value = true;
};

template <class T>
struct is_integral
{
    static const bool value
        = std::numeric_limits<typename remove_cv<T>::type>::is_integer;
};

template <class T, class U>
struct is_same
{
    static const bool value = false;
};

template <class T>
struct is_same<T,T>
{
    static const bool value = true;
};

template <bool B, class T, class F>
struct conditional
{
    typedef T type;
};

template <class T, class F>
struct conditional<false,T,F>
{
    typedef F type;
};

} // namespace rod


// from boost::preprocessor
// concatenate expanded arguments

#define PP_CAT(a, b) PP_CAT_I(a, b)
#define PP_CAT_I(a, b) a ## b

#define PP_CAT3(a, b, c) PP_CAT(a,PP_CAT(b,c))

#endif
