#ifndef GPURASTER_COLOR_UTIL_H
#define GPURASTER_COLOR_UTIL_H

#include <gpu/cutil.h>

// apply sRGB non-linearity to value
__device__ __host__ inline float srgbcurve(float f)
{
    //if (f < 0.f) f = 0.f;
    //if (f > 1.f) f = 1.f;
    const float a = 0.055f;
    if (f <= 0.0031308f)
        return 12.92f*f;
    else
        return (1.f + a)*powf(f, 1.f/2.4f)-a;
    // return powf(f, 1/2.2f); // faster approximation
    // return sqrtf(f); // even faster
    //return f; // disable
}
__device__ __host__ inline float4 lrgb2srgb(float4 f)
{
    return make_float4(srgbcurve(f.x), srgbcurve(f.y), srgbcurve(f.z), f.w);
}

// remove sRGB non-linearity from value
__device__ __host__ inline float srgbuncurve(float f)
{
    //if (f < 0.f) f = 0.f;
    //if (f > 1.f) f = 1.f;
    const float a = 0.055f;
    if(f <= 0.04045f)
        return f/12.92f;
    else
        return powf((f+a)/(1.f+a), 2.4f);
    // return powf(f, 2.2f); // faster approximation
    // return f*f; // even faster
    //return f; // disable
}

__device__ __host__ inline float4 srgb2lrgb(float4 f)
{
    return make_float4(srgbuncurve(f.x), srgbuncurve(f.y), srgbuncurve(f.z), f.w);
}

__host__ __device__ inline float4 premultiply(const float4 &f)
{
    return make_float4(f.x*f.w, f.y*f.w, f.z*f.w, f.w);
}

__device__ inline float4 postdivide(const float4 &f)
{
    float a = f.w + 0.00001f;
    return make_float4(f.x/a, f.y/a, f.z/a, f.w);
}

__device__ inline float4 blend(const float4 &fg,
                               const float4 &bg)
{
    float t = 1.f-fg.w;
    return bg*t + fg;
}

#endif
