#ifndef GPURAST_BEZIER_H
#define GPURAST_BEZIER_H

#include <gpu/array.h>
#include <gpu/box.h>

template <int N>
class Curve
{
public:
    HOSTDEV float2 &operator[](int i) { return m_ctrl_points[i]; }
    HOSTDEV const float2 &operator[](int i) const { return m_ctrl_points[i]; }

    HOSTDEV static size_t size() { return N+1; }

private:
    gpu::array<float2, N+1> m_ctrl_points;
};

template <class T>
__device__ T lerp(const T &p0, const T &p1, float t)
{
    return (1-t)*p0 + t*p1;
}


template <int N>
__device__ gpu::Box get_xy_mono_bbox(const Curve<N> &B)
{
    gpu::Box bbox;

    const int last = B.size()-1;

    if(B[0].x < B[last].x)
    {
        bbox.min.x = B[0].x;
        bbox.max.x = B[last].x;
    }
    else
    {
        bbox.min.x = B[last].x;
        bbox.max.x = B[0].x;
    }

    if(B[0].y < B[last].y)
    {
        bbox.min.y = B[0].y;
        bbox.max.y = B[last].y;
    }
    else
    {
        bbox.min.y = B[last].y;
        bbox.max.y = B[0].y;
    }

    return bbox;
}


const int MAX_ROOTS = 2 // mono_x
                    + 2 // mono_y
                    + 2  // to break cubic loop
                    + 1; // original segment (last "root" == 1)
typedef gpu::array<float,MAX_ROOTS> Roots;

#endif

