#ifndef GPURASTER_CURVE3_H
#define GPURASTER_CURVE3_H

#include "curve2.h"

__device__ inline float2 eval(const Curve<3> &B, float t)
{
    Curve<2> B2;
    B2[0] = lerp(B[0], B[1], t);
    B2[1] = lerp(B[1], B[2], t);
    B2[2] = lerp(B[2], B[3], t);

    return eval(B2, t);
}

__device__ inline float2 blossom(const Curve<3> &B, float u, float v, float w)
{
    return B[0]*(-u*v*w + v*w + u*w + u*v - (v + u + w) + 1) +
           B[1]*(3*u*v*w - 2*(v*w + u*w + u*v) + u + v + w) +
           B[2]*(-3*u*v*w + v*w + u*w + u*v) +
           B[3]*(u*v*w);
}

__device__ inline Curve<3> subcurve(const Curve<3> &B, float u, float v)
{
    Curve<3> out;

    if(u == 0)
    {
        out[0] = B[0];
        out[1] = blossom(B,u,u,v);
        out[2] = blossom(B,u,v,v);
        out[3] = blossom(B,v,v,v);
    }
    else if(v == 1)
    {
        out[0] = blossom(B,u,u,u);
        out[1] = blossom(B,u,u,v);
        out[2] = blossom(B,u,v,v);
        out[3] = B[3];
    }
    else
    {
        out[0] = blossom(B,u,u,u);
        out[1] = blossom(B,u,u,v);
        out[2] = blossom(B,u,v,v);
        out[3] = blossom(B,v,v,v);
    }

    return out;
}

#endif
