#ifndef GPURASTER_CURVE_ARC_H
#define GPURASTER_CURVE_ARC_H

#include "curve.h"
#include "segtype.h"

template <>
class Curve<SEG_ARC>
{
public:
    HOSTDEV float2 &operator[](int i) { return m_ctrl_points[i]; }
    HOSTDEV const float2 &operator[](int i) const { return m_ctrl_points[i]; }

    float w;

    HOSTDEV static size_t size() { return 3; }

private:
    gpu::array<float2, 3> m_ctrl_points;
};

__device__ inline
float2 blossom(const Curve<SEG_ARC> &B, float u, float v, float &w)
{
    float uv = u*v;
    float b0 = uv - u - v+1,
          b1 = u + v - 2*uv,
          b2 = uv;

    w = 1*b0 + B.w*b1 + 1*b2;

    return B[0]*b0 + B[1]*b1 + B[2]*b2;
}

__device__ inline
Curve<SEG_ARC> subcurve(const Curve<SEG_ARC> &seg, float u, float v)
{
    Curve<SEG_ARC> out;

    float wA, wB, wC;
    float2 A = blossom(seg, u, u, wA),
           B = blossom(seg, u, v, wB),
           C = blossom(seg, v, v, wC);

    float s = 1.0f/sqrt(wA * wC);
    out[1] = s*B;
    out.w = s*wB;

    if(u == 0)
    {
        out[0] = seg[0];
        out[2] = C/wC;
    }
    else if(v == 1)
    {
        out[0] = A/wA;
        out[2] = seg[2];
    }
    else
    {
        out[0] = A/wA;
        out[2] = C/wC;
    }

    return out;
}

#endif
