#include "dump.h"

#include <iostream>
#include <fstream>
#include <cassert>

static int dump_margin( const float2& min, const float2& max, FILE *out )
{
    fprintf( out, "DUMPgsave\n" );
    fprintf( out, "DUMPsetmargincolor\nDUMP%g %g %g %g 0.5 bbox\n",
            min.x, min.y ,
            max.x, max.y );
    fprintf( out, "DUMPgrestore\n" );
    return 1;
}

static int dump_bbox( const float2& min, const float2& max, FILE *out )
{
    float o = 0;
    fprintf( out, "DUMP%%%%BoundingBox: %g %g %g %g\n",
            min.x-o, min.y-o,
            max.x+o, max.y+o );
    return 1;
}

void dump_qtree( const std::vector<float2>& min, const std::vector<float2>& max, const std::vector<int4>& ch, FILE* out )
{
    assert(min.size() == max.size());
    const float2& cmin = min[0], cmax = max[0];
    std::ifstream tmpl("driver/cuda/src/template.eps");
    if ( tmpl.is_open() ) {
        while ( tmpl.good() ) {
            std::string line;
            getline( tmpl, line );
            if ( line.compare("%%CONTENT%%") == 0 ) {
                for (unsigned i=0; i<min.size(); ++i) {
                    int4 c = ch[i];
                    if ( c.x == -1 && c.y == -1
                            && c.z == -1 && c.w == -1 ) {
                        dump_margin(min[i], max[i], out);
                    }
                }
                //dump_cell( context, *cell, cbox, 0, 0, maxlevel - l, out, i, j, l );
            }
            else if ( line.compare("%%%BBOX%%%") == 0 ) {
                dump_bbox(cmin, cmax, out);
            }
            else {
                fprintf( out, "DUMP%s\n", line.data() );
            }
        }
        tmpl.close();
    }
    else {
        std::cerr << "template not found!" << std::endl;
    }
}
