#ifndef GPURASTER_GRADIENTS_H
#define GPURASTER_GRADIENTS_H

#include <fd/fontdump.h>
#include <gpu/vector.h>
#include <gpu/cutil.h>
#include "util.h"

struct Linear
{
    float2 p1, p2;
};

struct Radial
{
    float2 c, f;
    float r;
};

struct Gradient
{
    enum Type
    {
        LINEAR,
        RADIAL
    };

    Gradient() : type(LINEAR) {}

    Type type;

    union
    {
        Linear linear;
        Radial radial;
    };

    float inv_ramp_size;
    cudaTextureObject_t ramp;
    Transform transform;
};

struct TexRamp
{
    TexRamp() : tex(0), data(NULL), ramp_size(0) {}

    cudaTextureObject_t tex;
    cudaArray *data;
    size_t ramp_size;
};

TexRamp convert_ramp(const fd::Spread &spread,const fd::Ramp &ramp);

void free(TexRamp &ramp);

void convert_gradients(const fd::FontDump &fd,
                       std::vector<TexRamp> &out_texramp,
                       gpu::vector<Gradient> &out_grads,
                       float2 scale, float2 offset);

class Paint;

void invert_paint_xforms(gpu::vector<Paint *> &paints,
                         const gpu::vector<Transform> &orig_xform,
                         float2 scale, float2 off);

#endif
