#ifndef GPURASTER_ICURVE_H
#define GPURASTER_ICURVE_H

#include <gpu/box.h>

enum CurveFlags
{
    CFLAG_GOING_UP = 1,
    CFLAG_GOING_RIGHT = 2,
    CFLAG_GEOM_TO_LEFT = 4,
    CFLAG_WHICH_CORNER = 8,
    CFLAG_OUT_IS_LEFT = 16,
    CFLAG_NEGATIVE = 32
};

struct ImplicitCurveBase
{
    gpu::Box bbox;
    unsigned flags;

    __device__
    bool going_right() const { return flags & CFLAG_GOING_RIGHT; }

    __device__
    bool going_up() const { return flags & CFLAG_GOING_UP; }

    __device__
    bool geom_to_left() const { return flags & CFLAG_GEOM_TO_LEFT; }

    __device__
    bool out_is_left() const { return flags & CFLAG_OUT_IS_LEFT; }


    __device__
    float2 begpoint() const
    {
        float2 r;
        r.x = going_right() ? bbox.min.x : bbox.max.x;
        r.y = going_up()    ? bbox.min.y : bbox.max.y;
        return r;
    }

    __device__
    float2 endpoint() const
    {
        float2 r;
        r.x = going_right() ? bbox.max.x : bbox.min.x;
        r.y = going_up()    ? bbox.max.y : bbox.min.y;
        return r;
    }

#if 0
    __device__
    bool base_hit( const float2 &pt, int& isect ) const
    {
        int check = bbox.hit(pt);
        if(check == 0) {
            //return 0; //above, bellow or to the right
            // isect += 0;
            return true;
        }
        else if (check == 1) { // to the left
            isect += going_up() ? 1 : -1;
            return true;
        }
        else {
            float2 b = begpoint(), e = endpoint();
            check = hit_chull(pt, b, e);
            if (check == 0) {
                //isect += 0;
                return true;
            }
            else if(check == 1) {
                isect += going_up() ? 1 : -1;
                return true;
            }
            else {
                //if(eval(pt, b, e) < 0)
                //    return going_up() ? 1 : -1;
                //    exec eval for the correct type
                return false;
            }
        }
        return true;
    }
#endif

};

template <int N=0>
struct ImplicitCurve;

#endif
