#ifndef GPURASTER_ICURVE1_H
#define GPURASTER_ICURVE1_H

#include <gpu/linalg.h>
#include "icurve.h"
#include "util.h"

template <>
struct ImplicitCurve<1> : ImplicitCurveBase
{
    __device__
    float eval(const float2 &pt) const
    {
        float2 b = begpoint(),
               e = endpoint();

        float A = e.y - b.y,
              B = b.x - e.x,
              C = -b.y*B - b.x*A;

        float u = A*pt.x + B*pt.y + C;

        // If b.y < e.y then the ccw-left is the global left and we are OK.
        // Otherwise, we need to change the sign of the function
        if(A > 0)
            return u;
        else
            return -u;
    }

    __device__
    int hit_chull(const float2 &pt) const
    {
        (void) pt;
        return -1;
    }

    __device__
    int hit(const float2 &pt) const
    {
        int check = bbox.hit(pt);
        if(check == 0)
            return 0;

        if(check == 1 || eval(pt) < 0 ) // < 0 ? hit
            return going_up() ? 1 : -1;
        return 0;
    }
};

#endif
