#ifndef GPURASTER_ICURVE2_H
#define GPURASTER_ICURVE2_H

#include "icurve.h"
#include "util.h"

template <>
struct ImplicitCurve<2> : ImplicitCurveBase
{
    //float u,v;
    gpu::math::Matrix<float,2,2> M;

    __device__
    bool which_corner() const { return flags & CFLAG_WHICH_CORNER; }

    __device__
    bool negative() const { return flags & CFLAG_NEGATIVE; }

    __device__
    float eval(const float2 &pt, float2 b, float2 e) const
    {
        (void) e;
        float2 p = pt - b;

        float u = M[0][0]*p.x + M[0][1]*p.y;
        float v = M[1][0]*p.x + M[1][1]*p.y;

        float f = u*u - v;
        if(negative()) {
            return -f;
        }
        else {
            return f;
        }
    }

    __device__
    float eval(const float2 &pt) const
    {
        return eval(pt, begpoint(), endpoint());
    }


    __device__
    int hit_chull(const float2 &pt, float2 b, float2 e) const
    {
        bool sample_to_the_left = ccw(pt, b, e) < 0;

        if(sample_to_the_left == geom_to_left())
            return -1;

        // sample is on the opposite side of geometry
        if(out_is_left())
            return 1;
        else
            return 0;
    }

    __device__
    int hit_chull(const float2 &pt) const
    {
        return hit_chull(pt, begpoint(), endpoint());
    }

    __device__
    int hit(const float2 &pt) const
    {
        int check = bbox.hit(pt);
        if(check == 0)
            return 0; //above, bellow or to the right
        else if (check == 1) // to the left
            return going_up() ? 1 : -1;
        else
        {
            float2 b = begpoint(), e = endpoint();
            check = hit_chull(pt, b, e);
            if (check == 0)
                return 0;
            else if(check == 1)
                return going_up() ? 1 : -1;
            else
            {
                if(eval(pt, b, e) < 0)
                    return going_up() ? 1 : -1;
            }
        }
        return 0;
    }
};

#endif
