#ifndef GPURASTER_ICURVE3_H
#define GPURASTER_ICURVE3_H

#include "icurve.h"
#include <gpu/linalg.h>

template <>
struct ImplicitCurve<3> : ImplicitCurveBase
{
    float2 base, E;
    gpu::math::Matrix<float,3,3> M;

    __device__
    bool negative() const { return flags & CFLAG_NEGATIVE; }

    __device__
    int hit_chull(const float2 &pt) const
    {
        float2 b = begpoint(),
               e = endpoint();

        bool sample_to_the_left = ccw(pt, b, e) < 0;
        if(sample_to_the_left == geom_to_left())
        {
            // test if sample is inside the triangle beE
            if((ccw(pt, e, E) < 0) == sample_to_the_left)
            {
                if((ccw(pt, E, b) < 0) == sample_to_the_left)
                    return -1; // inside control polygon
            }
            // is outside
            if(out_is_left())
                return 0;
            else
                return 1;
        }
        // sample is on the opposite side of geometry
        if(out_is_left())
            return 1;
        else
            return 0;
    }

    __device__
    float eval(const float2 &pt) const
    {
        float2 s = pt - base;

        float k = M[0][0]*s.x + M[0][1]*s.y + M[0][2];
        float l = M[1][0]*s.x + M[1][1]*s.y + M[1][2];
        float m = M[2][0]*s.x + M[2][1]*s.y + M[2][2];

        float f = k*k*k - l*m;

        if(negative())
            return -f;
        else
            return f;
    }

    __device__
    int hit(const float2 &pt) const
    {
        int check = bbox.hit(pt);
        if(check == 0)
            return 0; //above, bellow or to the right
        else if (check == 1)  // to the left
            return going_up() ? 1 : -1;
        else 
        {
            // float2 b = begpoint(), e = endpoint(); // never used
            check = hit_chull(pt);
            if (check == 0)
                return 0;
            else if(check == 1)
                return going_up() ? 1 : -1;
            else
            {
                if (eval(pt) < 0)
                    return going_up() ? 1 : -1;
            }
        }
        return 0;
    }
};

#endif
