#ifndef GPURASTER_ICURVE_ARC_H
#define GPURASTER_ICURVE_ARC_H

#include "icurve.h"
#include <gpu/linalg.h>

template <>
struct ImplicitCurve<SEG_ARC> : ImplicitCurveBase
{
    gpu::math::Matrix<float,3,2> M;

    __device__
    bool negative() const { return flags & CFLAG_NEGATIVE; }

    __device__
    int hit_chull(const float2 &pt) const
    {
        float2 b = begpoint(),
               e = endpoint();

        bool sample_to_the_left = ccw(pt, b, e) < 0;

        if(sample_to_the_left == geom_to_left())
            return -1; // inside ctrl polygon
        else if(out_is_left())
            return 1;
        else
            return 0;
    }

    __device__
    float eval(const float2 &pt) const
    {
        float2 p = pt - begpoint();
        float k = M[0][0]*p.x + M[0][1]*p.y;
        float l = M[1][0]*p.x + M[1][1]*p.y;
        float m = M[2][0]*p.x + M[2][1]*p.y + 1.f;
        float f = k*k - l*m;
        if(negative())
            return -f;
        else
            return f;
    }

    __device__
    int hit(const float2 &pt) const
    {
        int check = bbox.hit(pt);
        if (check == 0)
            return 0; //above, bellow or to the right
        else if (check == 1) // to the left
            return going_up() ? 1 : -1;
        else 
        {
            // float2 b = begpoint(), e = endpoint(); // never used
            check = hit_chull(pt);
            if (check == 0)
                return 0;
            else if(check == 1)
                return going_up() ? 1 : -1;
            else
            {
                if (eval(pt) < 0)
                    return going_up() ? 1 : -1;
            }
        }
        return 0;
    }
};

#endif
