#ifndef GPURASTER_IMPLICITIZE2_H
#define GPURASTER_IMPLICITIZE2_H

#include "util.h" // for find_line_coefficients
#include "icurve2.h"

__device__
void create_quad_matrix( float2 p0, float2 p1, float2 p2, gpu::math::Matrix<float,2,2>& m )
{
    float a = p0.x, b = p0.y;
    float c = p1.x, d = p1.y;
    float e = p2.x, f = p2.y;
    float det_inv = 1.f/(-b*c + a*d + b*e - d*e - a*f + c*f);
    //{{(b - d)/det + (-b + f)/(2 det), (-a + c)/det + (a - e)/(2 det), 0},
    //{(b - d)/det, (-a + c)/det, 0}} 
    m[0][0] = det_inv*(b-d) + det_inv*(-b+f)/2.f; 
    m[0][1] = det_inv*(-a+c) + det_inv*(a-e)/2.f; 
    m[1][0] = det_inv*(b-d); 
    m[1][1] = det_inv*(-a+c); 
}

__device__ void implicitize(const Curve<2> &B, ImplicitCurve<2> &iB,
                            float t0, float t1,
                            const ImplicitizationContext<2> &ctx)
{
    iB.bbox = get_xy_mono_bbox(B);

    assert(iB.flags == 0);

    if(B[0].x < B[2].x)
        iB.flags |= CFLAG_GOING_RIGHT;

    if(B[0].y < B[2].y)
        iB.flags |= CFLAG_GOING_UP;

    // by definition the set { (x,y); f(x,y) < 0 } is the convex subset defined
    // by the quadratic. Assuming that we are dealing with monotonic segments,
    // we can use ccw to find if c is to the left(counterclockwise) of be.  We
    // have two possibles situations: b.y < e.y and othewise.  On the first
    // case, ccw(c,b,e) < 0 implies that c is to the left of be (the segment
    // be) wich implies that the set f<0 is to the right of be. We should then,
    // change sign to -1.
    const float2 &b = B[0], &c = B[1], &e = B[2];

    if(ccw(c,b,e) < 0)
        iB.flags |= CFLAG_GEOM_TO_LEFT;

    if(iB.geom_to_left() == iB.going_up())
        iB.flags |= CFLAG_NEGATIVE;

    if(iB.going_up() != iB.geom_to_left())
        iB.flags |= CFLAG_OUT_IS_LEFT;

    /*
    if((iB.going_up()!=iB.going_right()) != iB.geom_to_left())
        iB.flags |= CFLAG_WHICH_CORNER;

    float2 nc = iB.corner(b,e);

    float4 bar = baricentric_coordinates(c-b, e-b, nc-b);

    iB.u = (0.5f * bar.y + bar.z)/bar.w;
    iB.v = bar.z/bar.w;
    */
    create_quad_matrix( b, c, e, iB.M );
}

#endif
