#ifndef GPURASTER_IMPLICITIZE_ARC_H
#define GPURASTER_IMPLICITIZE_ARC_H

#include <gpu/solve.h>
#include <gpu/util.h>
#include "icurve_arc.h"

template<>
struct __align__(8) ImplicitizationContext<SEG_ARC>
{
};

__device__
void create_matrix( float2 p0, float2 p1, float2 p2, float w, gpu::math::Matrix<float,3,2>& m )
{
    float a = p0.x, b = p0.y;
    float c = p1.x, d = p1.y;
    float e = p2.x, f = p2.y;
    float det_inv = 1.0f/(-b*c + a*d - d*e + c*f + b*e*w - a*f*w);
    // {{(-b + f)/(2 det) + (-d + b w)/det, (a - e)/(2 det) + (c - a w)/det, 0},
    // {(-d + b w)/det, (c - a w)/det, 0},
    // {(-b + f)/det + (-d + b w)/det + (d - f w)/det, (a - e)/det + (c - a w)/det + (-c + e w)/det, 1}}
    m[0][0] = det_inv*((-b+f)/2.f + (-d+b*w));
    m[0][1] = det_inv*((a-e)/2.f + (c-a*w));
    m[1][0] = det_inv*(-d+b*w);
    m[1][1] = det_inv*(c-a*w);
    m[2][0] = det_inv*((-b+f) + (-d+b*w) + (d-f*w));
    m[2][1] = det_inv*((a-e) + (c-a*w) + (-c+e*w));
}

__device__ void implicitize(const Curve<SEG_ARC> &B, ImplicitCurve<SEG_ARC> &iB,
                            float t0, float t1,
                            const ImplicitizationContext<SEG_ARC> &ctx)
{
    iB.bbox = get_xy_mono_bbox(B);

    assert(iB.flags == 0);

    if(B[0].x < B[2].x)
        iB.flags |= CFLAG_GOING_RIGHT;

    if(B[0].y < B[2].y)
        iB.flags |= CFLAG_GOING_UP;

    float2 eb = B[1]/B.w;
    if(ccw(eb, B[0], B[2]) < 0)
        iB.flags |= CFLAG_GEOM_TO_LEFT;

    if(iB.going_up() != iB.geom_to_left())
        iB.flags |= CFLAG_OUT_IS_LEFT;
    else
        iB.flags |= CFLAG_NEGATIVE;
    create_matrix( B[0], B[1], B[2], B.w, iB.M );
}

#endif
