#ifndef GPURASTER_MONOTONIC2_H
#define GPURASTER_MONOTONIC2_H

#include "curve2.h"
#include "segment.h"

namespace detail
{
    __device__ inline
    void update_roots(Roots &roots, unsigned &nroots, float a, float b)
    {
        // is already monotonic?  ( ! (0 < -B/A < 1) )
        if(signbit(a) != signbit(b) && b!=0 && abs(b) < abs(a))
            roots[nroots++] = -b/a;
    }
}

__device__ inline
unsigned calc_roots(const Curve<2> &B, Roots &roots)
{
    // B_2(t) = (1-t)^2*P0 + 2*(1-t)t*P1 + t^2*P2
    // B_2(t)' = 2*(1-t)*(P1 - P0) + 2*t*(P2 - P1)
    // B_2(t)' = 2*(P0 - 2*P1 + P2)*t + 2*(P1 - P0)

    unsigned nroots = 0;

    // At + B == 0
    float2 a = B[0] - 2*B[1] + B[2],
           b = B[1] - B[0];

    detail::update_roots(roots, nroots, a.x, b.x);
    detail::update_roots(roots, nroots, a.y, b.y);

    return nroots;
}

#endif
