#ifndef GPURASTER_MONOTONIC3_H
#define GPURASTER_MONOTONIC3_H

#include "curve3.h"
#include "segment.h"
#include <fd/fontdump.h>

namespace detail
{
    __device__ inline
    void update_roots(Roots &roots, unsigned &nroots,
                      float a, float b, float c)
    {
        float t[2];
        unsigned n = gpu::quadratic_solve(a, b, c, t);

        if(n==2)
        {
            // compare negating condition to avoid having to compare against NaN

            if(!(0 < t[0] && t[0] < 1))
            {
                t[0] = t[1];
                n = 1;
            }
            else if(!(0 < t[1] && t[1] < 1))
                n = 1;
        }

        switch(n)
        {
        case 1:
            if(0 < t[0] && t[0] < 1)
                roots[nroots++] = t[0];
            break;
        case 2:
            roots[nroots++] = t[0];
            roots[nroots++] = t[1];
            break;
        }
    }
}

__device__ inline
unsigned calc_roots(const Curve<3> &B, Roots &roots)
{
    // B_3(t) = (1-t)^3*P0 + 3*(1-t)^2*t*P1 + 3*(1-t)*t^2*P2 + t^3*P3;
    // B_3(t)' = 3*(1-t)^2*P0 + 2*(1-t)*t*P1 + 6*(1-t)*t*P2 + 3*t^2*P3;
    // B_3(t)' = (3*(-P0 + 3*P1 - 3*P2 + P3)*t^2
    //          + 6*(P0 - 2*P1 + P2)*t
    //          + 3*(-P0 + P1)

    float2 a = 3*(-B[0] + 3*B[1] - 3*B[2] + B[3]),
           b = 6*(B[0] - 2*B[1] + B[2]),
           c = 3*(-B[0] + B[1]);

    unsigned nroots = 0;

    detail::update_roots(roots, nroots, a.x, b.x, c.x);
    detail::update_roots(roots, nroots, a.y, b.y, c.y);

    return nroots;
}

#endif
