#ifndef GPURASTER_MONOTONIC_ARC_H
#define GPURASTER_MONOTONIC_ARC_H

#include <gpu/solve.h>
#include "curve_arc.h"
#include "segment.h"

namespace detail
{
    __device__ inline
    void update_roots(Roots &roots, unsigned &nroots,
                      float a, float b, float w, float c)
    {
        // wxB - wB xA + t (-2 wxB - xA + 2 wB xA + xC) + t^2 (xA - wB xA - xC + wB xC), 

        float A = (a - w*a) - (c - w*c),
              B = (-2*b - a + 2*w*a + c)/2,
              C = b - w*a;

        int exp;
        float delta = gpu::discriminant(A, B, C, exp);
        if(delta >= 0)
        {
            float t[2], s[2];
            gpu::quadratic_solve(A, B, C, delta, exp, t, s);
#pragma unroll
            for(int i=0; i<2; ++i)
            {
                if((0 < t[i] && t[i] < s[i]) || (s[i] < t[i] && t[i] < 0))
                    roots[nroots++] = t[i]/s[i];
            }
        }
    }
}

__device__ inline
unsigned calc_roots(const Curve<SEG_ARC> &B, Roots &roots)
{
    unsigned nroots = 0;

    detail::update_roots(roots, nroots, B[0].x, B[1].x, B.w, B[2].x);
    detail::update_roots(roots, nroots, B[0].y, B[1].y, B.w, B[2].y);

    return nroots;
}

#endif
