#ifndef GPURASTER_PERSISTENT_H
#define GPURASTER_PERSISTENT_H

#include <gpu/symbol.h>
#include "persistent_config.h"

__constant__ unsigned global_pool_warp_count;
__device__ unsigned global_pool_next_warp;

template<int NB, class R>
__launch_bounds__(32*PERSISTENT_WARPS, NB)
__global__ void gpu_persistent_threads(R rasterizer)
{
    __shared__ volatile unsigned idwarp[PERSISTENT_WARPS];

    while(true)
    {
        if(threadIdx.x == 0)
            idwarp[threadIdx.y] = atomicAdd(&global_pool_next_warp, 1);

        if(idwarp[threadIdx.y] >= global_pool_warp_count)
            return;

        rasterizer(idwarp[threadIdx.y]);
    }
}

template <int NB, class R>
void persistent_threads(const R &rasterizer, unsigned packets_count)
{
    gpu::copy_to_symbol(global_pool_warp_count, packets_count);
    gpu::copy_to_symbol(global_pool_next_warp, 0u);

    dim3 bdim(32, PERSISTENT_WARPS),
         gdim(PERSISTENT_BLOCKS);

    gpu_persistent_threads<NB><<<gdim,bdim>>>(rasterizer);
}

template<int NB, int num_warps, class R>
//__launch_bounds__(32*num_warps, NB)
__global__ void gpu_persistent_threads(R rasterizer)
{
    //__shared__ volatile unsigned idwarp[num_warps];
    __shared__ volatile unsigned idwarp;

    while(true)
    {
        if(threadIdx.x == 0 && threadIdx.y == 0)
            idwarp = atomicAdd(&global_pool_next_warp, 1);
        __syncthreads();

        if(idwarp >= global_pool_warp_count)
            return;

        rasterizer(idwarp);
    }
}

template <int NB, int num_warps, class R>
void persistent_threads(const R &rasterizer, unsigned packets_count)
{
    gpu::copy_to_symbol(global_pool_warp_count, packets_count);
    gpu::copy_to_symbol(global_pool_next_warp, 0u);

    dim3 bdim(32, num_warps),
         //gdim(PERSISTENT_BLOCKS);
         gdim(56);

    gpu_persistent_threads<NB, num_warps><<<gdim,bdim>>>(rasterizer);
}



#endif
