#ifndef GPURASTER_RASTER_H
#define GPURASTER_RASTER_H

#include <gpu/image.h>
#include <image/image.h>
#include <image/color.h>
#include <fd/fontdump.h>
#include "warps.h"

struct Ramp;
struct Quads;
struct Segments;
struct TraversalInfo;
struct Transform;
struct Projective;

template <int N> struct ImplicitSegment;

void rasterize_shared(const float4 &bg_color,
               const Quads &qtree,
               const Segments &segs,
               unsigned nsamples,
               gpu::image_ptr<float4> out,
               unsigned flags,
               unsigned use_bs3);

template<class W>
void rasterize_warp(const W& warp, const float4 &bg_color,
               const Quads &qtree,
               const Segments &segs,
               unsigned nsamples,
               gpu::image_ptr<float4> out,
               unsigned flags,
               unsigned use_bs3);

template<>
void rasterize_warp(const NoWarp& nowarp, const float4 &bg_color,
               const Quads &qtree,
               const Segments &segs,
               unsigned nsamples,
               gpu::image_ptr<float4> out,
               unsigned flags,
               unsigned use_bs3);

template<>
void rasterize_warp(const ProjectiveWarp& fs, const float4 &bg_color,
               const Quads &qtree,
               const Segments &segs,
               unsigned nsamples,
               gpu::image_ptr<float4> out,
               unsigned flags,
               unsigned use_bs3);
template<>
void rasterize_warp(const LensWarp& lens, const float4 &bg_color,
               const Quads &qtree,
               const Segments &segs,
               unsigned nsamples,
               gpu::image_ptr<float4> out,
               unsigned flags,
               unsigned use_bs3);

template<>
void rasterize_warp(const SwirlWarp& swirl, const float4 &bg_color,
               const Quads &qtree,
               const Segments &segs,
               unsigned nsamples,
               gpu::image_ptr<float4> out,
               unsigned flags,
               unsigned use_bs3);

/*
void rasterize(image::rgba<> &out, const fd::FontDump &fd,
               unsigned nsamples = 16, unsigned qtree_levels = 6,
               unsigned min_segs_split=10,
               const image::Color &bg = image::Color(0,0,0,0));
               */
#endif
