#ifndef GPURASTER_SEGMENT_H
#define GPURASTER_SEGMENT_H

#include "pathdata.h"
#include "curve.h"
#include "segtype.h"

template <int N=0, class EN=void>
struct Segment
{
    // we need this to be layout compatible with Segment<0>
    SegType type;

    Curve<N> *B;

    void set_B_address(void *data)
    {
        B = reinterpret_cast<Curve<N> *>(data);
    }

    HOSTDEV float2 &operator[](int i) { return (*B)[i]; }
    HOSTDEV const float2 &operator[](int i) const { return (*B)[i]; }
    HOSTDEV static size_t size() { return Curve<N>::size(); }
};

template<>
struct Segment<SEG_PATH>
{
    SegType type;
    PathData *path;

    HOSTDEV static size_t size() { return sizeof(PathData); }
};


template<>
struct Segment<0>
{
    SegType type;
    void *data;

    template <int N>
    HOSTDEV Segment<N> &as()
    {
        return *reinterpret_cast<Segment<N> *>(this);
    }

    template <int N>
    HOSTDEV const Segment<N> &as() const
    {
        return *reinterpret_cast<const Segment<N> *>(this);
    }

    HOSTDEV size_t size() const
    {
        if(is_stack(type))
            return 0;

        switch(type)
        {
        case SEG_LINEAR:
            return Segment<1>::size();
        case SEG_QUADRATIC:
            return Segment<2>::size();
        case SEG_CUBIC:
            return Segment<3>::size();
        case SEG_ARC:
            return Segment<SEG_ARC>::size();
        case SEG_PATH:
            return Segment<SEG_PATH>::size();
        default:
            return 0;
        }
    }
};

#endif
