#ifndef SEGMENT_UTIL_H
#define SEGMENT_UTIL_H

__device__
inline int find_segment_size( const SegType& type )
{/*{{{*/
    //return 64;
    int s = 0;
    switch( type ) {
        case SEG_LINEAR:
            s = sizeof(ImplicitCurve<1>);
            break;
        case SEG_QUADRATIC:
            s = sizeof(ImplicitCurve<2>);
            break;
        case SEG_CUBIC:
            s = sizeof(ImplicitCurve<3>);
            break;
        case SEG_ARC:
            s = sizeof(ImplicitCurve<SEG_ARC>);
            break;
        case SEG_PATH:
            s = sizeof(PathData);
            break;
        case SEG_PUSH:
        case SEG_POP_FILL:
        case SEG_POP_CLIP:
        case SEG_COMMIT:
        case SEG_POINT:
            //TODO: change this to 0...
            s = 16;
            break;
        default:
            //assert(false);
            break;
    }
    if ( s & 0x0F ) {
        s &= ~0x0F;
        s += 16;
    }
    return s;
}/*}}}*/

__device__
inline int find_paint_size( const Paint::Type& type )
{/*{{{*/
    int s = 0;
    switch ( type ) {
        case Paint::SOLID:
            s = sizeof(PaintSolid);
            break;
        case Paint::SOLID_ALPHA:
            s = sizeof(PaintSolidAlpha);
            break;
        case Paint::GRAD_LINEAR:
            s = sizeof(PaintLinearGrad);
            break;
        case Paint::GRAD_RADIAL:
            s = sizeof(PaintRadialGrad);
            break;
        case Paint::TEXTURE:
            s = sizeof(PaintTexture);
            break;
        default:
            break;
    }
    if ( s & 0x0F ) {
        s &= ~0x0F;
        s += 16;
    }
    return s;
}/*}}}*/

__device__
inline int round_to_m16( int a )
{/*{{{*/
    if ( a & 0xF ) {
        a &= ~0xF;
        a += 16;
    }
    return a;
}/*}}}*/



#endif
