#ifndef GPURASTER_SEGTYPE_H
#define GPURASTER_SEGTYPE_H

enum SegType
{
    SEG_POINT,
    SEG_LINEAR,
    SEG_QUADRATIC,
    SEG_CUBIC,
    SEG_ARC,
    SEG_PATH,
    SEG_LAST_GEOM = SEG_PATH,

    SEG_FIRST_STACK,
    SEG_PUSH = SEG_FIRST_STACK,
    SEG_POP_FILL,
    SEG_POP_CLIP,
    SEG_COMMIT,
    SEG_LAST_STACK = SEG_COMMIT
};

inline std::ostream &operator<<(std::ostream &out, const SegType &type)
{
    switch(type)
    {
    case SEG_POINT: return out <<  "point";
    case SEG_LINEAR: return out <<  "linear";
    case SEG_QUADRATIC: return out <<  "quadratic";
    case SEG_CUBIC: return out <<  "cubic";
    case SEG_ARC: return out <<  "arc";
    case SEG_PATH: return out <<  "path";
    case SEG_PUSH: return out <<  "(";
    case SEG_POP_CLIP: return out <<  ")";
    case SEG_POP_FILL: return out <<  ")";
    case SEG_COMMIT: return out <<  "|";
    default: return out <<  "unknown";
    }
}

template <SegType type>
struct meta_is_stack
{
    static const bool value = type >= SEG_FIRST_STACK && type <= SEG_LAST_STACK;
};

HOSTDEV inline bool is_stack(SegType type)
{
    return type >= SEG_FIRST_STACK && type <= SEG_LAST_STACK;
}

inline unsigned size(SegType type)
{
    switch( type )
    {
    case SEG_POINT:
    case SEG_LINEAR:
    case SEG_PATH:
        return 1;
    case SEG_QUADRATIC:
        return 2;
    case SEG_CUBIC:
    case SEG_ARC:
        return 3;
    default:
        return 0;
    }
}

#endif
