#include "texture.h"
#include <gpu/error.h>


Texture convert_texture( const std::vector<float>& img, unsigned width, unsigned height, const fd::Spread spread )
{
    Texture out;
    //
    cudaChannelFormatDesc channelDesc = cudaCreateChannelDesc(32, 32, 32, 32, cudaChannelFormatKindFloat);
    cudaArray* cuArray;
    cudaMallocArray(&cuArray, &channelDesc, width, height );
    gpu::check_cuda_error("Allocating cudaArray");
    //
    cudaMemcpyToArray(cuArray, 0, 0, &img.front(), 4*width*height*sizeof(float), cudaMemcpyHostToDevice);
    gpu::check_cuda_error("Copying texture to cuda array");
    //
    cudaResourceDesc res_desc;
    memset(&res_desc, 0, sizeof(res_desc));
    res_desc.resType = cudaResourceTypeArray;
    res_desc.res.array.array = cuArray;
    //
    cudaTextureDesc tex_desc;
    memset(&tex_desc, 0, sizeof(tex_desc));
    tex_desc.filterMode = cudaFilterModeLinear;
    tex_desc.readMode = cudaReadModeElementType;
    // must use normalized coords so that Mirror and Wrap address modes
    // work
    tex_desc.normalizedCoords = true;
    switch(spread)
    {
    case fd::SPREAD_PAD:
        tex_desc.addressMode[0] = cudaAddressModeClamp;
        tex_desc.addressMode[1] = cudaAddressModeClamp;
        break;
    case fd::SPREAD_REPEAT:
        tex_desc.addressMode[0] = cudaAddressModeWrap;
        tex_desc.addressMode[1] = cudaAddressModeWrap;
        break;
    case fd::SPREAD_REFLECT:
        tex_desc.addressMode[0] = cudaAddressModeMirror;
        tex_desc.addressMode[1] = cudaAddressModeMirror;
        break;
    default:
        throw std::runtime_error("Invalid spread type");
    }
    //
    cudaCreateTextureObject(&out.m_tex, &res_desc, &tex_desc, NULL);
    gpu::check_cuda_error("Creating cuda texture");
    //
    return out;
}
