#ifndef GPURASTER_TRAVERSAL_H
#define GPURASTER_TRAVERSAL_H

#include <gpu/vector.h>

struct TraversalInfo
{
    int idquad;
    ushort2 pix;
};

struct TraversalInfosPtr
{
    TraversalInfosPtr() : size(0), idquad(NULL), pix(NULL) {}

    unsigned size;
    unsigned *idquad;
    ushort2 *pix;


    __device__
    void get(int idx, TraversalInfo &tinfo) const
    {
        tinfo.idquad = idquad[idx];
        tinfo.pix = pix[idx];
    }

    __device__
    void put(int idx, const TraversalInfo &tinfo)
    {
        idquad[idx] = tinfo.idquad;
        pix[idx] = tinfo.pix;
    }
};

struct const_TraversalInfosPtr
{
    const_TraversalInfosPtr() : size(0), idquad(NULL), pix(NULL) {}
    const_TraversalInfosPtr(const TraversalInfosPtr &tinfo)
        : size(tinfo.size)
        , idquad(tinfo.idquad)
        , pix(tinfo.pix) {}

    unsigned size;
    const unsigned *idquad;
    const ushort2 *pix;

    __device__
    void get(int idx, TraversalInfo &tinfo) const
    {
        tinfo.idquad = idquad[idx];
        tinfo.pix = pix[idx];
    }
};

struct TraversalInfos
{
    gpu::vector<unsigned> idquad;
    gpu::vector<ushort2> pix;

    size_t size() const { return pix.size(); }
    void clear()
    {
        idquad.clear();
        pix.clear();
    }
    void resize(size_t s)
    {
        idquad.resize(s);
        pix.resize(s);
    }

    TraversalInfosPtr operator&()
    {
        TraversalInfosPtr tinfo;
        tinfo.size = size();
        tinfo.idquad = idquad.data();
        tinfo.pix = pix.data();
        return tinfo;
    }

    const_TraversalInfosPtr operator&() const
    {
        const_TraversalInfosPtr tinfo;
        tinfo.size = size();
        tinfo.idquad = idquad.data();
        tinfo.pix = pix.data();
        return tinfo;
    }
};

struct Quads;

void traverse_qtree(TraversalInfos &tinfo,
                    const Quads &qtree,
                    unsigned imgwidth, unsigned imgheight,
                    float coord_adj);

void sort(TraversalInfos &tinfos);


#endif
