#ifndef RVG_DRIVER_SVG_H
#define RVG_DRIVER_SVG_H

#include <cstdio>
#include <string>
#include "base.h"

namespace rvg { namespace driver {

class SVG final: public Base {
public:

    void render(Scene source_scene, const Viewport &viewport, FILE *out);

protected:

    Image
    do_new_image(FILE *file) override; 

    Image
    do_new_image(const char *str, size_t len) override;

    Surface
    do_new_surface(int width, int height) override;

    paint::Ramp
    do_new_color_ramp(SpreadMethod spread,
                      const std::vector<paint::ramp::Stop> &stops) override;

    Scene
    do_new_scene(void) override;

    Paint
    do_new_solid_paint(const Color &color, float opacity = 1.f) override;

    Paint
    do_new_linear_gradient(const paint::Ramp ramp,
        r2::Point p1, r2::Point p2,
        Transform xf = xform::Identity{}, float opacity = 1.f) override;

    Paint
    do_new_radial_gradient(const paint::Ramp ramp,
        r2::Point c, r2::Point f, float r,
        Transform xf = xform::Identity{}, float opacity = 1.f) override;

    Paint
    do_new_texture(Image image, SpreadMethod spread_method,
        Transform xf = xform::Identity{}, float opacity = 1.f) override;

    Path
    do_new_path(void) override;

}; // class rvg::svg::Driver

}} // namespace rvg::svg

#endif // RVG_DRIVER_SVG_H
