//#include "../path/concrete.h"
#include "../scene/scene.h"

namespace rvg { namespace driver
{

using etype = scene::Element::Type;
template< class Iterator >
    void reverse_stream( Iterator begin, Iterator end, std::vector<scene::Element>& elements );

template< class Iterator >
    void reverse_bracket( Iterator begin, Iterator end, std::vector<scene::Element>& elements, etype tpush, etype tpop )
    {/*{{{*/
        auto it = begin, cit = begin;
        const auto& e = *begin;
        int depth = e.stack.depth;
        do {
            cit = ++it;
        }
        while( it != end && it->type != etype::commit );
        elements.push_back( scene::Element(tpush, depth) );
        reverse_stream( cit+1, end-1, elements );
        elements.push_back( scene::Element(etype::commit, depth) );
        reverse_stream( begin+1, cit, elements );
        elements.push_back( scene::Element(tpop, depth) );
    }/*}}}*/

template< class Iterator >
    void reverse_stream( Iterator begin, Iterator end, std::vector<scene::Element>& elements )
    {/*{{{*/
        for ( auto it = begin; it != end; ) {
            const auto& e = *it;
            switch( e.type ) {
                case scene::Element::Type::popfill:
                case scene::Element::Type::popclip:
                    {
                        int depth = e.stack.depth;
                        auto bit = it;
                        do { ++it; }
                        while( !((it->type == etype::pushfill || it->type == etype::pushclip) && it->stack.depth == depth )
                                && it != end );
                        assert( (it->type == etype::pushfill || it->type == etype::pushclip) && it->stack.depth == depth );
                        ++it; //skip to next element
                        if ( e.type == etype::popfill )
                            reverse_bracket( bit, it, elements, etype::pushfill, etype::popfill );
                        else
                            reverse_bracket( bit, it, elements, etype::pushclip, etype::popclip );
                    }
                    break;
                case scene::Element::Type::pushfill:
                case scene::Element::Type::pushclip:
                case scene::Element::Type::commit:
                    assert(0);
                    break;
                default:
                    {
                        elements.push_back( *it );
                        ++it;
                    }
            }
        }
    }/*}}}*/

void reverse_stream( const Scene& scene, std::vector<scene::Element>& elements )
{/*{{{*/
    reverse_stream( scene.elements().rbegin(), scene.elements().rend(), elements );
}/*}}}*/

}} // namespace rvg::driver
