#ifndef RVG_IMAGE_BASE_H
#define RVG_IMAGE_BASE_H

#include "../util/cloneable.h"

namespace rvg { namespace image {

class Base: public Cloneable<Base> {
public:
    virtual ~Base() { }

    // Pitch is the distance between consecutive rows (in floats)
    // Advance is the distance between consecutive elements (in floats)
    void load(int width, int height, const float *red, const float *green,
            const float *blue, const float *alpha, int pitch, int advance) {
        return do_load(width, height, red, green, blue, alpha, pitch, advance);
    }

    // Pitch is the distance between consecutive rows (in unsigned shorts)
    // Advance is the distance between consecutive elements (in unsigned shorts)
    void load(int width, int height, const unsigned short *red,
            const unsigned short *green, const unsigned short *blue,
            const unsigned short *alpha, int pitch, int advance) {
        return do_load(width, height, red, green, blue, alpha, pitch, advance);
    }

    // Pitch is the distance between consecutive rows (in unsigned chars)
    // Advance is the distance between consecutive elements (in unsigned chars)
    void load(int width, int height, const unsigned char *red,
            const unsigned char *green, const unsigned char *blue,
            const unsigned char *alpha, int pitch, int advance) {
        return do_load(width, height, red, green, blue, alpha, pitch, advance);
    }


    // Pitch is the distance between consecutive rows (in floats)
    // Advance is the distance between consecutive elements (in floats)
    void store(int width, int height, float *red, float *green,
            float *blue, float *alpha, int pitch, int advance) const {
        return do_store(width, height, red, green, blue, alpha, pitch, advance);
    }

    // Pitch is the distance between consecutive rows (in unsigned shorts)
    // Advance is the distance between consecutive elements (in unsigned shorts)
    void store(int width, int height, unsigned short *red,
            unsigned short *green, unsigned short *blue,
            unsigned short *alpha, int pitch, int advance) const {
        return do_store(width, height, red, green, blue, alpha, pitch, advance);
    }

    // Pitch is the distance between consecutive rows (in unsigned chars)
    // Advance is the distance between consecutive elements (in unsigned chars)
    void store(int width, int height, unsigned char *red,
            unsigned char *green, unsigned char *blue,
            unsigned char *alpha, int pitch, int advance) const {
        return do_store(width, height, red, green, blue, alpha, pitch, advance);
    }

    void resize(int width, int height) {
        return do_resize(width, height);
    }

    int width(void) const {
        return do_width();
    }

    int height(void) const {
        return do_height();
    }

protected:

    virtual void do_load(int width, int height, const float *red,
            const float *green, const float *blue, const float *alpha,
            int pitch, int advance) = 0;

    virtual void do_load(int width, int height, const unsigned short *red,
            const unsigned short *green, const unsigned short *blue,
            const unsigned short *alpha, int pitch, int advance) = 0;

    virtual void do_load(int width, int height, const unsigned char *red,
            const unsigned char *green, const unsigned char *blue,
            const unsigned char *alpha, int pitch, int advance) = 0;

    virtual void do_store(int width, int height, float *red,
            float *green, float *blue, float *alpha,
            int pitch, int advance) const = 0;

    virtual void do_store(int width, int height, unsigned short *red,
            unsigned short *green, unsigned short *blue,
            unsigned short *alpha, int pitch, int advance) const = 0;

    virtual void do_store(int width, int height, unsigned char *red,
            unsigned char *green, unsigned char *blue,
            unsigned char *alpha, int pitch, int advance) const = 0;

    virtual int do_width(void) const = 0;
    virtual int do_height(void) const = 0;

    virtual void do_resize(int width, int height) = 0;
};

} } // namespace rvg::image

#endif // RVG_IMAGE_BASE_H
