#ifndef RVG_IMAGE_CONCRETE_H
#define RVG_IMAGE_CONCRETE_H

#include <vector>
#include "image.h"

namespace rvg { namespace image {

class Concrete: public Base {
public:
    Concrete(void): m_width(0), m_height(0) { }

    const std::vector<float> &red(void) const { return m_red; }
    const std::vector<float> &green(void) const { return m_green; }
    const std::vector<float> &blue(void) const { return m_blue; }
    const std::vector<float> &alpha(void) const { return m_alpha; }

    virtual ~Concrete() { }

    IMPLEMENT_CLONEABLE(Concrete);

    friend std::ostream &operator<<(std::ostream &out, const Concrete &img);

protected:

    template <typename T, typename C> void do_load(int width, int height,
            const T *red, const T *green, const T *blue, const T *alpha,
            int pitch, int advance, const C &convert);

    template <typename T, typename C> void do_store(int width, int height,
            T *red, T *green, T *blue, T *alpha,
            int pitch, int advance, const C &convert) const;

    void do_load(int width, int height, const float *red,
            const float *green, const float *blue, const float *alpha,
            int pitch, int advance) override;

    void do_load(int width, int height, const unsigned short *red,
            const unsigned short *green, const unsigned short *blue,
            const unsigned short *alpha, int pitch, int advance) override;

    void do_load(int width, int height, const unsigned char *red,
            const unsigned char *green, const unsigned char *blue,
            const unsigned char *alpha, int pitch, int advance) override;

    void do_store(int width, int height, float *red,
            float *green, float *blue, float *alpha,
            int pitch, int advance) const override;

    void do_store(int width, int height, unsigned short *red,
            unsigned short *green, unsigned short *blue,
            unsigned short *alpha, int pitch, int advance) const override;

    void do_store(int width, int height, unsigned char *red,
            unsigned char *green, unsigned char *blue,
            unsigned char *alpha, int pitch, int advance) const override;

    int do_width(void) const override { return m_width; }

    int do_height(void) const override { return m_height; }

    void do_resize(int width, int height) override;

private:
    int m_width, m_height;
    std::vector<float> m_red, m_green, m_blue, m_alpha;
};

std::ostream &operator<<(std::ostream &out, const Concrete &img);

} }  // namespace rvg::image

#include "concrete.hpp"

#endif // RVG_IMAGE_CONCRETE_H
