namespace rvg { namespace image {

template <typename T, typename C>
void Concrete::do_load(int width, int height,
    const T *red, const T *green, const T *blue, const T *alpha,
    int pitch, int advance, const C &convert) {
    resize(width, height);
    if (red && green && blue && alpha) {
        for (int i = 0; i < height; i++) {
            int offset = 0;
            for (int j = 0; j < width; j++) {
                int index = i*width+j;
                m_red[index] = convert(red[offset]);
                m_green[index] = convert(green[offset]);
                m_blue[index] = convert(blue[offset]);
                m_alpha[index] = convert(alpha[offset]);
                offset += advance;
            }
            red += pitch;
            green += pitch;
            blue += pitch;
            alpha += pitch;
        }
    }
}

template <typename T, typename C>
void Concrete::do_store(int width, int height, T *red, T *green, T *blue,
    T *alpha, int pitch, int advance, const C &convert) const {
    assert(width == m_width && height == m_height);
    if (red && green && blue && alpha) {
        for (int i = 0; i < height; i++) {
            int offset = 0;
            for (int j = 0; j < width; j++) {
                int index = i*width+j;
                red[offset] = convert(m_red[index]);
                green[offset] = convert(m_green[index]);
                blue[offset] = convert(m_blue[index]);
                alpha[offset] = convert(m_alpha[index]);
                offset += advance;
            }
            red += pitch;
            green += pitch;
            blue += pitch;
            alpha += pitch;
        }
    }
}

} }  // namespace rvg::image
