#ifndef RVG_IMAGE_H
#define RVG_IMAGE_H

#include "../util/cow.h"
#include "base.h"

namespace rvg { namespace image {

class Image final {
public:

    Image() = default;

    explicit Image(Base *base): m_impl(base) { }

    void load(int width, int height, const float *red, const float *green,
            const float *blue, const float *alpha, int pitch, int advance) {
        return m_impl.rw()->load(width, height, red, green, blue, alpha,
                pitch, advance);
    }

    void load(int width, int height, const unsigned short *red,
            const unsigned short *green, const unsigned short *blue,
            const unsigned short *alpha, int pitch, int advance) {
        return m_impl.rw()->load(width, height, red, green, blue, alpha,
                pitch, advance);
    }

    void load(int width, int height, const unsigned char *red,
            const unsigned char *green, const unsigned char *blue,
            const unsigned char *alpha, int pitch, int advance) {
        return m_impl.rw()->load(width, height, red, green, blue, alpha,
                pitch, advance);
    }

    void store(int width, int height, float *red, float *green,
            float *blue, float *alpha, int pitch, int advance) const {
        return m_impl->store(width, height, red, green, blue, alpha,
                pitch, advance);
    }

    const Base *ro() const { return m_impl.ro(); }

    int width(void) const { return m_impl->width(); }
    int height(void) const { return m_impl->height(); }

private:
    COW<Base> m_impl;
};

} // namespace rvg::image

using image::Image;

} // namespace rvg

#endif // RVG_IMAGE_H
