#ifndef RVG_ITERATOR_BASE_H
#define RVG_ITERATOR_BASE_H

#include <boost/iterator/iterator_facade.hpp>

namespace rvg { namespace iterator {

// internal abstract base iterator
template <typename ValueType, typename RefType = ValueType &>
class Internal {
public:
    typedef ValueType value_type;
    typedef RefType reference;
    virtual ~Internal() {}
    virtual int index(void) const = 0;
    virtual void increment(void) = 0;
    virtual bool equal(const Internal *other) const = 0;
    virtual reference dereference(void) const = 0;
    virtual Internal *clone(void) const = 0;
};

// concrete iterator
template <typename InternalType>
class Iterator final: public boost::iterator_facade<Iterator<InternalType>,
        typename InternalType::value_type, boost::forward_traversal_tag,
        typename InternalType::reference> {
public:
    explicit Iterator(InternalType *base): m_base(base) {}

    ~Iterator() { delete m_base; }

    Iterator(const Iterator &other): m_base(other.m_base->clone()) {}

    const Iterator &operator=(const Iterator &other) {
        m_base = other.m_base->clone();
        return *this;
    }

    int index(void) const {
        return m_base->index();
    }

private:
    friend class boost::iterator_core_access;

    void increment(void) {
        return m_base->increment();
    }

    bool equal(const Iterator<InternalType> &other) const {
        return m_base->equal(other.m_base);
    }

    // here I would like to use simply "reference" as a
    // type, but it looks as though iterator_facade is *not*
    // defining reference or reference_type. :( ??
    typename InternalType::reference dereference(void) const {
        return m_base->dereference();
    }

    InternalType *m_base;
};

}} // namespace rvg::iterator

#endif // RVG_ITERATOR_BASE
