namespace rvg { namespace lua {

template <typename T>
static void push_const(lua_State *L, Base::Type type, T value) {
    lua_pushlightuserdata(L, reinterpret_cast<void *>(type)); // &type
    lua_rawget(L, lua_upvalueindex(1)); // type
    lua_rawgeti(L, -1, static_cast<int>(value)); // type,value
    lua_replace(L, -2); // value
}

template <typename T>
static void open_consts(lua_State *L, Base::Type type,
    const const_Reg<T> *entries) {
    const luaL_Reg const_meta[] = {
        {"__tostring", meta_tostring_const},
        {nullptr, nullptr}
    };
    // package,consts
    open_type(L, nullptr, type, nullptr, const_meta);
    lua_rawgeti(L, -1, static_cast<int>(type)); // package,consts,meta
    lua_pushstring(L, Base::name(type)); // package,consts,meta,"type"
    lua_newtable(L); // package,consts,meta,"type",type
    lua_newtable(L); // package,consts,meta,"type",type, typemeta
    lua_pushliteral(L, "__index");
    lua_pushvalue(L, -4);
    lua_pushcclosure(L, meta_index_const, 1);
    lua_rawset(L, -3);
    lua_pushliteral(L, "__newindex");
    lua_pushvalue(L, -4);
    lua_pushcclosure(L, meta_newindex_const, 1);
    lua_rawset(L, -3);
    lua_setmetatable(L, -2);
    for (int i = 0; entries[i].name; i++) {
        lua_pushstring(L, entries[i].name);
        // package,consts,meta,"type",type,name
        T *c = static_cast<T *>(lua_newuserdata(L, sizeof(T)));
        // package,consts,meta,"type",type,name,obj
        *c = entries[i].value;
        lua_pushvalue(L, -5); // package,consts,meta,"type",type,name,obj,meta
        lua_setmetatable(L, -2); // package,consts,meta,"type",type,name,obj
        lua_pushinteger(L, static_cast<int>(entries[i].value));
        lua_pushvalue(L, -2);
        // package,consts,meta,"type",type,name,obj,value,obj
        lua_rawset(L, -5);
        // package,consts,meta,"type",type,name,obj
        lua_pushvalue(L, -1);
        lua_pushvalue(L, -3);
        // package,consts,meta,"type",type,name,obj,obj,name
        lua_rawset(L, -8); 
        // package,consts,meta,"type",type,name,obj
        lua_rawset(L, -3); // package,consts,meta,"type",type
    }
    // package,consts,meta,"type",type
    lua_pushlightuserdata(L, reinterpret_cast<void *>(type));
    lua_pushvalue(L, -2);
    // package,consts,meta,"type",type,&type,type
    lua_rawset(L, -6);
    // package,consts,meta,"type",type
    lua_rawset(L, -5); // package,consts,meta
    lua_pop(L, 1); // package,consts
}

template <typename T> 
T check_const(lua_State *L, int idx, Base::Type type, 
    const char *msg) {
    if (!is_type(L, idx, type)) luaL_argerror(L, idx, msg);
    return *static_cast<T *>(lua_touserdata(L, idx));
}

}} // namespace rvg::lua
