#include "compat.h"

namespace rvg { namespace lua {

int abs_index(lua_State *L, int idx) {
#if LUA_VERSION_NUM > 501
    return lua_absindex(L, idx);
#else
    return idx < 0? lua_gettop(L) + idx + 1: idx;
#endif
}

FILE* check_file(lua_State *L, int idx) {
#if LUA_VERSION_NUM > 501
    luaL_Stream *ls = (luaL_Stream *) luaL_checkudata(L, idx, LUA_FILEHANDLE);
    if (ls->closef == NULL) luaL_argerror(L, idx, "file is closed");
    return ls->f;
#else
    FILE **fpp=(FILE **)luaL_checkudata(L, idx, LUA_FILEHANDLE);
    if (!*fpp) luaL_argerror(L, idx, "file is closed");
    return *fpp;
#endif
}

bool is_equal(lua_State *L, int idx1, int idx2) {
#if LUA_VERSION_NUM > 501
    return lua_compare(L, idx1, idx2, LUA_OPEQ);
#else
    return lua_equal(L, idx1, idx2);
#endif
}

int table_len(lua_State *L, int idx) {
#if LUA_VERSION_NUM > 501
    return static_cast<int>(lua_rawlen(L, idx));
#else
    return static_cast<int>(lua_objlen(L, idx));
#endif
}

#if LUA_VERSION_NUM < 502
// copied from Lua 5.2
void luaL_setfuncs (lua_State *L, const luaL_Reg *l, int nup) {
  luaL_checkstack(L, nup, "too many upvalues");
  for (; l->name != NULL; l++) {  /* fill the table with given functions */
    int i;
    for (i = 0; i < nup; i++)  /* copy upvalues to the top */
      lua_pushvalue(L, -nup);
    lua_pushcclosure(L, l->func, nup);  /* closure with those upvalues */
    lua_setfield(L, -(nup + 2), l->name);
  }
  lua_pop(L, nup);  /* remove upvalues */
}
#endif

}}
