#include <cstdio>
#include "cpu.h"

namespace rvg { namespace lua {

void
CPU::do_close(lua_State *L) {
    // clear lua::CPU from registry to allow for collection
    lua_pushlightuserdata(L, driver());
    lua_pushnil(L);
    lua_settable(L, LUA_REGISTRYINDEX);
}

static int render(lua_State *L) {
    Scene *s = check_scene(L, 1);
    Transform *xf = check_xform(L, 2);
    const char *fname = lua_tostring(L, 3);
    if(fname == NULL)
        throw std::runtime_error("Invalid parameter 3 to cpu::render");

    Viewport *v = check_viewport(L, 4);
    driver::CPU *cpu = static_cast<driver::CPU *>(get_driver(L));
    cpu->render(*s, *xf, fname, *v);
    return 0;
}

void
CPU::do_open(lua_State *L) {
    Base::do_open(L); // package,consts
    lua_pushliteral(L, "render"); // package,consts,"render"
    lua_pushvalue(L, -2); //package,consts,"render",consts
    lua_pushlightuserdata(L, driver());//package,consts,"render",consts,drv
    lua_pushcclosure(L, render, 2);//package,consts,"render",render
    lua_rawset(L, -4);
}

}}

static int delete_cpu(lua_State *L) {
    rvg::lua::CPU *p = static_cast<rvg::lua::CPU *>(lua_touserdata(L,1));
    p->~CPU();
    return 0;
}

extern "C"
__attribute__((visibility("default")))
int luaopen_rvg10_cpu(lua_State *L) {
    // create userdata holding lua::CPU object
    rvg::lua::CPU *p = static_cast<rvg::lua::CPU *>(
        lua_newuserdata(L, sizeof(rvg::lua::CPU))); // cpu
    new (p) rvg::lua::CPU;
    // set metatable with method for garbage collection
    lua_newtable(L); // cpu,meta
    lua_pushliteral(L, "__gc"); // cpu,meta,"gc"
    lua_pushcfunction(L, delete_cpu); // cpu,meta,"gc",delete
    lua_rawset(L, -3); // cpu,meta
    lua_setmetatable(L, -2); // cpu
    // store it in registry keyed by driver address
    lua_pushlightuserdata(L, p->driver()); // cpu,driver
    lua_pushvalue(L, -2); // cpu,driver,cpu
    lua_settable(L, LUA_REGISTRYINDEX); // cpu
    lua_pop(L, 1); //
    // initialize Lua binding
    lua_newtable(L); // package
    lua_newtable(L); // package,consts
    p->open(L); // package,consts
    lua_pop(L, 1);
    return 1;
}
