#include <cstdio>
#include "svg.h"
#include "compat.h"

namespace rvg { namespace lua {

void
SVG::do_close(lua_State *L) {
    // clear lua::SVG from registry to allow for collection
    lua_pushlightuserdata(L, driver());
    lua_pushnil(L);
    lua_settable(L, LUA_REGISTRYINDEX);
}

static int render(lua_State *L) {
    Scene *s = check_scene(L, 1);
    Viewport *v = check_viewport(L, 2);
    FILE *f = check_file(L, 3);
    driver::SVG *svg = static_cast<driver::SVG *>(get_driver(L));
    svg->render(*s, *v, f);
    return 0;
}

void
SVG::do_open(lua_State *L) {
    Base::do_open(L); // package,consts
    lua_pushliteral(L, "render"); // package,consts,"render"
    lua_pushvalue(L, -2); //package,consts,"render",consts
    lua_pushlightuserdata(L, driver());//package,consts,"render",consts,drv
    lua_pushcclosure(L, render, 2);//package,consts,"render",render
    lua_rawset(L, -4);
}

}}

static int delete_svg(lua_State *L) {
    rvg::lua::SVG *p = static_cast<rvg::lua::SVG *>(lua_touserdata(L,1));
    p->~SVG();
    return 0;
}

extern "C"
__attribute__((visibility("default")))
int luaopen_rvg10_svg(lua_State *L) {
    // create userdata holding lua::SVG object
    rvg::lua::SVG *p = static_cast<rvg::lua::SVG *>(
        lua_newuserdata(L, sizeof(rvg::lua::SVG))); // svg
    new (p) rvg::lua::SVG;
    // set metatable with method for garbage collection
    lua_newtable(L); // svg,meta
    lua_pushliteral(L, "__gc"); // svg,meta,"gc"
    lua_pushcfunction(L, delete_svg); // svg,meta,"gc",delete
    lua_rawset(L, -3); // svg,meta
    lua_setmetatable(L, -2); // svg
    // store it in registry keyed by driver address
    lua_pushlightuserdata(L, p->driver()); // svg,driver
    lua_pushvalue(L, -2); // svg,driver,svg
    lua_settable(L, LUA_REGISTRYINDEX); // svg
    lua_pop(L, 1); //
    // initialize Lua binding
    lua_newtable(L); // package
    lua_newtable(L); // package,consts
    p->open(L); // package,consts
    lua_pop(L, 1);
    return 1;
}
