#ifndef RVG_MATH_EUCLIDEAN_FWD_H
#define RVG_MATH_EUCLIDEAN_FWD_H

namespace rvg { namespace math
{

namespace euclidean
{
    // vector -----------------------------------

    template <class T, int N>
    struct Vector;

    template <int N>
    using Vectorf = Vector<float,N>;

    template <int N>
    using Vectori = Vector<int,N>;

    template <int N>
    using Vectord = Vector<double,N>;

    // point -----------------------------------

    template <class T, int N>
    struct Point;

    template <int N>
    using Pointf = Point<float,N>;

    template <int N>
    using Pointi = Point<int,N>;

    template <int N>
    using Pointd = Point<double,N>;
}

// R^2 types

namespace r2
{
    using Pointf = euclidean::Pointf<2>;
    using Pointi = euclidean::Pointi<2>;
    using Pointd = euclidean::Pointd<2>;
    using Point = euclidean::Pointf<2>;

    using Vectorf = euclidean::Vectorf<2>;
    using Vectori = euclidean::Vectori<2>;
    using Vectord = euclidean::Vectord<2>;
    using Vector = euclidean::Vectorf<2>;
}

// R^3 types

namespace r3
{
    using Pointf = euclidean::Pointf<3>;
    using Pointi = euclidean::Pointi<3>;
    using Pointd = euclidean::Pointd<3>;
    using Point = euclidean::Pointf<3>;

    using Vectorf = euclidean::Vectorf<3>;
    using Vectori = euclidean::Vectori<3>;
    using Vectord = euclidean::Vectord<3>;
    using Vector = euclidean::Vectorf<3>;
}

} // namespace math

namespace r2 = math::r2;
namespace r3 = math::r3;

} // namespace rvg

#endif
