#ifndef RVG_MATH_EUCLIDEAN_POINT_H
#define RVG_MATH_EUCLIDEAN_POINT_H

#include <iosfwd>
#include "vector.h"
#include <boost/operators.hpp>

namespace rvg { namespace math { namespace euclidean
{

namespace detail
{
    template <class T, int N>
    struct PointOperators
        : boost::totally_ordered<Point<T,N>
        , boost::additive<Point<T,N>, Vector<T,N>>>
    {
        friend Point<T,N> &operator-=(Point<T,N> &a, const Vector<T,N> &b)
            { return a += -b; }
    };
}

template <class T>
struct Point<T,2>
    : detail::PointOperators<T,2>
{
    Point() = default;
    Point(T _x, T _y) : x(_x), y(_y) {}

    T x, y;

    Point &operator*=(T a);
    Point &operator+=(const Vector<T,2> &that);
    Vector<T,2> operator-(const Point &that) const;
    bool operator==(const Point &that) const;
    bool operator<(const Point &that) const;
};

template <class T>
std::ostream &operator<<(std::ostream &out, const Point<T,2> &p);

template <class T>
struct Point<T,3>
    : detail::PointOperators<T,3>
{
    Point() = default;
    Point(T _x, T _y, T _z) : x(_x), y(_y), z(_z) {}
    Point(const Point<T,2> &p2, T _z) : x(p2.x), y(p2.y), z(_z) {}

    T x, y, z;

    Point &operator*=(T a);
    Point &operator+=(const Vector<T,3> &that);
    Vector<T,3> operator-(const Point &that) const;
    bool operator==(const Point &that) const;
    bool operator<(const Point &that) const;
};

template <class T>
std::ostream &operator<<(std::ostream &out, const Point<T,3> &p);

}}} // namespace rvg::math::euclidean

#include "point.hpp"

#endif
