#include <iostream>

namespace rvg { namespace math { namespace euclidean
{

// multiplication by scalar

template <class T>
auto Point<T,2>::operator*=(T a) -> Point &
{
    x *= a;
    y *= a;
    return *this;
}

template <class T>
auto Point<T,3>::operator*=(T a) -> Point &
{
    x *= a;
    y *= a;
    z *= a;
    return *this;
}

template <class T, int N>
Point<T,N> operator*(const Point<T,N> &v, T a)
{
    Point<T,N> u = v;
    return u *= a;
}

template <class T, int N>
Point<T,N> operator*(T a, const Point<T,N> &v)
{
    Point<T,N> u = v;
    return u *= a;
}

// operator +=

template <class T>
auto Point<T,2>::operator+=(const Vector<T,2> &that) -> Point &
{
    x += that.x;
    y += that.y;
    return *this;
}

template <class T>
auto Point<T,3>::operator+=(const Vector<T,3> &that) -> Point &
{
    x += that.x;
    y += that.y;
    z += that.z;
    return *this;
}

// operator -

template <class T>
auto Point<T,2>::operator-(const Point &that) const -> Vector<T,2>
{
    return {x-that.x, y-that.y};
}

template <class T>
auto Point<T,3>::operator-(const Point &that) const -> Vector<T,3>
{
    return {x-that.x, y-that.y, z-that.z};
}

// operator ==

template <class T>
bool Point<T,2>::operator==(const Point &that) const
{
    return x == that.x && y == that.y;
}

template <class T>
bool Point<T,3>::operator==(const Point &that) const
{
    return x == that.x && y == that.y && z == that.z;
}

// operator <

template <class T>
bool Point<T,2>::operator<(const Point &that) const
{
    if(x == that.x)
        return y < that.y;
    else
        return x < that.x;
}

template <class T>
bool Point<T,3>::operator<(const Point &that) const
{
    if(x == that.x)
    {
        if(y == that.y)
            return z < that.z;
        else
            return y < that.y;
    }
    else
        return x < that.x;
}

template <class T>
T det(const Point<T,2> &a, const Point<T,2> &b) {
    return a.x*b.y-b.x*a.y;
}

template <class T>
T det(const Point<T,3> &a, const Point<T,3> &b, const Point<T,3> &c) {
    return a.y*b.z*c.x-a.z*b.y*c.x+
           a.z*b.x*c.y-a.x*b.z*c.y+
           a.x*b.y*c.z-a.y*b.x*c.z;
}

// output stream

template <class T>
std::ostream &operator<<(std::ostream &out, const Point<T,2> &p)
{
    return out << '(' << p.x << ',' << p.y << ')';
}

template <class T>
std::ostream &operator<<(std::ostream &out, const Point<T,3> &p)
{
    return out << '(' << p.x << ',' << p.y << ',' << p.z << ')';
}

}}} // namespace rvg::math::euclidean
