#ifndef RVG_MATH_EUCLIDEAN_VECTOR_H
#define RVG_MATH_EUCLIDEAN_VECTOR_H

#include <iosfwd>
#include <boost/operators.hpp>
#include "fwd.h"

namespace rvg { namespace math { namespace euclidean
{

namespace detail
{
    template <class T, int N>
    struct VectorOperators
        : boost::totally_ordered<Vector<T,N>
        , boost::additive<Vector<T,N>>>
    {
        friend Vector<T,N> &operator-=(Vector<T,N> &a, const Vector<T,N> &b)
            { return a += -b; }
    };
}


template <class T>
struct Vector<T,2>
    : detail::VectorOperators<T,2>
{
    Vector() = default;
    Vector(T _x, T _y) : x(_x), y(_y) {}

    T x, y;

    Vector &operator+=(const Vector &that);
    Vector &operator*=(T a);
    Vector operator-() const;
    bool operator==(const Vector &that) const;
    bool operator<(const Vector &that) const;
};


template <class T>
std::ostream &operator<<(std::ostream &out, const Vector<T,2> &v);

template <class T>
struct Vector<T,3>
    : detail::VectorOperators<T,3>
{
    Vector() = default;
    Vector(T _x, T _y, T _z) : x(_x), y(_y), z(_z) {}

    T x, y, z;

    Vector &operator+=(const Vector &that);
    Vector operator-() const;
    Vector &operator*=(T a);
    bool operator==(const Vector &that) const;
    bool operator<(const Vector &that) const;
};

template <class T>
std::ostream &operator<<(std::ostream &out, const Vector<T,3> &v);

}}} // namespace rvg::math::euclidean

#include "vector.hpp"

#endif
