#include <iostream>

namespace rvg { namespace math { namespace euclidean
{

// operator +=

template <class T>
auto Vector<T,2>::operator+=(const Vector &that) -> Vector &
{
    x += that.x;
    y += that.y;
    return *this;
}

template <class T>
auto Vector<T,3>::operator+=(const Vector &that) -> Vector &
{
    x += that.x;
    y += that.y;
    z += that.z;
    return *this;
}

// multiplication by scalar

template <class T>
auto Vector<T,2>::operator*=(T a) -> Vector &
{
    x *= a;
    y *= a;
    return *this;
}

template <class T>
auto Vector<T,3>::operator*=(T a) -> Vector &
{
    x *= a;
    y *= a;
    z *= a;
    return *this;
}

template <class T, int N>
Vector<T,N> operator*(const Vector<T,N> &v, T a)
{
    Vector<T,N> u = v;
    return u *= a;
}

template <class T, int N>
Vector<T,N> operator*(T a, const Vector<T,N> &v)
{
    Vector<T,N> u = v;
    return u *= a;
}

// unary operator -

template <class T>
auto Vector<T,2>::operator-() const -> Vector
{
    return {-x, -y};
}

template <class T>
auto Vector<T,3>::operator-() const -> Vector
{
    return {-x, -y, -z};
}

// operator ==

template <class T>
bool Vector<T,2>::operator==(const Vector &that) const
{
    return x == that.x && y == that.y;
}

template <class T>
bool Vector<T,3>::operator==(const Vector &that) const
{
    return x == that.x && y == that.y && z == that.z;
}

// operator <

template <class T>
bool Vector<T,2>::operator<(const Vector &that) const
{
    if(x == that.x)
        return y < that.y;
    else
        return x < that.x;
}

template <class T>
Vector<T,2> cross(const Vector<T,2> &v) {
    return Vector<T,2>(-v.y, v.x);
}

template <class T>
T det(const Vector<T,2> &a, const Vector<T,2> &b) {
    return a.x*b.y-b.x*a.y;
}

template <class T>
T dot(const Vector<T,2> &a, const Vector<T,2> &b) {
    return a.x*b.x + a.y*b.y;
}

template <class T>
T dot(const Vector<T,3> &a, const Vector<T,3> &b) {
    return a.x*b.x + a.y*b.y + a.z*b.z;
}

template <class T>
bool Vector<T,3>::operator<(const Vector &that) const
{
    if(x == that.x)
    {
        if(y == that.y)
            return z < that.z;
        else
            return y < that.y;
    }
    else
        return x < that.x;
}



// output stream

template <class T>
std::ostream &operator<<(std::ostream &out, const Vector<T,2> &v)
{
    return out << '[' << v.x << ',' << v.y << ']';
}

template <class T>
std::ostream &operator<<(std::ostream &out, const Vector<T,3> &v)
{
    return out << '[' << v.x << ',' << v.y << ',' << v.z << ']';
}

}}} // namespace rvg::math::euclidean
