#ifndef MATH_LERP_H
#define MATH_LERP_H

#include "euclidean/point.h"

namespace rvg { namespace math {

template <class T, int N>
euclidean::Point<T,N> lerp(const euclidean::Point<T,N> &a,
    const euclidean::Point<T,N> &b, float t) {
    const euclidean::Point<T,N> zero{};
    return zero + (1.f-t)*(a-zero) + t*(b-zero);
}

template <class T, int N>
euclidean::Point<T,N> lerp(const euclidean::Point<T,N> &a,
    const euclidean::Point<T,N> &b, const euclidean::Point<T,N> &c,
    T t1, T t2) {
    const euclidean::Point<T,N> zero{};
    return zero +
        (a-zero)*(T(1)-t1)*(T(1)-t2)+
        (b-zero)*(t1*(T(1)-t2)+(T(1)-t1)*t2)+
        (c-zero)*t1*t2;
}

template <class T, int N>
euclidean::Point<T,N> lerp(const euclidean::Point<T,N> &a,
    const euclidean::Point<T,N> &b, const euclidean::Point<T,N> &c,
    const euclidean::Point<T,N> &d, T t1, T t2, T t3) {
    const euclidean::Point<T,N> zero{};
    return zero +
        (a-zero)*(T(1)-t1)*(T(1)-t2)*(T(1)-t3) +
        (b-zero)*(
            (T(1)-t1)*(T(1)-t2)*t3 +
            (T(1)-t2)*(T(1)-t3)*t1 +
            (T(1)-t3)*(T(1)-t1)*t2) +
        (c-zero)*(t1*t2*(T(1)-t3)+t2*t3*(T(1)-t1)+t3*t1*(T(1)-t2)) +
        (d-zero)*t1*t2*t3;
}

}} // namespace rvg::math

#endif
