#ifndef RVG_MATH_PROJECTIVE_VECTOR_H
#define RVG_MATH_PROJECTIVE_VECTOR_H

#include <iosfwd>
#include <boost/operators.hpp>
#include "fwd.h"

namespace rvg { namespace math { namespace projective
{

namespace detail
{
    template <class T, int N>
    struct PointOperators
        : boost::totally_ordered<Point<T,N>>
    {
    };
}

template <class T>
struct Point<T,2>
    : detail::PointOperators<T,2>
{
    Point() = default;
    Point(T _x, T _y, T _w) : x(_x), y(_y), w(_w) {}
    Point(const euclidean::Point<T,2> &p, float _w = T(1))
        : x(p.x), y(p.y), w(_w) {}

    Point(const euclidean::Vector<T,2> &p)
        : x(p.x), y(p.y), w(0) {}

    T x, y, w;

    Point &operator*=(T a);
    Point operator-() const;
    bool operator==(const Point &that) const;
    bool operator<(const Point &that) const;
};


template <class T>
std::ostream &operator<<(std::ostream &out, const Point<T,2> &v);

}}} // namespace rvg::math::projective

#include "point.hpp"

#endif
