#include <iostream>

namespace rvg { namespace math { namespace projective
{

// multiplication by scalar

template <class T>
auto Point<T,2>::operator*=(T a) -> Point &
{
    x *= a;
    y *= a;
    w *= a;
    return *this;
}

template <class T, int N>
Point<T,N> operator*(const Point<T,N> &v, T a)
{
    Point<T,N> u = v;
    return u *= a;
}

template <class T, int N>
Point<T,N> operator*(T a, const Point<T,N> &v)
{
    Point<T,N> u = v;
    return u *= a;
}

// unary operator -

template <class T>
auto Point<T,2>::operator-() const -> Point
{
    return {-x, -y, -w};
}

// operator ==

template <class T>
bool Point<T,2>::operator==(const Point &that) const
{
    return x*that.w == that.x*w && y*that.w == that.y*w;
}

// operator <

template <class T>
bool Point<T,2>::operator<(const Point &that) const
{
    float ax = x*that.w,
          bx = that.x*w;

    if(ax == bx)
        return y*that.w < that.y*w;
    else
        return ax < bx;
}

template <class T>
T det(const Point<T,2> &a, const Point<T,2> &b, const Point<T,2> &c) {
    return a.y*b.w*c.x-a.w*b.y*c.x+
           a.w*b.x*c.y-a.x*b.w*c.y+
           a.x*b.y*c.w-a.y*b.x*c.w;
}

// output stream

template <class T>
std::ostream &operator<<(std::ostream &out, const Point<T,2> &v)
{
    return out << '[' << v.x << ',' << v.y << ",(" << v.w << ")]";
}

}}} // namespace rvg::math::projective
