#ifndef RVG_MATH_QUADRATIC_H
#define RVG_MATH_QUADRATIC_H

#include <array>

namespace rvg { namespace math { namespace quadratic {

    // From "How to Solve a Quadratic Equation",
    // Jim Blinn's Corner, Nov/Dec 2005
    // and "How (Not) to Solve Quadratic Equations"
    // Yves Nievergelt, Mar/2003
    //
    // Computes the roots of the homogeneous polynomial
    //
    //     a s s + 2 b s t + c s s == 0.
    //
    // Please note the 2 b in the equation!!!
    //
    // Roots of a t t + 2 b t + c == 0 can be computed from
    // t[i]/s[i], but s[i] can be very small or even zero.
    // Never divide these numbers! We never really need the
    // If s[i] is zero, what this means is that the quadratic
    // degenerated to a linear equation.
    // If both s[i] are zero, then the linear equation
    // degenerated as well

    int solve(float a, float b, float c,
        std::array<float,2> &t,
        std::array<float,2> &s);

    int solve(double a, double b, double c,
        std::array<double,2> &t,
        std::array<double,2> &s);

    // discriminant really is the return of the function, times 2^(2*e)
    float discriminant(float a, float b, float c, int &e);

    double discriminant(double a, double b, double c, int &e);

    int solve(float a, float b, float c, float delta, int e,
        std::array<float,2> &t,
        std::array<float,2> &s);

    int solve(double a, double b, double c, double delta, int e,
        std::array<double,2> &t,
        std::array<double,2> &s);

}}} // namespace rvg::math::quadratic

#endif // RVG_MATH_QUADRATIC_H
