#ifndef RVG_PAINT_BASE
#define RVG_PAINT_BASE

#include <iosfwd>
#include "../util/cloneable.h"
#include "../xform/xform.h"
#include "visitor.h"

namespace rvg { namespace paint {

class Base: public Cloneable<Base> {
public:

    // accept is virtual and public??
    virtual void accept(Visitor &v) const = 0;

    Base& transform(const Transform &xform) {
        return do_transform(xform);
    }

    void print(std::ostream &out) {
        return do_print(out);
    }

    friend std::ostream &operator<<(std::ostream &out, const Base &base);

protected:
    virtual void do_print(std::ostream &out) const = 0;
    virtual Base &do_transform(const Transform &xform) = 0;
};

// use CRTP to implement some pure methods
template <class T>
class BaseHelper
    : public Base
{
public:
    void accept(Visitor &v) const final;

    RefPtr<T> clone() const { return static_pointer_cast<T>(Base::clone()); }

private:
    T &derived();
    const T &derived() const;

    Base *do_clone() const override;

    template <class U> friend class BaseHelper;
};

}} // namespace rvg::paint

#include "base.hpp"

#endif // RVT_PAINT_BASE_H
