#include "paint.h"
#include "solid.h"
#include "lineargradient.h"
#include "radialgradient.h"
#include "texture.h"

namespace rvg { namespace paint {

std::ostream &operator<<(std::ostream &out, const Paint &paint) {
    out << "dyn_paint " << paint.m_opacity << ' ' << *paint.m_cow.ro();
    return out;
}

namespace ramp {

    std::ostream &operator<<(std::ostream &out, const Ramp &ramp) {
        out << "dyn_" << *ramp.m_cow.ro();
        return out;
    }

    std::ostream &operator<<(std::ostream &out, const Base &stops) {
        bool first = true;
        out << "ramp" << ' ';
        for (const auto &stop: stops.m_stops) {
            if (first) first = false;
            else out << ' ';
            out << stop.offset << ':' << stop.color;
        }
        out << ' ' << stops.spread();
        return out;
    }
}

std::ostream &operator<<(std::ostream &out, const Base &base) {
    base.do_print(out);
    return out;
}

void LinearGradient::do_print(std::ostream &out) const {
    out << paint_type << ' '
        << m_p1 << ' ' << m_p2 << ' '
        << m_ramp << ' '
        << xf();
}

void RadialGradient::do_print(std::ostream &out) const {
    out << paint_type << ' '
        << m_c << ' ' << m_f << ' '
        << m_r << ' '
        << m_ramp << ' '
        << xf();
}

void Texture::do_print(std::ostream &out) const {
    out << paint_type << ' ' 
        << m_image.width() << 'x' << m_image.height() << ' '
        << m_spread_method << ' '
        << xf();
}

void Solid::do_print(std::ostream &out) const {
    out << paint_type << ' ' << m_color;
}

}} // namespace rvg::paint

