#ifndef RVG_PAINT_H
#define RVG_PAINT_H

#include <iosfwd>
#include "../xform/xform.h"
#include "../util/cow.h"
#include "base.h"
#include "solid.h"
#include "lineargradient.h"
#include "radialgradient.h"
#include "texture.h"
#include "visitor.h"

namespace rvg { namespace paint {

class Paint final {
public:
    explicit Paint(Base *base, float opacity = 1.f):
        m_cow(base), m_opacity(opacity) { ; }
    explicit Paint(const Base &base, float opacity = 1.f):
        m_cow(base.clone()), m_opacity(opacity) { ; }

    Paint& transform(const Transform &xf) {
        m_cow.rw()->transform(xf);
        return *this;
    }

    float opacity(void) const { return m_opacity; }
    Paint &set_opacity(float opacity) { m_opacity = opacity; return *this; }

    friend std::ostream &operator<<(std::ostream &out, const Paint &paint);

    void accept(Visitor &v) const { m_cow.ro()->accept(v); }

    IMPLEMENT_COW(Base, m_cow);

private:
    COW<Base> m_cow;
    float m_opacity;
};

} // namespace rvg::paint

using paint::Paint;

} // namespace rvg

#endif // RVG_PAINT_H
