#ifndef RVG_PAINT_RADIAL_GRADIENT_H
#define RVG_PAINT_RADIAL_GRADIENT_H

#include "../constants.h"
#include "xformedbase.h"
#include "ramp.h"

namespace rvg { namespace paint {

class RadialGradient final: public XFormedBase<RadialGradient> {
public:
    static constexpr const PaintType paint_type = PaintType::radial_gradient;
    RadialGradient(
        const ramp::Ramp &ramp,
        r2::Point c, r2::Point f, float r,
        const Transform &xf = Transform{}):
        XFormedBase(xf),
        m_ramp(ramp),
        m_c(c), m_f(f), m_r(r) {}

    const r2::Point c(void) const { return m_c; }
    const r2::Point f(void) const { return m_f; }
    float r(void) const { return m_r; }

    const ramp::Ramp &ramp(void) const { return m_ramp; }

    IMPLEMENT_PAINT_XFORMEDBASE(RadialGradient);
protected:
    virtual void do_print(std::ostream &out) const;
private:
    ramp::Ramp m_ramp;
    r2::Point m_c, m_f;
    float m_r;
};

}} // namespace rvg::paint

#endif // RVG_PAINT_RADIAL_GRADIENT_H
