#ifndef RVG_PATH_COMMAND_H
#define RVG_PATH_COMMAND_H

#include <cassert>

#include "../constants.h"

namespace rvg { namespace path {

class Command {
public:
    // Constructors for each number of argument, each checking
    // for consistency with the command type
    Command(CommandType type): type(type) {
        assert(type == CommandType::close_path_abs ||
            type == CommandType::close_path_rel);
    }

    Command(CommandType type, float a0): type(type) {
        assert(type == CommandType::hline_to_abs ||
            type == CommandType::hline_to_rel ||
            type == CommandType::vline_to_abs ||
            type == CommandType::vline_to_rel);
        arguments[0] = a0;
    }

    Command(CommandType type, float a0, float a1): type(type) {
        assert(type == CommandType::move_to_abs ||
            type == CommandType::move_to_rel ||
            type == CommandType::line_to_abs ||
            type == CommandType::line_to_rel ||
            type == CommandType::squad_to_abs ||
            type == CommandType::squad_to_rel);
        arguments[0] = a0;
        arguments[1] = a1;
    }

    // no command accepts 3 arguments, but failing to include this could
    // lead to weird compile-type errors
    Command(CommandType type, float a0, float a1, float a2): type(type) {
        assert(0);
        arguments[0] = a0;
        arguments[1] = a1;
        arguments[2] = a2;
    }

    Command(CommandType type, float a0, float a1, float a2, float a3):
        type(type) {
        assert(type == CommandType::quad_to_abs ||
            type == CommandType::quad_to_rel ||
            type == CommandType::scubic_to_abs ||
            type == CommandType::scubic_to_rel);
        arguments[0] = a0;
        arguments[1] = a1;
        arguments[2] = a2;
        arguments[3] = a3;
    }

    Command(CommandType type, float a0, float a1, float a2, float a3, 
        float a4): type(type) {
        assert(type == CommandType::rquad_to_abs ||
            type == CommandType::rquad_to_rel);
        arguments[0] = a0;
        arguments[1] = a1;
        arguments[2] = a2;
        arguments[3] = a3;
        arguments[4] = a4;
    }

    Command(CommandType type, float a0, float a1, float a2, float a3, float a4,
        float a5): type(type) {
        assert(type == CommandType::cubic_to_abs ||
            type == CommandType::cubic_to_rel ||
            type == CommandType::svgarc_to_abs ||
            type == CommandType::svgarc_to_rel);
        arguments[0] = a0;
        arguments[1] = a1;
        arguments[2] = a2;
        arguments[3] = a3;
        arguments[4] = a4;
        arguments[5] = a5;
    }
    CommandType type;
    float arguments[6]; // enough for largest command
};

} // namespace rvg:path

using path::Command;

} // namespace rvg

#endif // RVG_PATH_COMMAND_H
