#include "scene.h"

namespace rvg { namespace scene {

// trying to make the xformed clone as fast as possible...
RefPtr<RefElements>
RefElements::xform_clone(const Transform &xf) const {
    // allocate a new empty array (no elements are constructed)
    RefPtr<RefElements> cloned(new RefElements);
    // reserve enough space (no elements are constructed)
    cloned->m_elements.reserve(m_elements.size());
    // construct elements in-place
    // temporary transformed paths and paints are simply moved
    for (const auto &el: m_elements) {
        if (el.is_fill()) {
            cloned->m_elements.emplace_back(el.type,
                Path(el.fill.path).transform(xf),
                Paint(el.fill.paint).transform(xf));
        } else if (el.is_clip())  {
            cloned->m_elements.emplace_back(el.type,
                Path(el.clip.path).transform(xf), el.clip.depth);
        } else { // pop,push,commit
            cloned->m_elements.emplace_back(el.type, el.stack.depth);
        }
    }
    return cloned;
}

void
RefElements::transform(const Transform &xf) {
    // identity? nothing to do
    if(xf.is_identity())
        return;

    for (auto &el: m_elements) {
        if (el.is_fill()) {
            el.fill.path.transform(xf);
            el.fill.paint.transform(xf);
        } else if (el.is_clip()) {
            el.clip.path.transform(xf);
        }
    }
}

}} // namespace qvg::scene
