namespace rvg { namespace scene {

inline Scene &
Scene::append(const Element &element) {
    if (element.is_fill()) {
        fill(element.fill.path, element.fill.paint, element.type);
    } else if (element.is_clip()) {
        clip(element.clip.path, element.type);
    } else if (element.is_push()) {
        push();
    } else if (element.is_pop()) {
        pop();
    } else if (element.is_commit()) {
        commit();
    } else {
        assert(0); // what is it, then?
    }
    return *this;
}

inline Scene &
Scene::fill(Path path, Paint paint, Element::Type type) {
    //assert(m_uncommitted == 1);
    m_uncommitted = 0;
    rw()->elements().emplace_back(
        type,
        std::move(path),
        std::move(paint)
    );
    return *this;
}

inline Scene &
Scene::nzfill(const Path &path, const Paint &paint) {
    return fill(path, paint, Element::Type::nzfill);
}

inline Scene &
Scene::zfill(const Path &path, const Paint &paint) {
    return fill(path, paint, Element::Type::zfill);
}

inline Scene &
Scene::eofill(const Path &path, const Paint &paint) {
    return fill(path, paint, Element::Type::ofill);
}

inline Scene &
Scene::ofill(const Path &path, const Paint &paint) {
    return fill(path, paint, Element::Type::ofill);
}

inline Scene &
Scene::efill(const Path &path, const Paint &paint) {
    return fill(path, paint, Element::Type::efill);
}

inline Scene &
Scene::push(void) {
    m_depth++;
    rw()->elements().emplace_back(
        m_uncommitted > 0? Element::Type::pushclip: Element::Type::pushfill,
        m_depth
    );
    m_uncommitted++;
    return *this;
}

inline Scene &
Scene::pop(void) {
    assert(m_depth > 0);
    if (m_depth > 0) {
        rw()->elements().emplace_back(
            m_uncommitted > 0? Element::Type::popclip: Element::Type::popfill,
            m_depth
        );
    }
    m_depth--;
    return *this;
}

inline Scene &
Scene::commit(void) {
    assert(m_uncommitted > 0);
    if (m_uncommitted > 0) {
        rw()->elements().emplace_back(
            Element::Type::commit,
            m_depth
        );
        m_uncommitted--;
    }
    return *this;
}

inline Scene &
Scene::clip(Path path, Element::Type type) {
    assert(m_uncommitted > 0);
    if (m_uncommitted > 0) {
        rw()->elements().emplace_back(
            type,
            std::move(path),
            m_depth
        );
    }
    return *this;
}

inline Scene &
Scene::eoclip(const Path &path) {
    return clip(path, Element::Type::oclip);
}

inline Scene &
Scene::oclip(const Path &path) {
    return clip(path, Element::Type::oclip);
}

inline Scene &
Scene::eclip(const Path &path) {
    return clip(path, Element::Type::eclip);
}

inline Scene &
Scene::zclip(const Path &path) {
    return clip(path, Element::Type::zclip);
}

inline Scene &
Scene::nzclip(const Path &path) {
    return clip(path, Element::Type::nzclip);
}

}} // namespace rvg::scene
