local svg=require"rvg10.svg"
setmetatable(_G, { __index = svg} )

local function flip(s, v)
    return s:scale(1,-1):translate(0, v.max.y),
        viewport(v.min.x, 0, v.max.x, v.max.y-v.min.y)
end

function lerp(a, b, c, d, t1, t2, t3)
        return a*(1-t1)*(1-t2)*(1-t3) +
            b*((1-t1)*(1-t2)*t3 +
            (1-t2)*(1-t3)*t1 +
            (1-t3)*(1-t1)*t2) +
            c*(t1*t2*(1-t3)+t2*t3*(1-t1)+t3*t1*(1-t2)) +
            d*t1*t2*t3;
end

local function cut(x0, y0, x1, y1, x2, y2, x3, y3, t0, t1)
    u0 = lerp(x0, x1, x2, x3, t0, t0, t0)
    u1 = lerp(x0, x1, x2, x3, t1, t0, t0)
    u2 = lerp(x0, x1, x2, x3, t1, t1, t0)
    u3 = lerp(x0, x1, x2, x3, t1, t1, t1)
    v0 = lerp(y0, y1, y2, y3, t0, t0, t0)
    v1 = lerp(y0, y1, y2, y3, t1, t0, t0)
    v2 = lerp(y0, y1, y2, y3, t1, t1, t0)
    v3 = lerp(y0, y1, y2, y3, t1, t1, t1)
    return u0, v0, u1, v1, u2, v2, u3, v3
end

local v = viewport(195,-5,195+310,-5+410)
local p = path{M, 368.4375, 162.91666666666666, C, 364.0625,
138.75, 445.3125, 113.75, 407.8125, 132.91666666666663}

for i = 0.01, 1, 0.01 do
    local x0, y0, x1, y1, x2, y2, x3, y3 = cut(368.4375, 162.91666666666666,
        364.0625, 138.75, 445.3125, 113.75, 407.8125, 132.91666666666663, 0, i)
    local p = path{M, x0, y0, C, x1, y1, x2, y2, x3, y3}
    local f = io.open(string.format("b-%3.2f.svg", i), "w")
    local s, fv = flip(scene{
        fill(p, color.lightgoldenrodyellow),
        fill(p:stroke(108), color.olivedrab)
    }, v)
    render(s, fv, f)
    f:close()
end
