#include "math/point.h"
#include "math/vector.h"

using namespace rvg::math;

// r2 vectors --------------------------------------------------------

void r2_vector_addition()
{
    r2::Vector v1{1,3}, v2{5,6},
               r = v1+v2;
    std::cout << v1 << " + " << v2 << " = " << r << std::endl;
}

void r2_vector_subtraction()
{
    r2::Vector v1{1,3}, v2{5,6},
               r = v1-v2;
    std::cout << v1 << " - " << v2 << " = " << r << std::endl;
}

void r2_vector_negation()
{
    r2::Vector v{1,3},
               r = -v;
    std::cout << '-' << v << " = " << r << std::endl;
}

namespace detail
{
    template <class T>
    void do_comparison(const T &v1, const T &v2)
    {
        bool r;

        r = v1==v2;
        std::cout << v1 << " == " << v2 << ": " 
                  << (r?"true":"false") << std::endl;

        r = v1!=v2;
        std::cout << v1 << " != " << v2 << ": " 
                  << (r?"true":"false") << std::endl;

        r = v1<v2;
        std::cout << v1 << " < " << v2 << ": " 
                  << (r?"true":"false") << std::endl;

        r = v1<=v2;
        std::cout << v1 << " <= " << v2 << ": " 
                  << (r?"true":"false") << std::endl;

        r = v1>v2;
        std::cout << v1 << " > " << v2 << ": " 
                  << (r?"true":"false") << std::endl;

        r = v1>=v2;
        std::cout << v1 << " >= " << v2 << ": " 
                  << (r?"true":"false") << std::endl;
    }
}

void r2_vector_comparison()
{
    detail::do_comparison<r2::Vector>({1,5}, {2,5});
    detail::do_comparison<r2::Vector>({1,1}, {2,5});
    detail::do_comparison<r2::Vector>({1,7}, {2,5});
    detail::do_comparison<r2::Vector>({2,1}, {2,5});
    detail::do_comparison<r2::Vector>({2,5}, {2,5});
    detail::do_comparison<r2::Vector>({2,7}, {2,5});
    detail::do_comparison<r2::Vector>({3,1}, {2,5});
    detail::do_comparison<r2::Vector>({3,5}, {2,5});
    detail::do_comparison<r2::Vector>({3,7}, {2,5});
}

// r3 vectors --------------------------------------------------------

void r3_vector_addition()
{
    r3::Vector v1{1,3,6}, v2{5,6,7},
               r = v1+v2;
    std::cout << v1 << " + " << v2 << " = " << r << std::endl;
}

void r3_vector_subtraction()
{
    r3::Vector v1{1,3,6}, v2{5,6,7},
               r = v1-v2;
    std::cout << v1 << " - " << v2 << " = " << r << std::endl;
}

void r3_vector_negation()
{
    r3::Vector v{1,3,6},
               r = -v;
    std::cout << '-' << v << " = " << r << std::endl;
}

void r3_vector_comparison()
{
    detail::do_comparison<r3::Vector>({1,5,6}, {2,5,6});
    detail::do_comparison<r3::Vector>({1,5,5}, {2,5,6});
    detail::do_comparison<r3::Vector>({1,5,7}, {2,5,6});
    detail::do_comparison<r3::Vector>({1,1,6}, {2,5,6});
    detail::do_comparison<r3::Vector>({1,1,5}, {2,5,6});
    detail::do_comparison<r3::Vector>({1,1,7}, {2,5,6});
    detail::do_comparison<r3::Vector>({1,7,6}, {2,5,6});
    detail::do_comparison<r3::Vector>({1,7,5}, {2,5,6});
    detail::do_comparison<r3::Vector>({1,7,7}, {2,5,6});
    detail::do_comparison<r3::Vector>({2,1,6}, {2,5,6});
    detail::do_comparison<r3::Vector>({2,1,5}, {2,5,6});
    detail::do_comparison<r3::Vector>({2,1,7}, {2,5,6});
    detail::do_comparison<r3::Vector>({2,5,6}, {2,5,6});
    detail::do_comparison<r3::Vector>({2,5,5}, {2,5,6});
    detail::do_comparison<r3::Vector>({2,5,7}, {2,5,6});
    detail::do_comparison<r3::Vector>({2,7,6}, {2,5,6});
    detail::do_comparison<r3::Vector>({2,7,5}, {2,5,6});
    detail::do_comparison<r3::Vector>({2,7,7}, {2,5,6});
    detail::do_comparison<r3::Vector>({3,1,6}, {2,5,6});
    detail::do_comparison<r3::Vector>({3,1,5}, {2,5,6});
    detail::do_comparison<r3::Vector>({3,1,7}, {2,5,6});
    detail::do_comparison<r3::Vector>({3,5,6}, {2,5,6});
    detail::do_comparison<r3::Vector>({3,5,5}, {2,5,6});
    detail::do_comparison<r3::Vector>({3,7,6}, {2,5,6});
    detail::do_comparison<r3::Vector>({3,7,7}, {2,5,6});
}

// r2 points --------------------------------------------------------

void r2_point_vector_addition()
{
    r2::Point  p{1,3};
    r2::Vector v{5,6};
    r2::Point r = p+v;

    std::cout << p << " + " << v << " = " << r << std::endl;
}

void r2_vector_point_addition()
{
    r2::Point  p{1,3};
    r2::Vector v{5,6};
    r2::Point r = v+p;

    std::cout << v << " + " << p << " = " << r << std::endl;
}

void r2_point_subtraction()
{
    r2::Point  p1{1,3}, p2{5,6};
    r2::Vector r = p1-p2;

    std::cout << p1 << " - " << p2 << " = " << r << std::endl;
}

void r2_point_comparison()
{
    detail::do_comparison<r2::Point>({1,5}, {2,5});
    detail::do_comparison<r2::Point>({1,1}, {2,5});
    detail::do_comparison<r2::Point>({1,7}, {2,5});
    detail::do_comparison<r2::Point>({2,1}, {2,5});
    detail::do_comparison<r2::Point>({2,5}, {2,5});
    detail::do_comparison<r2::Point>({2,7}, {2,5});
    detail::do_comparison<r2::Point>({3,1}, {2,5});
    detail::do_comparison<r2::Point>({3,5}, {2,5});
    detail::do_comparison<r2::Point>({3,7}, {2,5});
}

// r3 points --------------------------------------------------------

void r3_point_vector_addition()
{
    r3::Point  p{1,3,6};
    r3::Vector v{5,6,7};
    r3::Point r = p+v;

    std::cout << p << " + " << v << " = " << r << std::endl;
}

void r3_vector_point_addition()
{
    r3::Point  p{1,3,6};
    r3::Vector v{5,6,7};
    r3::Point r = v+p;

    std::cout << v << " + " << p << " = " << r << std::endl;
}

void r3_point_subtraction()
{
    r3::Point  p1{1,3,6}, p2{5,6,7};
    r3::Vector r = p1-p2;

    std::cout << p1 << " - " << p2 << " = " << r << std::endl;
}

void r3_point_comparison()
{
    detail::do_comparison<r3::Point>({1,5,6}, {2,5,6});
    detail::do_comparison<r3::Point>({1,5,5}, {2,5,6});
    detail::do_comparison<r3::Point>({1,5,7}, {2,5,6});
    detail::do_comparison<r3::Point>({1,1,6}, {2,5,6});
    detail::do_comparison<r3::Point>({1,1,5}, {2,5,6});
    detail::do_comparison<r3::Point>({1,1,7}, {2,5,6});
    detail::do_comparison<r3::Point>({1,7,6}, {2,5,6});
    detail::do_comparison<r3::Point>({1,7,5}, {2,5,6});
    detail::do_comparison<r3::Point>({1,7,7}, {2,5,6});
    detail::do_comparison<r3::Point>({2,1,6}, {2,5,6});
    detail::do_comparison<r3::Point>({2,1,5}, {2,5,6});
    detail::do_comparison<r3::Point>({2,1,7}, {2,5,6});
    detail::do_comparison<r3::Point>({2,5,6}, {2,5,6});
    detail::do_comparison<r3::Point>({2,5,5}, {2,5,6});
    detail::do_comparison<r3::Point>({2,5,7}, {2,5,6});
    detail::do_comparison<r3::Point>({2,7,6}, {2,5,6});
    detail::do_comparison<r3::Point>({2,7,5}, {2,5,6});
    detail::do_comparison<r3::Point>({2,7,7}, {2,5,6});
    detail::do_comparison<r3::Point>({3,1,6}, {2,5,6});
    detail::do_comparison<r3::Point>({3,1,5}, {2,5,6});
    detail::do_comparison<r3::Point>({3,1,7}, {2,5,6});
    detail::do_comparison<r3::Point>({3,5,6}, {2,5,6});
    detail::do_comparison<r3::Point>({3,5,5}, {2,5,6});
    detail::do_comparison<r3::Point>({3,7,6}, {2,5,6});
    detail::do_comparison<r3::Point>({3,7,7}, {2,5,6});
}

// p2 points --------------------------------------------------------

void p2_point_equality1()
{
    p2::Point p1{1,2,1}, p2{1,2,1};
    bool r = p1 == p2;

    std::cout << p1 << " == " << p2 << ": " 
              << (r?"true":"false") << std::endl;
}

void p2_point_equality2()
{
    p2::Point p1{2,4,2}, p2{1,2,1};
    bool r = p1 == p2;

    std::cout << p1 << " == " << p2 << ": " 
              << (r?"true":"false") << std::endl;
}

void p2_point_ctor_from_euclid_vector()
{
    r2::Vector v{2,3};

    p2::Point r = v;
    std::cout << v << " = " << r << std::endl;
}

void p2_point_ctor_from_euclid_point()
{
    r2::Point p{2,3};

    p2::Point r = p;
    std::cout << p << " = " << r << std::endl;
}

void p2_point_assign_from_euclid_vector()
{
    r2::Vector v{2,3};

    p2::Point r;
    r = v;
    std::cout << v << " = " << r << std::endl;
}

void p2_point_assign_from_euclid_point()
{
    r2::Point p{2,3};

    p2::Point r;
    r = p;
    std::cout << p << " = " << r << std::endl;
}

#define CALL_TEST(x) else if(test == #x) x()

int main(int argc, char *argv[])
{
    if(argc != 2)
    {
        std::cerr << "wrong arguments" << std::endl;
        return -1;
    }

    std::string test = argv[1];

    if(false);
    CALL_TEST(r2_vector_addition);
    CALL_TEST(r2_vector_subtraction);
    CALL_TEST(r2_vector_negation);
    CALL_TEST(r2_vector_comparison);

    CALL_TEST(r3_vector_addition);
    CALL_TEST(r3_vector_subtraction);
    CALL_TEST(r3_vector_negation);
    CALL_TEST(r3_vector_comparison);

    CALL_TEST(r2_point_vector_addition);
    CALL_TEST(r2_vector_point_addition);
    CALL_TEST(r2_point_subtraction);
    CALL_TEST(r2_point_comparison);

    CALL_TEST(r3_point_vector_addition);
    CALL_TEST(r3_vector_point_addition);
    CALL_TEST(r3_point_subtraction);
    CALL_TEST(r3_point_comparison);

    CALL_TEST(p2_point_equality1);
    CALL_TEST(p2_point_equality2);
    CALL_TEST(p2_point_ctor_from_euclid_vector);
    CALL_TEST(p2_point_ctor_from_euclid_point);
    CALL_TEST(p2_point_assign_from_euclid_vector);
    CALL_TEST(p2_point_assign_from_euclid_point);
    
    else
    {
        std::cerr << "unknown test: " << test << std::endl;
        return -1;
    }

    return 0;
}
