#include <cmath>
#include <cstdio>
#include <algorithm>

#include "../math/quadratic.h"

using namespace rvg::math::quadratic;

using number = double;

template <typename T>
T error(T u, T t, T s) {
    return abs(u-t/s)/u;
}

template <typename T>
T error(T u0, T u1, T t0, T s0, T t1, T s1) {
    return std::min(
        std::max(error(u0,t0,s0), error(u1,t1,s1)),
        std::max(error(u1,t0,s0), error(u0,t1,s1))
    );
}

number max_a, max_u0, max_u1;
number max_e = 0.f;

void testroots(number a, number u0, number u1) {
    number b = -a*(u0+u1);
    number c = a*u0*u1;
    std::array<number,2> t, s;
    int n = solve(a, 0.5*b, c, t, s);
    if (n == 2) {
        number e = error(u0, u1, t[0], s[0], t[1], s[1]);
        if (e > max_e) {
            fprintf(stdout, "  max relative error for (%g, %g, %g) is %g\n",
                a, u0, u1, e);
            max_e = e;
            max_a = a;
            max_u0 = u0;
            max_u1 = u1;
        }
    } else {
        fprintf(stderr, "got no roots for (%g, %g, %g)!\n", a, u0, u1);
    }
}

int main(void) {
    int ri = 10;
    int rj = 10;
    int rk = 10;
    for (int j = -rj; j <= rj; j++) {
        number u0 = pow(number(10), number(j));
        for (int k = -rk; k <= rk; k++) {
            number u1 = pow(number(10), number(k));
            for (int i = -ri; i <= ri; i++) {
                number a = pow(number(10), number(i));
                for (float s0: {number(-1), number(1)}) {
                    for (float s1: {number(-1), number(1)}) {
                        testroots(a, s0*u0, s1*u1);
                    }
                }
            }
        }
    }
    return 0;
}
