#include "../math/svd.h"
#include "../math/util.h"

#include <iostream>

using namespace std;
using namespace rvg;
using namespace math;
using namespace xform;

int test_id = 0;
float max_e = 0.f;

void test(const Rotation &iu, const Scale &is, const Linear &iv) {
    Rotation u;
    Scale s;
    Linear v;
    Linear a = iu*is*transpose(iv);
    cout << "test " << test_id++ << endl;
    cout << "  input is" << endl;
    cout << "    u' (det) = " << iu << " (" << det(iu) << ")" << endl;
    cout << "    s' (det) = " << is << " (" << det(is) << ")" << endl;
    cout << "    v' (det) = " << iv << " (" << det(iv) << ")" << endl;
    svd(a, u, s, v);
    cout << "  svd is" << endl;
    cout << "    u  (det) = " << u << " (" << det(u) << ")" << endl;
    cout << "    s  (det) = " << s << " (" << det(s) << ")" << endl;
    cout << "    v  (det) = " << v << " (" << det(v) << ")" << endl;
    cout << "  residual is" << endl;
    Linear b = u*s*transpose(v);
    float e = sqrt(frobenius(a - b));
    if (e > max_e) max_e = e;
    cout << "    e  = " << e << endl;
}

int main(void) {
    test(Identity(), Scale(1,2), Rotation(35*math::to_rad()));
    test(Rotation(35*math::to_rad()), Scale(1,2), Identity());
    test(Identity(), Scale(123,0.123), Identity());
    test(Identity(), Scale(1234,0.01234), Identity());
    test(Identity(), Scale(12345,0.0012345), Identity());
    test(Identity(), Scale(123456,0.000123456), Identity());
    test(Identity(), Scale(0.000123456,123456), Identity());
    test(Identity(), Scale(0.00001234567,1234567), Identity());
    cout << "max error was " << max_e << endl;
    return 0;
}
