#ifndef QVG_XFORM_AFFINE_H
#define QVG_XFORM_AFFINE_H

#include "base.h"
#include <array>

namespace rvg { namespace xform
{

class Affine final : public BaseHelper<Affine>
{
public:
    static constexpr const char *name = "affine";

#if TRACE_XFORM
    Affine();
#else
    Affine() = default;
#endif

    Affine(float a11, float a12, float a13,
           float a21, float a22, float a23);

    // implicit conversions allowed
    Affine(const Identity &);
    Affine(const Rotation &b);
    Affine(const Scale &b);
    Affine(const Linear &b);

    Linear linear(void) const;

    auto operator[](int i) -> std::array<float,3> &;
    auto operator[](int i) const -> const std::array<float,3> &;

    Affine &operator *=(const Identity &) { return *this; }
    Affine &operator *=(const Translation &that);
    Affine &operator *=(const Rotation &that);

    Affine &operator *=(const Scale &that);
    Affine &operator *=(const Linear &that);
    Affine &operator *=(const Affine &that);

private:
    std::array<std::array<float,3>,2> m_matrix;

    virtual void do_print(std::ostream &out) const;

    r2::Point do_multiply(const r2::Point &p) const override;
    r2::Vector do_multiply(const r2::Vector &v) const override;
    r3::Point do_multiply(const r3::Point &p) const override;
};

Affine operator*(const Affine &a, const Affine &b);

Affine operator*(const Translation &a, const Rotation &b);
Affine operator*(const Translation &a, const Scale &b);
Affine operator*(const Translation &a, const Linear &b);
Affine operator*(const Translation &a, const Affine &b);

Affine operator*(const Rotation &a, const Translation &b);
Affine operator*(const Rotation &a, const Affine &b);

Affine operator*(const Scale &a, const Translation &b);
Affine operator*(const Scale &a, const Affine &b);

Affine operator*(const Linear &a, const Translation &b);
Affine operator*(const Linear &a, const Affine &b);

}} // namespace rvg::xform

#include "affine.hpp"

#endif
