namespace rvg { namespace xform
{

inline Identity::Identity()
{
#if TRACE_XFORM
    std::clog << "-- ctor " << *this << std::endl;
#endif
}

template<class XF>
auto operator*(XF &&lhs, const Identity &rhs)
    -> typename std::enable_if<is_xform<XF>::value &&
                               // for disambiguation
                               !std::is_same<
                                    typename std::remove_cv<
                                            typename std::remove_reference<XF>::type
                                    >::type,Identity
                                >::value,
                        XF &&>::type
{
    (void)rhs;

#if TRACE_XFORM
    std::clog << " -- forward-only composition of "
              << lhs << " and " << rhs << std::endl;
#endif
    return std::forward<XF>(lhs);
}

template<class XF>
auto operator*(const Identity &lhs, XF &&rhs)
    -> typename std::enable_if<is_xform<XF>::value &&

                               // for disambiguation
                               !std::is_same<
                                    typename std::remove_cv<
                                            typename std::remove_reference<XF>::type
                                    >::type,Identity
                                >::value,
                        XF &&>::type
{
    (void)lhs;

#if TRACE_XFORM
    std::clog << " -- forward-only composition of "
              << lhs << " and " << rhs << std::endl;
#endif
    return std::forward<XF>(rhs);
}

// Transposition
inline Identity Transpose(const Identity &) {
    return Identity();
}

inline r2::Point Identity::do_multiply(const r2::Point &p) const
{
    return p;
}

inline r2::Vector Identity::do_multiply(const r2::Vector &v) const
{
    return v;
}

inline r3::Point Identity::do_multiply(const r3::Point &p) const
{
    return p;
}

}} // namespace rvg::xform
