#ifndef QVG_XFORM_LINEAR_H
#define QVG_XFORM_LINEAR_H

#include <cmath>
#include <array>
#include "base.h"

namespace rvg { namespace xform
{

class Linear final : public BaseHelper<Linear>
{
public:
    static constexpr const char *name = "linear";

#if TRACE_XFORM
    Linear();
#else
    Linear() = default;
#endif

    // implicit conversions allowed
    Linear(const Identity &);
    Linear(const Rotation &rot);
    Linear(const Scale &s);
    explicit Linear(const Affine &a); // truncates information
    explicit Linear(const Projective &p); // truncates information

    Linear(float a11, float a12,
           float a21, float a22);

    auto operator[](int i) -> std::array<float,2> &;

    auto operator[](int i) const -> const std::array<float,2> &;

    Linear &operator *=(float that);
    Linear &operator *=(const Identity &) { return *this; }
    Linear &operator *=(const Linear &that);
    Linear &operator *=(const Scale &that);
    Linear &operator *=(const Rotation &that);

    Linear &operator -=(const Linear &that);

    r2::Vector col(int i) const;
    r2::Vector row(int i) const;

private:
    std::array<std::array<float,2>,2> m_matrix;

    virtual void do_print(std::ostream &out) const;

    r2::Point do_multiply(const r2::Point &p) const override;
    r2::Vector do_multiply(const r2::Vector &v) const override;
    r3::Point do_multiply(const r3::Point &p) const override;
};

Linear operator*(float a, const Linear &b);

Linear operator*(const Linear &a, const Linear &b);

Linear operator*(const Rotation &a, const Scale &b);
Linear operator*(const Rotation &a, const Linear &b);

Linear operator*(const Scale &a, const Rotation &b);
Linear operator*(const Scale &a, const Linear &b);

}} // namespace rvg::xform

#include "linear.hpp"

#endif

