#ifndef QVG_XFORM_PROJECTIVE_H
#define QVG_XFORM_PROJECTIVE_H

#include <array>
#include "base.h"

namespace rvg { namespace xform
{

class Projective final : public BaseHelper<Projective>
{
public:
    static constexpr const char *name = "projective";

#if TRACE_XFORM
    Projective();
#else
    Projective() = default;
#endif

    Projective(float a11, float a12, float a13,
               float a21, float a22, float a23,
               float a31, float a32, float a33);

    // implicit conversions allowed
    Projective(const Identity &);
    Projective(const Rotation &b);
    Projective(const Scale &b);
    Projective(const Linear &b);
    Projective(const Affine &b);

    Linear linear(void) const;
    Affine affine(void) const;

    auto operator[](int i) -> std::array<float,3> &;

    auto operator[](int i) const -> const std::array<float,3> &;

    Projective &operator *=(const Identity &) { return *this; }
    Projective &operator *=(const Translation &that);
    Projective &operator *=(const Rotation &that);
    Projective &operator *=(const Scale &that);
    Projective &operator *=(const Linear &that);
    Projective &operator *=(const Affine &that);
    Projective &operator *=(const Projective &that);

private:
    std::array<std::array<float,3>,3> m_matrix;

    virtual void do_print(std::ostream &out) const;

    r2::Point do_multiply(const r2::Point &p) const override;
    r2::Vector do_multiply(const r2::Vector &v) const override;
    r3::Point do_multiply(const r3::Point &p) const override;
};

inline Projective operator*(const Projective &a, const Projective &b);

inline Projective operator*(const Rotation &a, const Projective &b);
inline Projective operator*(const Translation &a, const Projective &b);
inline Projective operator*(const Scale &a, const Projective &b);
inline Projective operator*(const Linear &a, const Projective &b);
inline Projective operator*(const Affine &a, const Projective &b);

}} // namespace rvg::xform

#include "projective.hpp"

#endif
