#ifndef QVG_XFORM_ROTATION_H
#define QVG_XFORM_ROTATION_H

#include "base.h"

namespace rvg { namespace xform
{

class Rotation final : public BaseHelper<Rotation>
{
public:
    static constexpr const char *name = "rotation";

#if TRACE_XFORM
    Rotation();
#else
    Rotation() = default;
#endif

    explicit Rotation(float ang); // explicit makes sure a scalar is not
                                  // mistakenly promoted to a rotation!

    Rotation(float c, float s);

    // implicit conversion allowed
    Rotation(const Identity &);

    float c, s;

    float ang(void) const;

    Rotation &operator *=(const Identity &) { return *this; }
    Rotation &operator *=(const Rotation &that);

private:
    virtual void do_print(std::ostream &out) const;

    r2::Point do_multiply(const r2::Point &p) const override;
    r2::Vector do_multiply(const r2::Vector &v) const override;
    r3::Point do_multiply(const r3::Point &p) const override;
};

}} // namespace rvg::xform

#include "rotation.hpp"

#endif
