#include <cmath>

namespace rvg { namespace xform
{

using std::sin;
using std::cos;
using std::atan2;

#if TRACE_XFORM
inline Rotation::Rotation()
{
    std::clog << "-- default ctor " << name << "()" << std::endl;
}
#endif

inline Rotation::Rotation(float ang)
    : c(cos(ang)), s(sin(ang))
{
#if TRACE_XFORM
    std::clog << "-- ctor " << *this << std::endl;
#endif
}

inline float Rotation::ang(void) const {
    return atan2(s, c);
}

inline Rotation::Rotation(float c, float s)
    : c(c), s(s)
{
#if TRACE_XFORM
    std::clog << "-- copy ctor (c, s) " << *this << std::endl;
#endif
}

inline Rotation::Rotation(const Identity &)
    : c(1.f), s(0.f)
{
#if TRACE_XFORM
    std::clog << "-- copy ctor (Identity) " << *this << std::endl;
#endif
}

inline Rotation transpose(const Rotation &that) {
    return Rotation(that.c, that.s);
}

inline float det(const Rotation &) {
    return 1.f;
}

inline Rotation inv(const Rotation& r ) {
    return Rotation( r.c, -r.s );
}

inline Rotation &Rotation::operator *=(const Rotation &that)
{
    float tc = c*that.c-s*that.s;
    s = s*that.c + that.s*c;
    c = tc;
    return *this;
}

inline r2::Point Rotation::do_multiply(const r2::Point &p) const
{
    return {p.x*c - p.y*s, p.x*s + p.y*c};
}

inline r2::Vector Rotation::do_multiply(const r2::Vector &v) const
{
    return {v.x*c - v.y*s, v.x*s + v.y*c};
}

inline r3::Point Rotation::do_multiply(const r3::Point &p) const
{
    return {p.x*c - p.y*s, p.x*s + p.y*c, p.z};
}

}} // namespace rvg::xform
