namespace rvg { namespace xform
{

#if TRACE_XFORM
inline Scale::Scale()
{
    std::clog << "-- default ctor " << name << "()" << std::endl;
}
#endif

inline Scale::Scale(float _sx, float _sy)
    : sx(_sx), sy(_sy)
{
#if TRACE_XFORM
    std::clog << "-- ctor " << *this << std::endl;
#endif
}

inline Scale::Scale(const Identity &)
    : sx(1.f), sy(1.f)
{
#if TRACE_XFORM
    std::clog << "-- copy ctor (Identity) " << *this << std::endl;
#endif
}

inline Scale transpose(const Scale &a) {
    return a;
}

inline Scale inv(const Scale &a) {
    return Scale(1.f/a.sx, 1.f/a.sy);
}

inline float det(const Scale &a) {
    return a.sx*a.sy;
}

inline Scale &Scale::operator *=(float b)
{
    sx *= b;
    sy *= b;
    return *this;
}

inline Scale &Scale::operator *=(const Scale &that)
{
    sx *= that.sx;
    sy *= that.sy;
    return *this;
}

inline r2::Point Scale::do_multiply(const r2::Point &p) const
{
    return {p.x*sx, p.y*sy};
}

inline r2::Vector Scale::do_multiply(const r2::Vector &v) const
{
    return {v.x*sx, v.y*sy};
}

inline r3::Point Scale::do_multiply(const r3::Point &p) const
{
    return {p.x*sx, p.y*sy, p.z};
}

inline Scale operator*(float a, const Scale &b) {
    return Scale(a*b.sx, a*b.sy);
}

}} // namespace rvg::xform
