#include <iostream>
#include "xform.h"

namespace rvg { namespace xform
{

void Transform::translate_by(float dx, float dy)
{
    *this *= Translation(dx,dy);
}

void Transform::rotate_by(float ang)
{
    *this *= Rotation(ang);
}

void Transform::scale_by(float sx, float sy)
{
    *this *= Scale(sx,sy);
}

Transform &Transform::operator*=(const Transform &xform)
{
    if(xform.m_impl)
    {
        if(m_impl)
            // virtual call
            m_impl.rw()->compose_into(*this, *xform.m_impl);
        else
            m_impl.reset(xform.m_impl->clone());
    }
    return *this;
}

Transform &Transform::operator*=(Transform &&xform)
{
    if(xform.m_impl)
    {
        if(m_impl)
            // virtual call
            m_impl.rw()->compose_into(*this, *xform.m_impl);
        else
            m_impl = std::move(xform.m_impl);
    }
    return *this;
}

r2::Point Transform::operator*(const r2::Point &p) const
{
    if(m_impl)
        return *m_impl * p;
    else
        return p; // identity
}

r2::Vector Transform::operator*(const r2::Vector &v) const
{
    if(m_impl)
        return *m_impl * v;
    else
        return v; // identity
}

void Transform::accept(Visitor &v) const {
    if (m_impl) {
        m_impl->accept(v); 
    } else {
        v.visit(xform::Identity{});
    }
}

r3::Point Transform::operator*(const r3::Point &p) const
{
    if(m_impl)
        return *m_impl * p;
    else
        return p; // identity
}

Transform operator*(const Base &lhs, const Transform &rhs)
{
    if(rhs.m_impl)
    {
        Transform xf;
        lhs.compose_into(xf, *rhs.m_impl);
        return std::move(xf);
    }
    else
        return lhs;
}

Transform &&operator*(const Base &lhs, Transform &&rhs)
{
    if(rhs.m_impl)
        lhs.compose_into(rhs, *rhs.m_impl.rw());

    return std::move(rhs);
}

Transform operator*(Transform lhs, const Transform &rhs)
{
    return std::move(lhs *= rhs);
}

Transform operator*(Transform lhs, Transform &&rhs)
{
    return std::move(lhs *= std::move(rhs));
}

std::ostream &operator<<(std::ostream &out, const Transform &xf)
{
    if(xf.m_impl)
        return out << "dyn_" << *xf.m_impl;
    else
        return out << "dyn_identity"; // Identity();
}

Projective Transform::projective() const
{
    if(auto *proj = dynamic_cast<const Projective *>(m_impl.ro()))
       return *proj;
    else
        return (Projective(Identity()) * *this).projective();
}

}} // namespace rvg::xform

