#ifndef QVG_XFORM_TRANSFORM_H
#define QVG_XFORM_TRANSFORM_H

#include "../util/cow.h"
#include "base.h"
#include "identity.h"

namespace rvg { namespace xform
{

class Transform final
{
public:
    Transform() = default;

    explicit Transform(Base *base)
        : m_impl(base) {}

    Transform(const Base &base)
        : m_impl(base.clone()) {
#if TRACE_XFORM
        std::clog << "-- transform copy initialization" << std::endl;
#endif
    }

    void translate_by(float dx, float dy);
    void rotate_by(float rad);
    void scale_by(float sx, float sy);

    Transform &operator*=(const Transform &xform);
    Transform &operator*=(Transform &&xform);

    template <class T, class EN =
        typename std::enable_if<
            std::is_base_of<Base,T>::value
        >::type>
    Transform &operator*=(const T &xform)
    {
        if(m_impl)
            // virtual call
            m_impl.rw()->compose_into(*this, xform);
        else
            m_impl.reset(xform.clone());
        return *this;
    }

    r2::Point operator*(const r2::Point &p) const;
    r2::Vector operator*(const r2::Vector &v) const;
    r3::Point operator*(const r3::Point &p) const;

    Projective projective() const;

    bool is_identity() const
    {
        return !m_impl || dynamic_cast<const Identity *>(m_impl.ro());
    }

    friend Transform operator*(Transform lhs, const Transform &rhs);
    friend Transform operator*(Transform lhs, Transform &&rhs);

    friend Transform operator*(const Base &lhs, const Transform &rhs);
    friend Transform &&operator*(const Base &lhs, Transform &&rhs);

    friend std::ostream &operator<<(std::ostream &out, const Transform &xf);

    void accept(Visitor &v) const;

private:
    // Base::DoComposition need access to m_impl
    friend class Base;

    COW<Base> m_impl; // == NULL ? is identity
};

} // namespace rvg::xform

using xform::Transform;

} // namespace rvg


#endif
