#ifndef RVG_TRANSFORMABLE_H
#define RVG_TRANSFORMABLE_H

#include "transform.h"
#include "../util/cow.h"

namespace rvg {

template <class BASE>
class Transformable {
    class TransformClone
    {
    public:
        TransformClone() = default;
        TransformClone(const Transform &xf) : m_xf(xf) {}
        RefPtr<BASE> operator()(const BASE &obj) const
        {
            return obj.xform_clone(m_xf);
        }

        Transform &xf() { return m_xf; }
        const Transform &xf() const { return m_xf; }
        void set_xf(Transform xf) { m_xf = std::move(xf); }

    private:
        Transform m_xf;
    };

    typedef COW<BASE, TransformClone> cow_type;

public:
    explicit Transformable(BASE *base,
        const Transform &xf = {})
        : m_cow(base, TransformClone(xf)) {}

    explicit Transformable(const BASE &base,
        const Transform &xf = {})
        : m_cow(base.clone(), TransformClone(xf)) {}

    const Transform &xf(void) const { return m_cow.cloner().xf(); }

    void set_xf(Transform xf = {}) { m_cow.cloner().set_xf(std::move(xf)); }

    template <class XF, class EN = typename std::enable_if<
            is_xform<XF>::value
        >::type>
    void transform(const XF &xf) {
        m_cow.cloner().set_xf(xf * m_cow.cloner().xf());
    }

    void reset(RefPtr<BASE> obj = {}) {
        m_cow.reset(obj);
    }

    const BASE *ro() const { return m_cow.ro(); }

    BASE *rw() {
        if(ro()->ref_count() >= 2)
            m_cow.rw(); // will invoke clone_xform for us
        else
            m_cow.rw()->transform(xf());
        m_cow.cloner().set_xf({});
        return m_cow.rw();
    }

private:

    cow_type m_cow;
};

} // namespace rvg

#endif // RVG_TRANSFORMABLE_H
