namespace rvg { namespace xform
{

#if TRACE_XFORM
inline Translation::Translation()
{
    std::clog << "-- default ctor " << name << "()" << std::endl;
}
#endif

inline Translation::Translation(float _dx, float _dy)
    : dx(_dx), dy(_dy)
{
#if TRACE_XFORM
    std::clog << "-- ctor " << *this << std::endl;
#endif
}

inline Translation::Translation(const r2::Vector _v)
    : dx(_v.x), dy(_v.y)
{
#if TRACE_XFORM
    std::clog << "-- ctor " << *this << std::endl;
#endif
}

inline Translation::Translation(const Identity &)
    : dx(0.f), dy(0.f)
{
#if TRACE_XFORM
    std::clog << "-- copy ctor Identity " << *this << std::endl;
#endif
}

inline Translation &Translation::operator *=(const Translation &that)
{
    dx += that.dx;
    dy += that.dy;
    return *this;
}

inline r2::Point Translation::do_multiply(const r2::Point &p) const
{
    return {p.x+dx, p.y+dy};
}

inline r2::Vector Translation::do_multiply(const r2::Vector &v) const
{
    return v;
}

inline r3::Point Translation::do_multiply(const r3::Point &p) const
{
    return {p.x+dx*p.z, p.y+dy*p.z, p.z};
}

}} // namespace rvg::xform
