#ifndef QVG_XFORM_VISITOR_H
#define QVG_XFORM_VISITOR_H

#include "fwd.h"

namespace rvg { namespace xform
{

class Visitor
{
public:
    // virtual template method pattern
    template <class T>
    void visit(const T &xf)
    {
        do_visit(xf);
    }

private:
    virtual void do_visit(const Identity &xf) = 0;
    virtual void do_visit(const Translation &xf) = 0;
    virtual void do_visit(const Rotation &xf) = 0;
    virtual void do_visit(const Scale &xf) = 0;
    virtual void do_visit(const Linear &xf) = 0;
    virtual void do_visit(const Affine &xf) = 0;
    virtual void do_visit(const Projective &xf) = 0;
};

// CRTP pattern
template <class T>
class VisitorHelper : public Visitor
{
private:
    T &derived() { return static_cast<T &>(*this); }
    const T &derived() const { return static_cast<const T &>(*this); }

    void do_visit(const Identity &xf) final
        { derived().visit(xf); }
    void do_visit(const Translation &xf) final
        { derived().visit(xf); }
    void do_visit(const Rotation &xf) final
        { derived().visit(xf); }
    void do_visit(const Scale &xf) final
        { derived().visit(xf); }
    void do_visit(const Linear &xf) final
        { derived().visit(xf); }
    void do_visit(const Affine &xf) final
        { derived().visit(xf); }
    void do_visit(const Projective &xf) final
        { derived().visit(xf); }
};

class Visitable
{
public:
    // won't use NVI pattern because gcc can't devirtualize 
    // it when it should
    virtual void accept(Visitor &v) const = 0;
    virtual void accept(const Visitor &v) const = 0;
};

}}

#endif

