#include <sstream>
#include "visitor.h"
#include "transform.h"
#include "identity.h"
#include "translation.h"
#include "scale.h"
#include "rotation.h"
#include "linear.h"
#include "affine.h"
#include "projective.h"

namespace rvg { namespace xform
{

// Auxiliar functions ----------------------------------------------------------

namespace
{
    template <class T>
    void print_matrix(std::ostream &out, const T &mat)
    {
        for(size_t i=0; i<mat.size(); ++i)
        {
            if(i > 0)
                out << ',';
            out << '[';
            for(size_t j=0; j<mat[i].size(); ++j)
            {
                if(j > 0)
                    out << ',';
                out << mat[i][j];
            }
            out << ']';
        }
    }
}

// Base ------------------------------------------------------------------------

std::ostream &operator<<(std::ostream &out, const Base &xf)
{
    xf.do_print(out);
    return out;
}

// Identity --------------------------------------------------------------------

template class BaseHelper<Identity>;

void Identity::do_print(std::ostream &out) const
{
    out << name;
}

// Translation -----------------------------------------------------------------
//
template class BaseHelper<Translation>;

void Translation::do_print(std::ostream &out) const
{
    out << name << '(' << dx << ',' << dy << ')';
}

// Rotation --------------------------------------------------------------------

template class BaseHelper<Rotation>;

void Rotation::do_print(std::ostream &out) const
{
    out << name << '(' << ang() << ')';
}

// Scale -----------------------------------------------------------------------

template class BaseHelper<Scale>;

void Scale::do_print(std::ostream &out) const
{
    out << name << '(' << sx << ',' << sy << ')';
}

// Linear ----------------------------------------------------------------------

template class BaseHelper<Linear>;

void Linear::do_print(std::ostream &out) const
{
    out << name << '(';
    print_matrix(out,m_matrix);
    out << ')';
}

// Affine ----------------------------------------------------------------------

template class BaseHelper<Affine>;

void Affine::do_print(std::ostream &out) const
{
    out << name << '(';
    print_matrix(out,m_matrix);
    out << ')';
}

// Projective ------------------------------------------------------------------

template class BaseHelper<Projective>;

void Projective::do_print(std::ostream &out) const
{
    out << name << '(';
    print_matrix(out,m_matrix);
    out << ')';
}

}} // namespace rvg::xform

