local drv = require"rvg10.cuda-master"
setmetatable(_G, {__index = drv})
--local fd = dofile(arg[1])
--
local fd;
local config = {}
local fout = "output.png"
local width = 1024
local xform=identity()
local fs_w = 0
config["nsamples"] = 32
config["kernel"] = 42
config["bs3"] = 0

function show_help()
    local str = [[
        -i:<file_name>      -- input file, se exemples/*.rvg
        -o:<output_name>    -- output image (PNG)
        -width:<n>          -- set the output width to <n>, scales height uniformly
        -nsamples:<n>       -- set the number of samples: 4, 8, 16, or 32(default)
        -cbs3               -- use Cardinal B-Spline3 filter
        -warp:<what>        -- apply warp
            what:
                foreshorten:<w> -- foreshorten with with <w>, ex: -warp:foreshorten:128
                swirl:<i>       -- swirl with intensity <i>, i between 0 and 1.
                swirl:<i>:<a>:<s> -- swirl with intensity <i>, starting angle <a> and scale <s>
                lens:(cu, cv, ir, or, z)    -- lens with center (cu, cv) in [-1,1]^2
                                            -- inner radius <ir>, outer radius <or>
                                            -- zoom <z>, ex: -warp:lens:\(0,0,0.6,0.5,3\)
]]
    print(str)
    os.exit(1)
end

func = {
    i = function( fname ) fd = dofile(fname) end,
    o = function( fname ) fout = fname end,
    dump = function( val )
        if val == "qtree" then
            config["dump"] = 1;
        elseif val == "packet" then
            config["dump"] = 2;
        elseif val == "num_segs" then
            config["dump"] = 4;
        elseif val == "data_size" then
            config["dump"] = 8;
        elseif val == "data_area" then
            config["dump"] = 16;
        elseif val == "all" then
            config["dump"] = 3;
        end
    end,
    kernel = function( k )
        config["kernel"] = k;
    end,
    min_segs_split = function( k )
        config["min_segs_split"] = k;
    end,
    qtree_max_level = function( k )
        config["qtree_max_level"] = k;
    end,
    help = function( k )
        show_help()
    end,
    h = function( k )
        show_help()
    end,
    width = function( k )
        width = tonumber( k )
    end,
    bs3 = function( k )
        config["bs3"] = 1;
    end,
    cbs3 = function( k )
        config["bs3"] = 1;
    end,
    nsamples = function( k )
        config["nsamples"] = tonumber(k);
    end,
    warp = function( k )
        config["kernel"] = 43
        if k:find("foreshorten:") == 1 then
            local f = k:match("foreshorten:(%d+)")
            config["warp"] = 1
            config["foreshorten"] = -tonumber(f)
            fs_w = -tonumber(f)
        elseif k:find("lens:") == 1 then
            local cu, cv, outer, inner, zoom = k:match("lens:%((-?%d+.?%d*),%s*(-?%d+.?%d*),%s*(-?%d+.?%d*),%s*(-?%d+.?%d*),%s*(-?%d+.?%d*)%)")
            if cu == nil or cv == nil or outer == nil or inner == nil or zoom == nil then
                print("Lens: parameter error (cu, cv, outer, inner, zoom):", cu or "?", cv or "?" , outer or "?", inner or "?", zoom or "?")
                os.exit(1)
            end
                print("Lens: parameter error (cu, cv, outer, inner, zoom):", cu or "?", cv or "?" , outer or "?", inner or "?", zoom or "?")
            --print( "lens", f )
            config["warp"] = 2
            config["lens_cu"] = tonumber(cu)
            config["lens_cv"] = tonumber(cv)
            config["lens_outer"] = tonumber(outer)
            config["lens_inner"] = tonumber(inner)
            config["lens_zoom"] = tonumber(zoom)
        elseif k:find("swirl:") == 1 then
            local f, a, s = k:match("swirl:(-?%d+.?%d*):(-?%d+.?%d*):(-?%d+.?%d*)")
            if f and a and s then
                config["warp"] = 3
                config["swirl"] = tonumber(f)
                config["swirl_ang"] = tonumber(a)
                config["swirl_scale"] = tonumber(s)
                print("swirl", f, a, s)
            else
                local f = k:match("swirl:(-?%d+.?%d*)")
                if f then
                    config["warp"] = 3
                    config["swirl"] = tonumber(f)
                    config["swirl_ang"] = 0
                    config["swirl_scale"] = 1
                    print("swirl", f)
                end
            end
        end
    end
}
for i=1,#arg do
    local v = arg[i]
    local name, value = v:match("-([_%w]+):*([_%a]*.*%d*)")
    if not func[name] then
        print(string.format("Unknown parameter: %s", v))
        show_help()
    end
    func[name]( value )
end

local vp_ = fd.viewport(drv)
local dim = { w=(vp_.max.x - vp_.min.x), h=(vp_.max.y - vp_.min.y) }
local vp = drv.viewport(0, 0, width, width*dim.h/dim.w);
local win = drv.window(0, 0, width, width*dim.h/dim.w);
local scene, paths = fd.scene(drv)
local nscene = scene:translate(-vp.min.x,-vp.min.y):scale(width/dim.w, width/dim.w)
if config.bs3 == 1 then
    nscene = nscene:translate(-0.5,-0.5)
end
if fs_w ~= 0 then
    xform  = foreshorten(vp, fs_w)
end

function print_result( result, level, pre )
    print(string.format("%s [%d]%s %s", pre, result.level, result.name, result.time) )
    for k,v in pairs(result.children) do
        print_result( v, level+1, pre.."  " )
    end
end


reset_timer()
qtree = shortcut(nscene, win, vp, config);
local timer = get_timer_results()
print_result( timer, 0, " " )

reset_timer()
render(qtree, xform, fout, vp, config)
timer = get_timer_results()
print_result( timer, 0, " " )

