local config = { }
config["kernel"] = 42
config["min_segs_split"] = 4
config["qtree_max_level"] = 12
config["bs3"] = 1
config["nsamples"] = 32
config["is_fs"] = false
config["fd_w"] = 0
config["purge"] = false
config["width"] = 1024
config["iterations"] = 10

func = {
    kernel = function( k )
        config["kernel"] = k;
    end,
    min_segs_split = function( k )
        config["min_segs_split"] = k;
    end,
    qtree_max_level = function( k )
        config["qtree_max_level"] = k;
    end,
    help = function( k )
        print("options:");
        for k,v in pairs(func) do
            print( string.format("\t-%s:<args>", k) )
        end
    end,
    purge = function( k )
        config["purge"] = true
    end,
    width = function( k )
        config["width"] = tonumber( k )
    end,
    iterations = function( k )
        config["iterations"] = tonumber( k )
    end,
    bs3 = function( k )
        if not k then
            config["bs3"] = 1
        else
            config["bs3"] = tonumber(k);
        end
    end,
    nsamples = function( k )
        config["nsamples"] = tonumber(k);
    end,
    warp = function( k )
        if k:find("foreshorten:") == 1 then
            local f = k:match("foreshorten:(%d+.*%d*)")
            config["warp"] = 1
            config["foreshorten"] = -tonumber(f)
            config["fs_w"] = -tonumber(f)
            config["is_fs"] = true
        elseif k:find("lens:") == 1 then
            local cu, cv, outer, inner, zoom = k:match("lens:%((-?%d+.?%d*),%s*(-?%d+.?%d*),%s*(-?%d+.?%d*),%s*(-?%d+.?%d*),%s*(-?%d+.?%d*)%)")
            if cu == nil or cv == nil or outer == nil or inner == nil or zoom == nil then
                print("Lens: parameter error (cu, cv, outer, inner, zoom):", cu or "?", cv or "?" , outer or "?", inner or "?", zoom or "?")
                os.exit(1)
            end
            config["warp"] = 2
            config["lens_cu"] = tonumber(cu)
            config["lens_cv"] = tonumber(cv)
            config["lens_outer"] = tonumber(outer)
            config["lens_inner"] = tonumber(inner)
            config["lens_zoom"] = tonumber(zoom)
        elseif k:find("swirl:") == 1 then
            local f, a, s = k:match("swirl:(-?%d+.?%d*):(-?%d+.?%d*):(-?%d+.?%d*)")
            if f and a and s then
                config["warp"] = 3
                config["swirl"] = tonumber(f)
                config["swirl_ang"] = tonumber(a)
                config["swirl_scale"] = tonumber(s)
            else
                local f = k:match("swirl:(-?%d+.?%d*)")
                if f then
                    config["warp"] = 3
                    config["swirl"] = tonumber(f)
                    config["swirl_ang"] = 0
                    config["swirl_scale"] = 1
                end
            end
        end
    end
}

local arg = { }

arg.parse = function( args )
    for i=1,#args do
        local v = args[i]
        local name, value = v:match("-([_%w]+):*([_%a]*.*%d*)")
        --print( name, value )
        if not func[name] then
            print(string.format("Unknown parameter: %s", v))
            os.exit(1)
        end
        func[name]( value )
    end
    return config
end

return arg
