local commits = require "lua.commits"
local inputs = require "lua.inputs"
local project = require "lua.project"
local run = require "lua.run"
local data = require "lua.data"
local cmd = require "lua.command"

local bench = {}

function unrequire(m)
    package.loaded[m] = nil
    _G[m] = nil
    collectgarbage()
end

bench.run = function( drv, name, inp, conf, pre, imgpath )
    for k, v in pairs( inp ) do
        print(string.format("\t%s", v.file))
        local ret, msg, figure = inputs.open( { v.dir, v.file }, drv )
        if v.width then
            local s, w, v = inputs.resize( drv, v.width, figure.scene, figure.win, figure.vp )
            if conf.bs3 > 0 then
                figure.scene = s:translate(-0.5,-0.5)
            else
                figure.scene = s
            end
            figure.win = w
            figure.vp = v
        end
        local fnamestr = string.format( "%s/%s.png", imgpath, v.file )
        local ret, msg, benchmark = run.run( drv, figure, conf, conf["iterations"], conf["is_fs"], conf["fs_w"], fnamestr )
        data.store( name, "master", v.file, benchmark )
        --data.dump( benchmark )
    end
end

bench.run2 = function( name, co, inp, conf, pre, num )
    if not pre then pre = "" end
    for k, hash in pairs( co ) do
        if string.len(hash) > 6 then
            hash = string.sub(hash, 1, 6)
        end
        -- test if commit is locked
        local ret, val = cmd.execute(string.format("cat data/%s/%s/lock 2>&1", name, hash))
        if conf["purge"] == true or not ret or hash == "master" then
            ---[[
            print("git checkout commit "..hash)
            assert( commits.checkout( hash ) )
            print("compiling... ")
            assert( project.recompile() )
            print("copying modules... ")
            assert( project.copy_modules( pre..hash ) )
            --]]
            print("loading cuda module.")
            local drvname = string.format("rvg10.cuda-%s", pre..hash)
            local drv = require(drvname)
            print("aqui", drv)
            --
            for k, v in pairs( inp ) do
                print(string.format("\t%s", v.file))
                local ret, msg, figure = inputs.open( { v.dir, v.file }, drv )
                if v.width then
                    local s, w, v = inputs.resize( drv, v.width, figure.scene, figure.win, figure.vp )
                    figure.scene = s
                    figure.win = w
                    figure.vp = v
                end
                --local ret, msg, benchmark = run.run( drv, figure, conf, conf["iterations"], conf["is_fs"], conf["fs_w"], string.format("img/%s.png", v.file) )
                local ret, msg, benchmark = run.run( drv, figure, conf, conf["iterations"], conf["is_fs"], conf["fs_w"], "" )
                data.store( name, hash, v.file.."-"..num, benchmark )
                data.dump( benchmark )
            end
            cmd.execute(string.format("echo true > data/%s/%s/lock", name, hash))
        else
            -- marking commit as tested
            print( "skipping commit %s", hash )
        end
    end
    commits.checkout( "master" )
end

return bench
