local cmd = require "lua.command"
local color = require "lua.color"

local report = { }

--[[
report.load_data = function( this )
    local str = "find data -name \"*.lua\""
    local rcode, out = cmd.execute( str )
    this.results = { }
    if rcode then
        for l in string.gmatch( out, "data/%w*/[%w-_]*.rvg.lua" ) do
            local co, name = string.match( l, "data/(%w*)/([%w-_]*).rvg.lua" )
            if not this.results[name] then
                this.results[name] = {}
            end
            local t = this.results[name]
            t[co] = dofile(l)
        end
    end
end
--]]
--
report.load_data = function( test )
    local str = string.format("find data/%s -name \"*.lua\"", test)
    local rcode, out = cmd.execute( str )
    local results = {}
    if rcode then
        for l in string.gmatch( out, "data/[%w_-]*/%w*/[%w-_]*.rvg%-*%d*.lua" ) do
            local co, name, num = string.match( l, "data/[%w_-]*/(%w*)/([%w-_]*).rvg%-*(%d*).lua" )
            local nn = tonumber(num)
            --if num and num:len() > 0 then name = name.."-"..num end
            --print( string.format("%s %s '%s'", co, name, num ) )
            if not results[name] then
                results[name] = {}
            end
            if nn then
                if not results[name][num] then
                    results[name][num] = { }
                end
            end
            local t
            if nn then
                t = results[name][num]
            else
                t = results[name]
            end
            --print( l, name, num, nn, t )
            t[co] = dofile(l)
        end
    end
    return results
end

report.dump_data = function( this )
    print("dump")
    for k, v in pairs( this.results ) do
        io.write(k.."{ ")
        for c, _ in pairs( v ) do
            io.write(c..", ")
        end
        io.write("}\n")
    end
end

local function sample_data( data )
    -- find the first tree in the data set
    -- assuming that they all have the same
    -- structure
    local name, co
    for n,v in pairs(data) do
        name = n
        for c,_ in pairs(v) do
            co = c
            print(name, co)
            break
        end
        break
    end
    return data[name][co]
end

report.parse_labels = function( this, labels )
    local l = labels
    local paths = { }
    if type(labels) == "string" then
        l = { labels }
    end
    local data = { sample_data( this.results ) }
    for _, str in pairs( l ) do
        local p = { }
        for i in str:gmatch("[%d%*]+") do
            table.insert( p, i )
        end
        print(#p)
    end
end

function create_plot_file( test, name, label, children, size )
    local dir = string.format("output/%s", test)
    cmd.dir_exists( dir, true )
    local fd = assert(io.open(string.format("%s/%s.gnuplot", dir, name), "w"))
    local tpl = assert(io.open("plot.tpl", "r"))
    if tpl then
        for line in tpl:lines() do
            --print(name, size, line)
            if line:find("#PLOT") then
                --fd:write(string.format("plot newhistogram \"run time(ms)\", 'output/%s.dat' u 2:xtic(1) ls 1 t '%s'", name, children[1]))
                fd:write(string.format("plot newhistogram \"run time(ms)\""))
                for i=1,size do
                    local label = ""
                    if i == 1 then label = dir.."/"..name..".dat" end
                    fd:write(string.format(", '%s' u %d:xtic(1) ls %d t '%s'", label, i+1, color:get_index(i), children[i]))
                end
                fd:write("\n")
            elseif line:find("#LABEL") then
                fd:write(string.format("set xlabel  \"%s\"\n", label))
            elseif line:find("#OUTPUT") then
                fd:write(string.format("set output  \"%s/%s.eps\"\n", dir, name))
            elseif line:find("#LSTYLE") then
                for i=1,color:size() do
                    local str = "set style line %d lt 1 lw 4 lc rgb \"%s\"\n"
                    fd:write( str:format(i, color:get_color(i)) )
                end
            else
                fd:write( line.."\n" )
            end
        end
    end
    fd:close()
    tpl:close()
end

report.create_report = function( test, hashs, results, path )
    local dir = string.format("output/%s", test)
    cmd.dir_exists(dir, true)
    for name, v in pairs(results) do
        --create data file
        local fd = assert(io.open(string.format("%s/%s.dat", dir, name), "w"))
        local psize = math.huge
        local label = ""
        local children = {}
        for _, co in ipairs( hashs ) do
            co = co:sub(1,6)
            local l = v[co]
            --print(name, co, l.name)
            local nodes = { l }
            local node = { }
            for i in string.gmatch(path, "%d+") do
                node = nodes[tonumber(i)]
                nodes = node.children
            end
            fd:write( string.format("%s-%s", name, co) )
            local i=1
            for k, v in pairs( nodes ) do
                fd:write( string.format(" %f", v.time) )
                children[i] = v.name
                i = i+1
            end
            psize = math.min(psize, #nodes)
            label = node.name
            fd:write( "\n" )
        end
        fd:close()
        -- create plot file
        create_plot_file( test, name, label, children, psize )
        local gnuplot = "gnuplot output/%s/%s.gnuplot"
        cmd.execute( gnuplot:format( test, name ) )
    end
end

report.create_report_multiple = function( test, hashs, results, path, fname )
    local dir = string.format("output/%s", test)
    cmd.dir_exists(dir, true)
    local fd = assert(io.open(string.format("%s/%s.dat", dir, fname), "w"))
    local psize = math.huge
    local label = ""
    for name, v in pairs(results) do
        --create data file
        label = ""
        children = {}
        psize = math.huge
        for _, co in ipairs( hashs ) do
            co = co:sub(1,6)
            local l = v[co]
            --print(name, co, l.name)
            local nodes = { l }
            local node = { }
            for i in string.gmatch(path, "%d+") do
                node = nodes[tonumber(i)]
                nodes = node.children
            end
            fd:write( string.format("%s-%s", name, co) )
            local i=1
            for k, v in pairs( nodes ) do
                fd:write( string.format(" %f", v.time) )
                children[i] = v.name
                i = i+1
            end
            psize = math.min(psize, #nodes)
            label = node.name
            fd:write( "\n" )
        end
    end
    fd:close()
    -- create plot file
    create_plot_file( test, fname, label, children, psize )
    local gnuplot = "gnuplot output/%s/%s.gnuplot"
    cmd.execute( gnuplot:format( test, fname ) )
end

return report
