local benchmark = require "lua.benchmark"
local args = require "lua.args"
local params = require "lua.resolution.inputs"
local commits = require "lua.commits"
local inputs = require "lua.inputs"
local project = require "lua.project"
local run = require "lua.run"
local data = require "lua.data"

local argv = {
    "-kernel:42",
    "-bs3:1",
    "-iterations:20",
}
-- default configuration
local config = args.parse( argv )
local min=256
local max=2048
local num=10
local size = {}
for i=0,num do
    local width = min + i * (max-min)/num
    size[#size+1] = math.floor(width*width)
end
--
local hash = "master"
--[[
print("git checkout commit "..hash)
assert( commits.checkout( hash ) )
print("compiling... ")
assert( project.recompile() )
--]]
print("copying modules... ")
assert( project.copy_modules( "master" ) )
print("loading cuda module.")
local drvname = string.format("rvg10.cuda-%s", "master")
local drv = require(drvname)
--
for k, inp in pairs( params.inputs ) do
    ---[[
    local ret, msg, figure = inputs.open( { inp.dir, inp.file }, drv )
    local vw = figure.vp.max.x - figure.vp.min.x
    local vh = figure.vp.max.y - figure.vp.min.y
    local p = vh/vw
    for _, ww in pairs(size) do
        local width = math.floor(math.sqrt( ww/p )+0.5)
        local s, w, v = inputs.resize( drv, width, figure.scene, figure.win, figure.vp )
        figure.scene = s
        figure.win = w
        figure.vp = v
        ---[[
        local ret, msg, benchmark = run.run2( drv, figure, config, config["iterations"], config["is_fs"], config["fs_w"], "" )
        data.store( "resolution_4x4x32", hash, inp.file.."-"..ww, benchmark )
        --data.dump( benchmark )
        print(string.format("\t%s at: %dx%d -> %.02f", inp.file, width, v.max.y, benchmark.children[2].children[1].time*1000))
        --]]
    end
    --]]
end

for k, inp in pairs( params.warps ) do
    ---[[
    local ret, msg, figure = inputs.open( { inp.dir, inp.file }, drv )
    local vw = figure.vp.max.x - figure.vp.min.x
    local vh = figure.vp.max.y - figure.vp.min.y
    local p = vh/vw
    local argv = {
        "-kernel:43",
        "-bs3:1",
        "-iterations:20",
        inp.srt
    }
    config = {}
    config = args.parse( argv )
    for _, ww in pairs(size) do
        local width = math.floor(math.sqrt( ww/p )+0.5)
        local s, w, v = inputs.resize( drv, width, figure.scene, figure.win, figure.vp )
        figure.scene = s
        figure.win = w
        figure.vp = v
        ---[[
        local ret, msg, benchmark = run.run2( drv, figure, config, config["iterations"], config["is_fs"], config["fs_w"], "" )
        data.store( "resolution_4x4x32", hash, inp.file.."-"..ww, benchmark )
        --data.dump( benchmark )
        print(string.format("\t%s at: %dx%d -> %.02f", inp.file, width, v.max.y, benchmark.children[2].children[1].time*1000))
        --]]
    end
    --]]
end
