local table = {}

function merge_level( base, other, n )
    base.name = other.name
    base.level = other.level
    if not base.time then
        base.time = 0
    end
    base.time = base.time + other.time/n
    if not base.children then
       base.children = { }
    end
    for k=1,#other.children do
        if not base.children[k] then
            base.children[k] = { time = 0 } 
        end
        merge_level( base.children[k], other.children[k], n )
    end
end

function merge_times( times )
    local time = {}
    for i=1,#times do
        merge_level( time, times[i], #times )
    end
    return time
end

function merge_qtree_levels( times )
    local time = {}
    for i=1,#times do
        merge_level( time, times[i], 1 )
    end
    return time
end

table.run = function( drv, figure, config, iterations, is_fs, fs_w, fname )
    local qtree = nil;
    local qtree_times = {}
    for i=1,iterations do
        if qtree ~= nil then
            drv.release_shortcut( qtree )
        end
        drv.reset_timer()
        qtree = drv.shortcut( figure.scene, figure.win, figure.vp, config);
        qtree_times[#qtree_times+1] = drv.get_timer_results()
    end
    local qtime = merge_times( qtree_times )
    --print("--------------------", qtime.children[1].children[3].children[1].name)
    local qtree_levels = merge_qtree_levels( qtime.children[1].children[3].children[1].children )
    qtime.children[1].children[3].children[1].children = { qtree_levels }
    local xform = drv.identity()
    if is_fs then
        local w = figure.vp.max.x - figure.vp.min.x
        xform = drv.foreshorten(figure.vp, fs_w*w)
    end
    local render_times = {}
    for i=1,iterations do
        drv.reset_timer()
        local bench = drv.render(qtree, xform, "", figure.vp, config)
        render_times[#render_times+1] = drv.get_timer_results()
    end
    if string.len(fname) > 0 then
        drv.render(qtree, xform, fname, figure.vp, config)
    end
    local rtime = merge_times( render_times )
    --local qtree_levels = merge_qtree_levels( rtime.children
    -- put together both times
    local time = {}
    time.name = "total"
    time.level = 0
    time.time = qtime.children[1].time + rtime.children[1].time
    time.children = {}
    time.children[1] = qtime.children[1]
    time.children[2] = rtime.children[1]
    --
    return true, "", time
end

table.run2 = function( drv, figure, config, iterations, is_fs, fs_w, fname )
    local qtree = nil;
    local qtree_times = {}
    --for i=1,iterations do
    for i=1,1 do
        if qtree ~= nil then
            drv.release_shortcut( qtree )
        end
        drv.reset_timer()
        qtree = drv.shortcut( figure.scene, figure.win, figure.vp, config);
        qtree_times[#qtree_times+1] = drv.get_timer_results()
    end
    local qtime = merge_times( qtree_times )
    --print("--------------------", qtime.children[1].children[3].children[1].name)
    local qtree_levels = merge_qtree_levels( qtime.children[1].children[3].children[1].children )
    qtime.children[1].children[3].children[1].children = { qtree_levels }
    local xform = drv.identity()
    if is_fs then
        local w = figure.vp.max.x - figure.vp.min.x
        xform = drv.foreshorten(figure.vp, fs_w*w)
    end
    local render_times = {}
    for i=1,iterations do
        drv.reset_timer()
        local bench = drv.render(qtree, xform, fname, figure.vp, config)
        render_times[#render_times+1] = drv.get_timer_results()
    end
    local rtime = merge_times( render_times )
    --local qtree_levels = merge_qtree_levels( rtime.children
    -- put together both times
    local time = {}
    time.name = "total"
    time.level = 0
    time.time = qtime.children[1].time + rtime.children[1].time
    time.children = {}
    time.children[1] = qtime.children[1]
    time.children[2] = rtime.children[1]
    --
    drv.release_shortcut( qtree )
    drv.reset_timer()
    --
    return true, "", time
end

return table
